@@ -28,6 +28,17 @@
 </ul>
 </p>
 
+<p>Mac OSX systems:
+<ul>
+	<li><b>gettext</b> - the default gettext autoconf scripts do not
+		always do a very good job of detecting gettext and libintl
+		libraries on the Mac.  If it cannot find libintl, it will
+		automatically disable NLS support.  If this is important
+		to you, add --with-libintl-prefix=/opt/local/ to the
+		configure command line when building from source.</li>
+</ul>
+</p>
+
 <? createSubHeader("Master List of Dependencies"); ?>
 
 <p>The following list contains all software that Barry depends on, and
@@ -40,10 +51,12 @@ some are only needed for building CVS.
 	<li><b>pkg-config</b> (source build: so configure can autodetect library locations) </li>
 	<li><b>libusb, stable (0.1.x)</b> - found at <a href="http://www.libusb.org/">http://www.libusb.org/</a> </li>
 	<li><b>pthread</b>
-	<li><b>boost</b> version 1.33 or higher (optional, needed for the serialization library, which you need if you want to save downloads for later uploads to the device, using btool) <a href="http://www.boost.org/">http://www.boost.org</a></li>
+	<li><b>boost</b> version 1.33 or higher (optional, Boost serialization support.) <a href="http://www.boost.org/">http://www.boost.org</a></li>
 	<li><b>automake</b> version 1.9 (CVS builds only) </li>
 	<li><b>autoconf</b> version 2.61 (CVS builds only) </li>
 	<li><b>libtool</b> version 1.5.22 (CVS builds only) </li>
+	<li><b>autopoint</b> on some systems, this is a separate package, yet
+		on others, it is part of gettext (CVS builds only) </li>
 	<li><b>doxygen</b> suggested version 1.5.6, only for building API documentation</li>
 	<li><b>gtkmm, glademm, glibmm</b> C++ versions of the GTK libraries (needed for the barrybackup GUI)</li>
 	<li><b>libtar</b> (barrybackup GUI) </li>
@@ -55,9 +68,15 @@ some are only needed for building CVS.
 	<li><b>libiconv</b>, needed for international charset conversions... most
 		Linux distros have this as part of libc.  If you are using another
 		OS such as FreeBSD, you'll have to install this separately.</li>
+	<li><b>libxml++</b> version 2.6 for the desktop</li>
 	<li><b>gettext</b>, needed for the iconv.m4 file, on some systems,
 		when building from CVS to generate configure.</li>
 	<li><b>php5</b>, needed for generating static HTML documentation (CVS builds only) </li>
+	<li><b>rpmdevtools and rpm-build</b>, if building RPMs yourself</li>
+	<li><b>fakeroot</b>, if building DEB packages yourself</li>
+	<li><b>wxWidgets</b>, if building the Barry Desktop</li>
+	<li><b>libgcal</b>, version 0.9.6 or higher, if building the
+		Barry Desktop</li>
 </ul>
 </p>
 
@@ -72,8 +91,8 @@ Depending on your system, and how up to date it is, it may already have been
 fixed.</p>
 
 <p>The curious can read more about this bug
-<a href="http://sourceforge.net/mailarchive/message.php?msg_id=20070803200729.GA7068%40foursquare.net">here</a> and
-<a href="http://sourceforge.net/mailarchive/message.php?msg_name=20080417204336.GA15423%40foursquare.net">here</a>.</p>
+<a href="http://www.mail-archive.com/barry-devel@lists.sourceforge.net/msg00746.html">here</a> and
+<a href="http://www.mail-archive.com/barry-devel@lists.sourceforge.net/msg00754.html">here</a>.</p>
 
 <p>Of course, you probably don't want to read the intricate details of
 distro bugs.  You just want it to work!  For such systems, I usually
@@ -95,6 +114,9 @@ rpm -i ../RPMS/*/libtar*rpm
 from source, if you are using one of the below common distributions.  Other
 distributions should have similar package names.
 
+<p>You can also find scripts to automate this in the source tree under
+maintainer/depscripts.</p>
+
 <ul>
 
 <p><b>Fedora 11:</b></p>
@@ -110,17 +132,24 @@ distributions should have similar package names.
 	<li> libglademm24-devel </li>
 	<li> fuse-devel </li>
 	<li> zlib-devel </li>
+	<li> libxml++-devel </li>
 	<li> gettext-devel </li>
 	<li> libopensync-devel (0.22) </li>
 	<li> ccache (optional) </li>
 	<li> doxygen (optional) </li>
 	<li> php-cli (optional, for generating docs) </li>
+	<li> wxGTK-devel (optional, for Barry Desktop) </li>
 </ul>
 
-<p><b>Debian stable:</b></p>
+<p><b>Debian:</b></p>
 <p>Use the apt-get package manager to install the following:
 <ul>
 	<li> pkg-config </li>
+	<li> fakeroot </li>
+	<li> cdbs </li>
+	<li> autoconf </li>
+	<li> automake </li>
+	<li> autopoint (only on some newer Debian/Ubuntu systems) </li>
 	<li> libtool </li>
 	<li> libusb-dev </li>
 	<li> libboost-serialization-dev </li>
@@ -129,12 +158,14 @@ distributions should have similar package names.
 	<li> libglibmm-2.4-dev </li>
 	<li> libglademm-2.4-dev </li>
 	<li> libfuse-dev </li>
+	<li> libxml++2.6-dev </li>
 	<li> zlib1g-dev </li>
 	<li> gettext </li>
 	<li> libopensync0-dev </li>
 	<li> ccache (optional) </li>
 	<li> doxygen (optional) </li>
 	<li> php5-cli (optional, for generating docs) </li>
+	<li> libwxgtk2.8-dev and wx-common (optional, for Barry Desktop) </li>
 </ul>
 
 </ul>
