@@ -9,6 +9,18 @@ when using Barry with various versions of the BlackBerry handheld.
 It is not guaranteed to be complete, and is based on feedback from
 users such as yourself.</p>
 
+<hr/>
+
+<p><b><i>Please note:</i></b> A very large majority of BlackBerry devices
+work with Barry by default, and so a compatibility list like this is
+somewhat redundant.  Please try Barry with your device, and
+<? createLink("contact", "report"); ?>
+any issues you may have.  Please keep in mind the
+<? createLink("index", "Known Issues"); ?> on the main page when testing
+and reporting.</p>
+
+<hr/>
+
 <p>If you wish to contribute a report, please send
 it to the <? createLink("contact", "mailing list"); ?>.</p>
 
@@ -22,6 +34,7 @@ it to the <? createLink("contact", "mailing list"); ?>.</p>
 	<th>Device Charges?</th>
 	<th>Reported By</th>
 </tr>
+
 <tr>
 	<td>Storm 9530</td>
 	<td>v4.7.0.48, v4.7.0.141, v4.7.0.151 (current)</td>
@@ -30,5 +43,17 @@ it to the <? createLink("contact", "mailing list"); ?>.</p>
 	<td>All.  Sync with 0.4x: Memos, Calendar, Address Book, Tasks</td>
 	<td>Yes, bcharge</td>
 	<td><a href="http://www.mail-archive.com/barry-devel@lists.sourceforge.net/msg02403.html">Nicolas Vivien</a></td>
+</tr>
+
+<tr>
+	<td>...</td>
+	<td><i>more to come as time permits...</i></td>
+	<td>...</td>
+	<td>...</td>
+	<td>...</td>
+	<td>...</td>
+	<td>...</td>
+</tr>
+
 </table>
 
