@@ -5,7 +5,7 @@
 ///
 
 /*
-    Copyright (C) 2006-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2006-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
     root directory of this project for more details.
 */
 
-#define _XOPEN_SOURCE
+#define _XOPEN_SOURCE 500
 #include <time.h>
 #include <string.h>
 #include <barry/barry.h>
@@ -44,7 +44,7 @@ bool ReadBool(const char *prompt)
 	return data[0] == 'y' || data[0] == 'Y';
 }
 
-time_t ReadTime(const char *prompt)
+Barry::TimeT ReadTime(const char *prompt)
 {
 	string data;
 	char *unprocessed = 0;
@@ -58,7 +58,7 @@ time_t ReadTime(const char *prompt)
 		unprocessed = strptime(data.c_str(), "%Y/%m/%d %H:%M", &tm);
 	} while( !unprocessed );
 
-	return mktime(&tm);
+	return Barry::TimeT(mktime(&tm));
 }
 
 void ReadInput(Barry::Calendar &cal)
