@@ -5,7 +5,7 @@
 ///
 
 /*
-    Copyright (C) 2006-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2006-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -136,8 +136,8 @@ int main(int argc, char *argv[])
 			return 1;
 		}
 		else {
-			cout << "Using PIN: " << std::hex
-			     << probe.Get(0).m_pin << endl;
+			cout << "Using PIN: "
+			     << probe.Get(0).m_pin.Str() << endl;
 		}
 
 
