@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,14 +22,17 @@
 #include "ConfigDlg.h"
 #include "DatabaseSelectDlg.h"
 #include "util.h"
+#include "i18n.h"
 #include <barry/barry.h>
 
 ConfigDlg::ConfigDlg(const Barry::DatabaseDatabase &dbdb,
-		     const ConfigFile &config)
+		     const Barry::ConfigFile &config)
 	: m_dbdb(dbdb)
 	, m_backupList(config.GetBackupList())
 	, m_restoreList(config.GetRestoreList())
 	, m_backupPath(config.GetPath())
+	, m_promptBackupLabel(config.PromptBackupLabel())
+	, m_autoSelectAll(config.AutoSelectAll())
 {
 	Glib::RefPtr<Gnome::Glade::Xml> xml = LoadXml("ConfigDlg.glade");
 
@@ -41,7 +44,7 @@ ConfigDlg::ConfigDlg(const Barry::DatabaseDatabase &dbdb,
 	m_pDeviceNameEntry->set_text(config.GetDeviceName());
 
 	xml->get_widget("BackupPath", m_pBackupPath);
-	m_pBackupPath->set_filename(config.GetPath());
+	m_pBackupPath->set_current_folder(config.GetPath());
 
 	xml->get_widget("PromptBackupLabel", m_pPromptBackupLabelCheck);
 	m_pPromptBackupLabelCheck->set_active(config.PromptBackupLabel());
@@ -72,20 +75,21 @@ int ConfigDlg::run()
 
 void ConfigDlg::on_configure_backup()
 {
-	DatabaseSelectDlg dlg(m_dbdb, m_backupList,
-		"Select the device databases you wish to backup:");
+	DatabaseSelectDlg dlg(m_dbdb, m_backupList, m_autoSelectAll,
+		_("Select the device databases you wish to backup:"), true);
 	if( dlg.run() == Gtk::RESPONSE_OK ) {
 		m_backupList = dlg.GetSelections();
+		m_autoSelectAll = dlg.AutoSelectAll();
 	}
 }
 
 void ConfigDlg::on_configure_restore()
 {
-	DatabaseSelectDlg dlg(m_dbdb, m_restoreList,
-		"Select the device databases you wish to recover.  "
+	DatabaseSelectDlg dlg(m_dbdb, m_restoreList, m_autoSelectAll,
+		_("Select the device databases you wish to recover.  "
 		"This selection acts like a filter, in that only the databases "
 		"you select here will be restored, even if more exist in the "
-		"backup data.");
+		"backup data."), false);
 	if( dlg.run() == Gtk::RESPONSE_OK ) {
 		m_restoreList = dlg.GetSelections();
 	}
