@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,15 +21,20 @@
 
 #include "DatabaseSelectDlg.h"
 #include "util.h"
+#include "i18n.h"
 #include <barry/barry.h>
 #include <sstream>
 
 DatabaseSelectDlg::DatabaseSelectDlg(const Barry::DatabaseDatabase &dbdb,
-				     const ConfigFile::DBListType &selections,
-				     const Glib::ustring &label)
-	: m_pTopLabel(0),
-	m_pTree(0),
-	m_selections(selections)
+				     const Barry::ConfigFile::DBListType &selections,
+				     bool auto_select_all,
+				     const Glib::ustring &label,
+				     bool backup_mode)
+	: m_backupMode(backup_mode)
+	, m_pTopLabel(0)
+	, m_pTree(0)
+	, m_selections(selections)
+	, m_auto_select_all(auto_select_all)
 {
 	Glib::RefPtr<Gnome::Glade::Xml> xml = LoadXml("DatabaseSelectDlg.glade");
 
@@ -50,6 +55,12 @@ DatabaseSelectDlg::DatabaseSelectDlg(const Barry::DatabaseDatabase &dbdb,
 	pButton->signal_clicked().connect(
 		sigc::mem_fun(*this, &DatabaseSelectDlg::on_deselect_all));
 
+	xml->get_widget("auto_select_all", m_pAutoSelectAllCheck);
+	m_pAutoSelectAllCheck->set_active(m_auto_select_all);
+	if( !m_backupMode ) {
+		m_pAutoSelectAllCheck->hide();
+	}
+
 	m_pTopLabel->set_text(label);
 
 	m_pListStore = Gtk::ListStore::create(m_Columns);
@@ -72,8 +83,8 @@ void DatabaseSelectDlg::LoadTree(const Barry::DatabaseDatabase &dbdb)
 		(*row)[m_Columns.m_name] = i->Name;
 	}
 	m_pTree->set_model(m_pListStore);
-	m_pTree->append_column_editable("Active", m_Columns.m_selected);
-	m_pTree->append_column("Name", m_Columns.m_name);
+	m_pTree->append_column_editable(_("Active"), m_Columns.m_selected);
+	m_pTree->append_column(_("Name"), m_Columns.m_name);
 }
 
 bool DatabaseSelectDlg::IsSelected(const std::string &dbname)
@@ -96,6 +107,9 @@ void DatabaseSelectDlg::SaveSelections()
 			m_selections.push_back( name );
 		}
 	}
+
+	// save the auto flag
+	m_auto_select_all = m_pAutoSelectAllCheck->get_active();
 }
 
 int DatabaseSelectDlg::run()
