@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 
 #include <gtkmm.h>
 #include <memory>
-#include "ConfigFile.h"
+#include <barry/barry.h>
 
 namespace Barry {
 	class DatabaseDatabase;
@@ -45,15 +45,20 @@ class DatabaseSelectDlg
 		}
 	};
 
+	// meta class flags
+	bool m_backupMode;	// if true, the checkbox is visible
+
 	// Widgets
 	std::auto_ptr<Gtk::Dialog> m_pDialog;
 	Gtk::Label *m_pTopLabel;
+	Gtk::CheckButton *m_pAutoSelectAllCheck;
 	Gtk::TreeView *m_pTree;
 	Columns m_Columns;
 	Glib::RefPtr<Gtk::ListStore> m_pListStore;
 
 	// data
-	ConfigFile::DBListType m_selections;
+	Barry::ConfigFile::DBListType m_selections;
+	bool m_auto_select_all;		// holds checkbox setting
 
 protected:
 	void LoadTree(const Barry::DatabaseDatabase &dbdb);
@@ -62,11 +67,13 @@ protected:
 
 public:
 	DatabaseSelectDlg(const Barry::DatabaseDatabase &dbdb,
-		const ConfigFile::DBListType &selections,
-		const Glib::ustring &label);
+		const Barry::ConfigFile::DBListType &selections,
+		bool auto_select_all,
+		const Glib::ustring &label, bool backup_mode);
 	~DatabaseSelectDlg();
 
-	const ConfigFile::DBListType& GetSelections() const { return m_selections; }
+	const Barry::ConfigFile::DBListType& GetSelections() const { return m_selections; }
+	bool AutoSelectAll() const { return m_auto_select_all; }
 
 	int run();
 
