@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,11 +23,10 @@
 #define __BARRYBACKUP_DEVICEIFACE_H__
 
 #include <barry/barry.h>
+#include <barry/barrybackup.h>
 #include <string>
 #include <memory>
 #include <stdint.h>
-#include "ConfigFile.h"
-#include "tarfile.h"
 
 #define DI_THREAD_DONE 100
 #define DI_THREAD_PROGRESS 101
@@ -37,6 +36,18 @@ namespace Glib {
 	class Mutex;
 }
 
+class Device
+{
+	Barry::ProbeResult result;
+
+public:
+	Device(const Barry::ProbeResult &result);
+
+	Barry::Pin GetPIN() const { return Barry::Pin(result.m_pin); };
+
+	friend class DeviceInterface;
+};
+
 class DeviceInterface :
 	public Barry::Parser,
 	public Barry::Builder
@@ -46,12 +57,18 @@ public:
 	{
 		Glib::Dispatcher *m_erase_db;	// to notify the app about the
 						// db erase stage of restore
+		Glib::Dispatcher *m_restored_db;// to notify the app that the
+						// previous erase_db was
+						// restored... do not rely
+						// on the current db name ==
+						// the name found at erase time
 		Glib::Dispatcher *m_progress;
 		Glib::Dispatcher *m_error;
 		Glib::Dispatcher *m_done;
 
 		AppComm() :
 			m_erase_db(0),
+			m_restored_db(0),
 			m_progress(0),
 			m_error(0),
 			m_done(0)
@@ -59,16 +76,18 @@ public:
 		AppComm(Glib::Dispatcher *progress,
 			Glib::Dispatcher *error,
 			Glib::Dispatcher *done,
-			Glib::Dispatcher *erase_db) :
+			Glib::Dispatcher *erase_db,
+			Glib::Dispatcher *restored_db) :
 			m_erase_db(erase_db),
+			m_restored_db(restored_db),
 			m_progress(progress),
 			m_error(error),
 			m_done(done)
 			{}
 		bool IsValid() const
-			{ return m_erase_db && m_progress && m_error && m_done; }
+			{ return m_erase_db && m_restored_db && m_progress && m_error && m_done; }
 		void Invalidate()
-			{ m_erase_db = m_progress = m_error = m_done = 0; }
+			{ m_erase_db = m_restored_db = m_progress = m_error = m_done = 0; }
 	};
 
 	class Quit	// quit exception to break out of upload/download
@@ -76,27 +95,23 @@ public:
 	};
 
 private:
+	Device *m_dev;
 	Barry::Controller *m_con;
 	Barry::Mode::Desktop *m_desktop;
 	std::string m_last_error;
 	std::string m_last_thread_error;
 
 	AppComm m_AppComm;
-	std::auto_ptr<reuse::TarFile> m_tar, m_tarback;
 
-	std::auto_ptr<Barry::Probe> m_probe;
+	std::auto_ptr<Barry::Backup> m_backup;
+	std::auto_ptr<Barry::Restore> m_restore;
 
-	// parser and builder data (only one side uses these at a time)
-	ConfigFile::DBListType m_dbList;
+	mutable Barry::Backup::StatsType m_backupStats;
+
+	// parser and builder data
+	Barry::ConfigFile::DBListType m_dbBackupList;
 	mutable Glib::Mutex *m_dbnameMutex;
-	std::string m_current_dbname_not_thread_safe;
 	std::string m_current_dbname;
-	uint8_t m_rec_type;
-	uint32_t m_unique_id;
-	std::string m_tar_id_text;
-	std::string m_record_data;
-	bool m_end_of_tar;
-	bool m_tar_record_loaded;
 
 	// thread quit flag... not locked, only a byte
 	volatile bool m_thread_quit;
@@ -107,36 +122,33 @@ protected:
 	// threads
 	void BackupThread();
 	void RestoreThread();
-	void RestoreAndBackupThread();
 
 	// helpers
-	std::string MakeFilename(const std::string &pin, const std::string &label = "");
-	int CountFiles(reuse::TarFile &tar, const ConfigFile::DBListType &restoreList);
+	std::string MakeFilename(const std::string &label = "") const;
 	bool SplitTarPath(const std::string &tarpath, std::string &dbname,
-		std::string &dbid_text, uint8_t &dbrectype, uint32_t &dbid);
+		std::string &dbid_text, uint8_t &dbrectype, uint32_t &dbid) const;
 
 	// Sets the name of the database the thread is currently working on
 	void SetThreadDBName(const std::string &dbname);
 
 public:
-	DeviceInterface();
+	DeviceInterface(Device *dev = 0);
 	~DeviceInterface();
 
 	const std::string& get_last_error() const { return m_last_error; }
 	const std::string& get_last_thread_error() const { return m_last_thread_error; }
 
-	void Probe();
-	unsigned int ProbeCount();
-	uint32_t GetPin(unsigned int index);
-	Usb::DeviceIDType GetDev(unsigned int index);
-
-	bool Connect(unsigned int index);
-	bool Connect(const Barry::ProbeResult &dev);
+	void Reset();
+	bool Connect();
 	bool Password(const char *password);
 	void Disconnect();
 
+	const Barry::ProbeResult& GetProbeResult() const { return m_con->GetProbeResult(); }
 	const Barry::DatabaseDatabase& GetDBDB() const { return m_desktop->GetDBDB(); }
-	int GetDeviceRecordTotal(const ConfigFile::DBListType &backupList) const;
+	unsigned int GetRecordTotal(const Barry::ConfigFile::DBListType &backupList) const;
+	unsigned int GetRecordTotal(const Barry::ConfigFile::DBListType &restoreList, const std::string &filename) const;
+
+	std::vector<std::string> CompareTotals(const Barry::ConfigFile::DBListType &backupList) const;
 
 	void QuitThread()	{ m_thread_quit = true; }
 
@@ -144,34 +156,20 @@ public:
 	std::string GetThreadDBName() const;
 
 	bool StartBackup(AppComm comm,
-		const ConfigFile::DBListType &backupList,
-		const std::string &directory, const std::string &pin,
-		const std::string &backupLabel);
+		const Barry::ConfigFile::DBListType &backupList,
+		const std::string &directory, const std::string &backupLabel);
 	bool StartRestore(AppComm comm,
-		const ConfigFile::DBListType &restoreList,
-		const std::string &tarfilename, int *pRecordCount = 0);
-	// this is for debugging... starts a restore, and then does an
-	// immediate backup of the same DB before moving on to the next
-	bool StartRestoreAndBackup(AppComm comm,
-		const ConfigFile::DBListType &restoreAndBackupList,
-		const std::string &tarfilename,
-		const std::string &directory, const std::string &pin,
-		int *pRecordCount = 0);
+		const Barry::ConfigFile::DBListType &restoreList,
+		const std::string &tarfilename);
 
 	// Barry::Parser overrides
-	virtual void Clear();
-	virtual void SetIds(uint8_t RecType, uint32_t UniqueId);
-	virtual void ParseHeader(const Barry::Data &data, size_t &offset);
-	virtual void ParseFields(const Barry::Data &data, size_t &offset,
+	virtual void ParseRecord(const Barry::DBData &data,
 		const Barry::IConverter *ic);
-	virtual void Store();
 
 	// Barry::Builder overrides
-	virtual bool Retrieve(unsigned int dbId);
-	virtual uint8_t GetRecType() const;
-	virtual uint32_t GetUniqueId() const;
-	virtual void BuildHeader(Barry::Data &data, size_t &offset);
-	virtual void BuildFields(Barry::Data &data, size_t &offset, const Barry::IConverter *ic);
+	virtual bool BuildRecord(Barry::DBData &data, size_t &offset, const Barry::IConverter *ic);
+	virtual bool FetchRecord(Barry::DBData &data, const Barry::IConverter *ic);
+	virtual bool EndOfFile() const { return false; } // not used
 	void SkipCurrentDB() throw();	// helper function for halding restore errors
 };
 
