@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include <barry/barry.h>
 #include "BackupWindow.h"
 #include "util.h"
+#include "i18n.h"
 
 //
 // The catch-all handler for exceptions that occur inside signals
@@ -49,7 +50,7 @@ void main_exception_handler()
 		// special check for EBUSY to make the error message
 		// more user friendly
 		Gtk::MessageDialog msg("");
-		if( e.libusb_errcode() == -EBUSY ) {
+		if( e.system_errcode() == -EBUSY ) {
 			msg.set_message("Device busy.  This is likely due to the usb_storage kernel module being loaded.  Try 'rmmod usb_storage'.");
 		}
 		else {
@@ -91,6 +92,8 @@ void terminate_handler()
 
 int main(int argc, char *argv[])
 {
+	INIT_I18N(PACKAGE);
+
 	old_unexpected_handler = std::set_unexpected(&unexpected_handler);
 	old_terminate_handler = std::set_terminate(&terminate_handler);
 
