@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,8 +26,6 @@
 #include <string>
 
 Glib::RefPtr<Gnome::Glade::Xml> LoadXml(const char *filename);
-std::string GetPath(const std::string &filename);
-bool CheckPath(const std::string &path, std::string *perr = 0);
 
 #endif
 
