@@ -1,11 +1,12 @@
-#!/bin/sh
+#!/bin/bash
 
-if [ -z "$1" -o -z "$2" -o -z "$3" ] ; then
+if [ -z "$1" -o -z "$2" -o -z "$3" -o -z "$4" ] ; then
 	echo
-	echo "Usage: ./git-extract.sh MAJOR MINOR commit"
+	echo "Usage: ./git-extract.sh LOGICAL MAJOR MINOR commit"
 	echo
-	echo "MAJOR is the desired major version number"
-	echo "MINOR is the desired minor version number"
+	echo "LOGICAL is the desired logical version number"
+	echo "MAJOR is the desired libmajor version number"
+	echo "MINOR is the desired libminor version number"
 	echo "commit is a git commit tag to use for the extraction"
 	echo
 	echo "A build directory, containing the packaged results,"
@@ -19,16 +20,17 @@ if [ -z "$1" -o -z "$2" -o -z "$3" ] ; then
 	echo "          cd /tmp/play"
 	echo "          /path/to/git/repo/barry/maintainer/git-extract.sh 0 14 master"
 	echo
-	echo "This will create /tmp/play/build containing the results, and"
+	echo "This will create /tmp/play/barry-*.*.* containing the results, and"
 	echo "use /path/to/git/repo as the repository."
 	echo
 	exit 1
 fi
 
-DIRNAME="barry-$1.$2"
-MAJOR="$1"
-MINOR="$2"
-COMMIT="$3"
+DIRNAME="barry-$1.$2.$3"
+LOGICAL="$1"
+MAJOR="$2"
+MINOR="$3"
+COMMIT="$4"
 RUNDIR="$(dirname "$0")/.."
 
 set -e
@@ -36,9 +38,8 @@ set -e
 echo "Using git dir: $RUNDIR"
 sleep 3s
 
-mkdir build
 umask 0022
 (cd "$RUNDIR" && git archive --format=tar --prefix=$DIRNAME/ $COMMIT) |\
-	tar -x -C build -f -
+	tar -x -f -
 echo "Extracted $COMMIT..."
 
