@@ -1,6 +1,5 @@
-#!/bin/sh
+#!/bin/bash
 
-CHROOT="/var/chroot"
 BUILDUSER="cdfrey"
 
 if [ -z "$1" ] ; then
