@@ -5,7 +5,7 @@
 
 /*
 	Copyright (C) 2009, Nicolas VIVIEN (opensync plugin porting on opensync 0.4x ; Task & Memo support)
-    Copyright (C) 2006-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2006-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,13 +36,14 @@
 #include "vevent.h"
 #include "vcard.h"
 #include "trace.h"
+#include "config.h"
 #include <string>
 #include <glib.h>
 #include <stdint.h>
 #include <string.h>
 #include <errno.h>
 
-typedef vSmartPtr<OSyncList, OSyncList, &osync_list_free> AutoOSyncList;
+typedef Barry::vSmartPtr<OSyncList, OSyncList, &osync_list_free> AutoOSyncList;
 
 // All functions that are callable from outside must look like C
 extern "C" {
@@ -114,7 +115,7 @@ void GetChanges(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx
 	// shortcut references
 	using namespace Barry;
 	using Barry::RecordStateTable;
-	Mode::Desktop &desktop = *env->m_pDesktop;
+	Mode::Desktop &desktop = *env->GetDesktop();
 
 	// find hash table
 	//
@@ -316,10 +317,14 @@ bool FinishSync(OSyncContext *ctx, BarryEnvironment *env, DatabaseSyncState *pSy
 		return true;
 	}
 
-	Barry::Mode::Desktop &desktop = *env->m_pDesktop;
+	// we reconnect to the device here, since dirty flags
+	// for records we've just touched do not show up until
+	// a disconnect... as far as I can tell.
+	env->ReconnectForDirtyFlags();
 
 	// get the state table again, so we can update
 	// the cache properly
+	Barry::Mode::Desktop &desktop = *env->GetDesktop();
 	desktop.GetRecordStateTable(pSync->m_dbId, pSync->m_Table);
 
 	// clear all dirty flags in device
@@ -595,7 +600,7 @@ static void *initialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError *
 		if (osync_plugin_authentication_option_is_supported(optauth, OSYNC_PLUGIN_AUTHENTICATION_PASSWORD)) {
 			const char *val = osync_plugin_authentication_get_password(optauth);
 
-			env->m_password = val;
+			env->SetPassword(val);
 		}
 
 
@@ -699,10 +704,9 @@ static void connect(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext
 			Barry::Probe probe;
 			int nIndex = probe.FindActive(env->m_pin);
 			if( nIndex == -1 ) {
-				osync_context_report_error(ctx, OSYNC_ERROR_NO_CONNECTION, "Unable to find PIN %lx", env->m_pin);
+				osync_context_report_error(ctx, OSYNC_ERROR_NO_CONNECTION, "Unable to find PIN %x", env->m_pin);
 				return;
 			}
-			env->m_ProbeResult = probe.Get(nIndex);
 
 			trace.log("connecting...");
 
@@ -854,7 +858,7 @@ static void commit_change(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncCo
 
 		// make references instead of pointers
 		Barry::RecordStateTable &table = pSync->m_Table;
-		Barry::Mode::Desktop &desktop = *env->m_pDesktop;
+		Barry::Mode::Desktop &desktop = *env->GetDesktop();
 		unsigned int dbId = pSync->m_dbId;
 
 
@@ -896,6 +900,11 @@ static void commit_change(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncCo
 			}
 		}
 
+		// if we get here, we are about to update the device,
+		// and dirty flags will change in such a way that a
+		// reconnect will be required later... so flag this state
+		env->RequireDirtyReconnect();
+
 
 		std::string errmsg;
 		bool status;
@@ -968,11 +977,6 @@ static void contact_sync_done(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSy
 
 		BarryEnvironment *env = (BarryEnvironment *) userdata;
 
-		// we reconnect to the device here, since dirty flags
-		// for records we've just touched do not show up until
-		// a disconnect... as far as I can tell.
-		env->Reconnect();
-
 		// do cleanup for each database
 		if( FinishSync(ctx, env, &env->m_ContactsSync) )
 		{
@@ -999,11 +1003,6 @@ static void event_sync_done(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSync
 
 		BarryEnvironment *env = (BarryEnvironment *) userdata;
 
-		// we reconnect to the device here, since dirty flags
-		// for records we've just touched do not show up until
-		// a disconnect... as far as I can tell.
-		env->Reconnect();
-
 		// do cleanup for each database
 		if( FinishSync(ctx, env, &env->m_CalendarSync) )
 		{
@@ -1030,11 +1029,6 @@ static void journal_sync_done(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSy
 
 		BarryEnvironment *env = (BarryEnvironment *) userdata;
 
-		// we reconnect to the device here, since dirty flags
-		// for records we've just touched do not show up until
-		// a disconnect... as far as I can tell.
-		env->Reconnect();
-
 		// do cleanup for each database
 		if( FinishSync(ctx, env, &env->m_JournalSync) )
 		{
@@ -1061,11 +1055,6 @@ static void todo_sync_done(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncC
 
 		BarryEnvironment *env = (BarryEnvironment *) userdata;
 
-		// we reconnect to the device here, since dirty flags
-		// for records we've just touched do not show up until
-		// a disconnect... as far as I can tell.
-		env->Reconnect();
-
 		// do cleanup for each database
 		if( FinishSync(ctx, env, &env->m_TodoSync) )
 		{
@@ -1109,24 +1098,27 @@ osync_bool get_sync_info(OSyncPluginEnv *env, OSyncError **error)
 
 	// Create a new OpenSync plugin
 	OSyncPlugin *plugin = osync_plugin_new(error);
-
-	if (!plugin) {
-		osync_error_unref(error);
-
+	if( !plugin ) {
+		trace.log(osync_error_print(error));
 		return false;
 	}
 
 	// Describe our plugin
 	osync_plugin_set_name(plugin, "barry-sync");
-	osync_plugin_set_longname(plugin, "Barry OpenSync plugin v0.15 for the Blackberry handheld");
+	osync_plugin_set_longname(plugin, "Barry OpenSync plugin v" PACKAGE_VERSION " for the Blackberry handheld");
 	osync_plugin_set_description(plugin, "Plugin to synchronize note, task, calendar and contact entries on USB Blackberry handhelds");
 
 	// Set the callback functions
-	osync_plugin_set_initialize(plugin, initialize);
-	osync_plugin_set_finalize(plugin, finalize);
-	osync_plugin_set_discover(plugin, discover);
+	osync_plugin_set_initialize_func(plugin, initialize);
+	osync_plugin_set_finalize_func(plugin, finalize);
+	osync_plugin_set_discover_func(plugin, discover);
+	osync_plugin_set_start_type(plugin, OSYNC_START_TYPE_PROCESS);
+
+	if( !osync_plugin_env_register_plugin(env, plugin, error) ) {
+		trace.log(osync_error_print(error));
+		return false;
+	}
 
-	osync_plugin_env_register_plugin(env, plugin);
 	osync_plugin_unref(plugin);
 
 	return true;
