@@ -4,7 +4,7 @@
 //
 
 /*
-    Copyright (C) 2006-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2006-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,9 +27,11 @@
 #include "vevent.h"
 #include "vcard.h"
 #include "trace.h"
+#include "config.h"
 #include <string>
 #include <glib.h>
 #include <stdint.h>
+#include <stdlib.h>
 #include <string.h>
 
 // All functions that are callable from outside must look like C
@@ -61,7 +63,7 @@ void GetChanges(OSyncContext *ctx, BarryEnvironment *env,
 	// shortcut references
 	using namespace Barry;
 	using Barry::RecordStateTable;
-	Mode::Desktop &desktop = *env->m_pDesktop;
+	Mode::Desktop &desktop = *env->GetDesktop();
 
 	// find the matching cache, state table, and id map for this change
 	DatabaseSyncState::cache_type &cache = pSync->m_Cache;
@@ -209,7 +211,7 @@ bool FinishSync(OSyncContext *ctx, BarryEnvironment *env, DatabaseSyncState *pSy
 		return true;
 	}
 
-	Barry::Mode::Desktop &desktop = *env->m_pDesktop;
+	Barry::Mode::Desktop &desktop = *env->GetDesktop();
 
 	// get the state table again, so we can update
 	// the cache properly
@@ -316,7 +318,6 @@ static void connect(OSyncContext *ctx)
 			osync_context_report_error(ctx, OSYNC_ERROR_NO_CONNECTION, "Unable to find PIN %lx", env->m_pin);
 			return;
 		}
-		env->m_ProbeResult = probe.Get(nIndex);
 
 		env->Connect(probe.Get(nIndex));
 
@@ -342,14 +343,15 @@ static void get_changeinfo(OSyncContext *ctx)
 	try {
 
 		BarryEnvironment *env = (BarryEnvironment *)osync_context_get_plugin_data(ctx);
+		OSyncMember *member = osync_context_get_member(ctx);
 
-		if( env->m_CalendarSync.m_Sync ) {
+		if( env->m_CalendarSync.m_Sync && osync_member_objtype_enabled(member, "event") ) {
 			GetChanges(ctx, env, &env->m_CalendarSync,
 				"Calendar", "event", "vevent20",
 				&VEventConverter::GetRecordData);
 		}
 
-		if( env->m_ContactsSync.m_Sync ) {
+		if( env->m_ContactsSync.m_Sync && osync_member_objtype_enabled(member, "contact") ) {
 			GetChanges(ctx, env, &env->m_ContactsSync,
 				"Address Book", "contact", "vcard30",
 				&VCardConverter::GetRecordData);
@@ -402,7 +404,7 @@ static osync_bool commit_change(OSyncContext *ctx, OSyncChange *change)
 		DatabaseSyncState::cache_type &cache = pSync->m_Cache;
 		Barry::RecordStateTable &table = pSync->m_Table;
 		idmap &map = pSync->m_IdMap;
-		Barry::Mode::Desktop &desktop = *env->m_pDesktop;
+		Barry::Mode::Desktop &desktop = *env->GetDesktop();
 		unsigned int dbId = pSync->m_dbId;
 
 
@@ -546,7 +548,7 @@ void get_info(OSyncEnv *env)
 	OSyncPluginInfo *info = osync_plugin_new_info(env);
 
 	info->name = "barry-sync";
-	info->longname = "Barry OpenSync plugin v0.15 for the Blackberry handheld";
+	info->longname = "Barry OpenSync plugin v" PACKAGE_VERSION " for the Blackberry handheld";
 	info->description = "Plugin to synchronize calendar and contact entries on USB Blackberry handhelds";
 	info->version = 1;		// API version (opensync api?)
 	info->is_threadsafe = TRUE;
