@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2006-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2006-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,70 +19,30 @@
     root directory of this project for more details.
 */
 
-#ifndef __BARRY_SYNC_VCARD_H__
-#define __BARRY_SYNC_VCARD_H__
+#ifndef __BARRYSYNC_VCARD_H__
+#define __BARRYSYNC_VCARD_H__
 
 #include <barry/barry.h>
 #include <stdint.h>
 #include <string>
-#include "vbase.h"
-#include "vformat.h"
 
 // forward declarations
 class BarryEnvironment;
 
-
-//
-// vCard
-//
-/// Class for converting between RFC 2425/2426 vCard data format,
-/// and the Barry::Contact class.
-///
-class vCard : public vBase
-{
-	// data to pass to external requests
-	char *m_gCardData;	// dynamic memory returned by vformat()... can
-				// be used directly by the plugin, without
-				// overmuch allocation and freeing (see Extract())
-	std::string m_vCardData;// copy of m_gCardData, for C++ use
-	Barry::Contact m_BarryContact;
-
-protected:
-	void AddAddress(const char *rfc_type, const Barry::PostalAddress &addr);
-	void AddCategories(const Barry::CategoryList &categories);
-	void AddPhoneCond(const std::string &phone);
-	void AddPhoneCond(const char *rfc_type, const std::string &phone);
-
-	void ParseAddress(vAttr &adr, Barry::PostalAddress &address);
-	void ParseCategories(vAttr &cat, Barry::CategoryList &cats);
-
-public:
-	vCard();
-	~vCard();
-
-	const std::string&	ToVCard(const Barry::Contact &con);
-	const Barry::Contact&	ToBarry(const char *vcal, uint32_t RecordId);
-
-	const std::string&	GetVCard() const { return m_vCardData; }
-	const Barry::Contact&	GetBarryContact() const { return m_BarryContact; }
-
-	char* ExtractVCard();
-
-	void Clear();
-};
-
-
 class VCardConverter
 {
 	char *m_Data;
 	Barry::Contact m_Contact;
 	uint32_t m_RecordId;
+	std::string m_last_errmsg;
 
 public:
 	VCardConverter();
 	explicit VCardConverter(uint32_t newRecordId);
 	~VCardConverter();
 
+	const std::string& GetLastError() const { return m_last_errmsg; }
+
 	// Transfers ownership of m_Data to the caller
 	char* ExtractData();
 
@@ -93,7 +53,7 @@ public:
 	void operator()(const Barry::Contact &rec);
 
 	// Barry builder operator
-	bool operator()(Barry::Contact &rec, unsigned int dbId);
+	bool operator()(Barry::Contact &rec, Barry::Builder &);
 
 	// Handles calling of the Barry::Controller to fetch a specific
 	// record, indicated by index (into the RecordStateTable).
