@@ -4,7 +4,7 @@
 //
 
 /*
-    Copyright (C) 2006-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2006-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,63 +19,22 @@
     root directory of this project for more details.
 */
 
-#ifndef __BARRY_SYNC_VEVENT_H__
-#define __BARRY_SYNC_VEVENT_H__
+#ifndef __BARRYSYNC_VEVENT_H__
+#define __BARRYSYNC_VEVENT_H__
 
 #include <barry/barry.h>
 #include <stdint.h>
 #include <string>
-#include "vbase.h"
-#include "vformat.h"
 
 // forward declarations
 class BarryEnvironment;
 
-//
-// vCalendar
-//
-/// Class for converting between RFC 2445 iCalendar data format,
-/// and the Barry::Calendar class.
-///
-class vCalendar : public vBase
-{
-	// data to pass to external requests
-	char *m_gCalData;	// dynamic memory returned by vformat()... can
-				// be used directly by the plugin, without
-				// overmuch allocation and freeing (see Extract())
-	std::string m_vCalData;	// copy of m_gCalData, for C++ use
-	Barry::Calendar m_BarryCal;
-
-	static const char *WeekDays[7];
-
-protected:
-	void RecurToVCal();
-	void RecurToBarryCal();
-
-	static unsigned short GetWeekDayIndex(const char *dayname);
-	bool HasMultipleVEvents() const;
-
-public:
-	vCalendar();
-	~vCalendar();
-
-	const std::string&	ToVCal(const Barry::Calendar &cal);
-	const Barry::Calendar&	ToBarry(const char *vcal, uint32_t RecordId);
-
-	const std::string&	GetVCal() const { return m_vCalData; }
-	const Barry::Calendar&	GetBarryCal() const { return m_BarryCal; }
-
-	char* ExtractVCal();
-
-	void Clear();
-};
-
-
 class VEventConverter
 {
 	char *m_Data;
 	Barry::Calendar m_Cal;
 	uint32_t m_RecordId;
+	std::string m_last_errmsg;
 
 public:
 	VEventConverter();
@@ -85,14 +44,20 @@ public:
 	// Transfers ownership of m_Data to the caller
 	char* ExtractData();
 
+	const Barry::Calendar& GetCalendar() const { return m_Cal; };
+	const std::string& GetLastError() const { return m_last_errmsg; }
+
 	// Parses vevent data
 	bool ParseData(const char *data);
 
+	// Merge vevent data
+	bool MergeData(const Barry::Calendar &origin);
+
 	// Barry storage operator
 	void operator()(const Barry::Calendar &rec);
 
 	// Barry builder operator
-	bool operator()(Barry::Calendar &rec, unsigned int dbId);
+	bool operator()(Barry::Calendar &rec, Barry::Builder &);
 
 	// Handles calling of the Barry::Controller to fetch a specific
 	// record, indicated by index (into the RecordStateTable).
@@ -110,6 +75,5 @@ public:
 		const char *data, bool add, std::string &errmsg);
 };
 
-
 #endif
 
