@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,6 +51,7 @@
 #include "usbwrap.h"			// to be moved to libusb someday
 #include "common.h"			// Init()
 #include "error.h"			// exceptions
+#include "configfile.h"
 #include "probe.h"			// device prober class
 #include "dataqueue.h"
 #include "socket.h"
@@ -59,11 +60,14 @@
 #include "parser.h"
 #include "builder.h"
 #include "ldif.h"
+#include "ldifio.h"
 #include "controller.h"
 #include "m_desktop.h"
 #include "m_ipmodem.h"
 #include "m_serial.h"
 #include "m_javaloader.h"
+#include "m_raw_channel.h"
+#include "m_jvmdebug.h"
 #include "version.h"
 #include "log.h"
 #include "sha1.h"
@@ -71,9 +75,22 @@
 #include "bmp.h"
 #include "cod.h"
 #include "record.h"
+#include "threadwrap.h"
+#include "vsmartptr.h"
+#include "pipe.h"
+#include "connector.h"
+#include "fifoargs.h"
+
+// Include the JDW Debug Parser classes
+#include "dp_codinfo.h"
+
+// Include the JDWP Server classes
+#include "j_manager.h"
+#include "j_server.h"
 
 // Include the template helpers after the record classes
 #include "m_desktoptmpl.h"
+#include "recordtmpl.h"
 
 #ifdef __BARRY_BOOST_MODE__
 // Boost serialization seems to be picky about header order, do them all here
