@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2009-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,12 +23,20 @@
 #define __BARRY_BMP_H__
 
 #include "dll.h"
-#include "data.h"
-#include "m_javaloader.h"
+#include <sys/types.h>
+#include <stdint.h>
 
 namespace Barry {
 
+class Data;
+class JLScreenInfo;
+
+BXEXPORT size_t GetBitmapHeadersSize();
 BXEXPORT size_t GetTotalBitmapSize(const JLScreenInfo &info);
+BXEXPORT void ScreenshotToRGB(const JLScreenInfo &info,
+	const Data &screenshot, Data &buffer, size_t offset,
+	int depth, bool invert,
+	bool overwrite_alpha = false, uint8_t alpha = 0xFF);
 BXEXPORT void ScreenshotToBitmap(const JLScreenInfo &info,
 	const Data &screenshot, Data &bitmap);
 
