@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,12 +20,13 @@
 */
 
 #include "controller.h"
-#include "probe.h"
+#include "controllerpriv.h"
 #include "common.h"
 #include "protocol.h"
 #include "protostructs.h"
 #include "data.h"
 #include "endian.h"
+#include "platform.h"
 #include <string.h>
 
 #define __DEBUG_MODE__
@@ -41,14 +42,11 @@ namespace Barry {
 ///
 /// \param[in]	device		One of the ProbeResult objects from the
 ///				Probe class.
+/// \param[in]	default_timeout	Override Usb::Device's default timeout
 ///
-Controller::Controller(const ProbeResult &device)
-	: m_result(device)
-	, m_dev(device.m_dev)
-	, m_iface(0)
-	, m_pin(device.m_pin)
-	, m_zero(m_dev, device.m_ep.write, device.m_ep.read, device.m_zeroSocketSequence)
-	, m_queue(0)
+Controller::Controller(const ProbeResult &device,
+			int default_timeout)
+	: m_priv(new PrivateControllerData(device, default_timeout))
 {
 	dout("Controller: Using non-threaded sockets");
 	SetupUsb(device);
@@ -64,82 +62,48 @@ Controller::Controller(const ProbeResult &device)
 ///				Probe class.
 /// \param[in]	queue		Plugin router object for reading data
 ///				from sockets.
+/// \param[in]	default_timeout	Override Usb::Device's default timeout
 ///
-Controller::Controller(const ProbeResult &device, SocketRoutingQueue &queue)
-	: m_result(device)
-	, m_dev(device.m_dev)
-	, m_iface(0)
-	, m_pin(device.m_pin)
-	, m_zero(queue, device.m_ep.write, device.m_zeroSocketSequence)
-	, m_queue(&queue)
+Controller::Controller(const ProbeResult &device,
+			SocketRoutingQueue &queue,
+			int default_timeout)
+	: m_priv(new PrivateControllerData(device, queue, default_timeout))
 {
 	dout("Controller: Using threaded socket router");
 
 	SetupUsb(device);
 
 	// set the queue to use our device
-	queue.SetUsbDevice(&m_dev, device.m_ep.write, device.m_ep.read);
+	queue.SetUsbDevice(&m_priv->m_dev, device.m_ep.write, device.m_ep.read);
 }
 
 void Controller::SetupUsb(const ProbeResult &device)
 {
 	unsigned char cfg;
-	if( !m_dev.GetConfiguration(cfg) )
-		throw Usb::Error(m_dev.GetLastError(),
+	if( !m_priv->m_dev.GetConfiguration(cfg) )
+		throw Usb::Error(m_priv->m_dev.GetLastError(),
 			"Controller: GetConfiguration failed");
 
-	if( cfg != BLACKBERRY_CONFIGURATION ) {
-		if( !m_dev.SetConfiguration(BLACKBERRY_CONFIGURATION) )
-			throw Usb::Error(m_dev.GetLastError(),
+	if( cfg != BLACKBERRY_CONFIGURATION || MUST_SET_CONFIGURATION ) {
+		if( !m_priv->m_dev.SetConfiguration(BLACKBERRY_CONFIGURATION) )
+			throw Usb::Error(m_priv->m_dev.GetLastError(),
 				"Controller: SetConfiguration failed");
 	}
 
-	m_iface = new Usb::Interface(m_dev, device.m_interface);
+	m_priv->m_iface = new Usb::Interface(m_priv->m_dev, device.m_interface);
 
-	m_dev.ClearHalt(device.m_ep.read);
-	m_dev.ClearHalt(device.m_ep.write);
+	if( device.m_needSetAltInterface ) {
+		m_priv->m_iface->SetAltInterface(device.m_altsetting);
+	}
+
+	if( device.m_needClearHalt ) {
+		m_priv->m_dev.ClearHalt(device.m_ep.read);
+		m_priv->m_dev.ClearHalt(device.m_ep.write);
+	}
 }
 
 Controller::~Controller()
 {
-//	// trap exceptions in the destructor
-//	try {
-//		// a non-default socket has been opened, close it
-//		m_socket.Close();
-//	}
-//	catch( std::runtime_error &re ) {
-//		// do nothing... log it?
-//		dout("Exception caught in ~Socket: " << re.what());
-//	}
-
-	// detach the router from our device
-	if( m_queue ) {
-		m_queue->ClearUsbDevice();
-		m_queue = 0;
-	}
-
-	// cleanup the interface
-	delete m_iface;
-
-	// this happens when for some reason the Desktop mode
-	// is not fully opened, but the device has already recommended
-	// a socket to open... in this case, reset the device
-	// in the hopes that on next open, it will be in a
-	// recognizable state.
-	//
-	// FIXME - this should not be necessary, and someday we
-	// we should figure out how to handle the "already open"
-	// response we get for the Desktop
-	//
-	// FIXME - halfOpen now seems to be handled in the Socket class...
-	// perhaps move this there if needed
-	//
-/*
-	if( m_halfOpen ) {
-		dout("Controller object destroyed in halfopen state, resetting device");
-		m_dev.Reset();
-	}
-*/
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -150,6 +114,20 @@ Controller::~Controller()
 // socket ID to use for that mode.
 //
 uint16_t Controller::SelectMode(ModeType mode)
+{
+	return SelectMode(mode, NULL);
+}
+//
+// Tells device which mode is desired, and returns the suggested
+// socket ID to use for that mode.
+//
+// If explicitModeName is not NULL then it will be used as the mode name.
+// Otherwise the default mode name for the given mode will be used.
+// It should be a nul terminated string if it is provided.
+//
+// The RawChannel mode requires an explicitModeName to be specified.
+//
+uint16_t Controller::SelectMode(ModeType mode, const char *explicitModeName)
 {
 	// select mode
 	Protocol::Packet packet;
@@ -161,31 +139,49 @@ uint16_t Controller::SelectMode(ModeType mode)
 	memset(packet.u.socket.u.mode.name, 0, sizeof(packet.u.socket.u.mode.name));
 
 	char *modeName = (char *) packet.u.socket.u.mode.name;
-	switch( mode )
-	{
-	case Bypass:
-		strcpy(modeName, "RIM Bypass");
-		break;
 
-	case Desktop:
-		strcpy(modeName, "RIM Desktop");
-		break;
+	if( explicitModeName ) {
+		if( strlen(explicitModeName) >= sizeof(packet.u.socket.u.mode.name) ) {
+			throw std::logic_error("Controller: explicit mode name too long");
+		}
+		strcpy(modeName, explicitModeName);
+	}
+	else {
+		// No modeName given, use the default
+		switch( mode )
+		{
+		case Bypass:
+			strcpy(modeName, "RIM Bypass");
+			break;
 
-	case JavaLoader:
-		strcpy(modeName, "RIM_JavaLoader");
-		break;
+		case Desktop:
+			strcpy(modeName, "RIM Desktop");
+			break;
 
-	case UsbSerData:
-		strcpy(modeName, "RIM_UsbSerData");
-		break;
+		case JavaLoader:
+			strcpy(modeName, "RIM_JavaLoader");
+			break;
 
-	case UsbSerCtrl:
-		strcpy(modeName, "RIM_UsbSerCtrl");
-		break;
+		case JVMDebug:
+			strcpy(modeName, "RIM_JVMDebug");
+			break;
 
-	default:
-		throw std::logic_error("Controller: Invalid mode in SelectMode");
-		break;
+		case UsbSerData:
+			strcpy(modeName, "RIM_UsbSerData");
+			break;
+
+		case UsbSerCtrl:
+			strcpy(modeName, "RIM_UsbSerCtrl");
+			break;
+
+		case RawChannel:
+			throw std::logic_error("Controller: No channel name given with RawChannel mode");
+			break;
+
+		default:
+			throw std::logic_error("Controller: Invalid mode in SelectMode");
+			break;
+		}
 	}
 
 	// send mode command before we open, as a default socket is socket 0
@@ -193,28 +189,20 @@ uint16_t Controller::SelectMode(ModeType mode)
 	Data response;
 
 	try {
-		m_zero.Send(command, response);
+		m_priv->m_zero.Send(command, response);
 
 		// get the data socket number
 		// indicates the socket number that
 		// should be used below in the Open() call
-		Protocol::CheckSize(response, SB_MODE_PACKET_RESPONSE_SIZE);
 		MAKE_PACKET(modepack, response);
+		if( modepack->command == SB_COMMAND_MODE_NOT_SELECTED ) {
+		        throw Error("Controller: requested mode not supported");
+		}
 		if( modepack->command != SB_COMMAND_MODE_SELECTED ) {
 			eeout(command, response);
 			throw Error("Controller: mode not selected");
 		}
 
-		if (mode == Desktop) {
-			// On the BlackBerry Storm, I have to read a packet
-			// after opening a socket. (only for Desktop mode)
-			// Otherwise, barrybackup and opensync-plugin can crash (timeout)
-			// I don't know why ! Maybe a bug on the handheld.
-			m_zero.HideSequencePacket(false);
-			m_zero.Receive(response);
-			m_zero.HideSequencePacket(true);
-		}
-
 		// return the socket that the device is expecting us to use
 		return btohs(modepack->u.socket.socket);
 	}
@@ -225,10 +213,35 @@ uint16_t Controller::SelectMode(ModeType mode)
 	}
 }
 
+//
+// OpenSocket
+//
+/// Can be called multiple times, in case of password retries.
+/// See also Mode::RetryPassword()
+///
+SocketHandle Controller::OpenSocket(uint16_t socket, const char *password)
+{
+	return m_priv->m_zero.Open(socket, password);
+}
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // public API
 
+bool Controller::HasQueue() const
+{
+	return m_priv->m_queue;
+}
+
+SocketRoutingQueue* Controller::GetQueue()
+{
+	return m_priv->m_queue;
+}
+
+const ProbeResult& Controller::GetProbeResult() const
+{
+	return m_priv->m_result;
+}
 
 } // namespace Barry
 
