@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,9 +23,7 @@
 #define __BARRY_CONTROLLER_H__
 
 #include "dll.h"
-#include "usbwrap.h"
 #include "socket.h"
-#include "probe.h"
 
 /// Project namespace, containing all related functions and classes.
 /// This is the only namespace applications should be concerned with,
@@ -34,12 +32,17 @@ namespace Barry {
 
 // forward declarations
 class SocketRoutingQueue;
+class ProbeResult;
+
+class PrivateControllerData;
 
 namespace Mode {
 	class Mode;
 	class IpModem;
 	class Serial;
 	class JavaLoader;
+	class JVMDebug;
+	class RawChannel;
 }
 
 //
@@ -71,6 +74,8 @@ class BXEXPORT Controller
 	friend class Barry::Mode::IpModem;
 	friend class Barry::Mode::Serial;
 	friend class Barry::Mode::JavaLoader;
+	friend class Barry::Mode::JVMDebug;
+	friend class Barry::Mode::RawChannel;
 
 public:
 	/// Handheld mode type
@@ -80,33 +85,36 @@ public:
 		Desktop,		//< desktop mode required for database
 					//< operation
 		JavaLoader,		//< experimental
+		JVMDebug,		//< experimental
 		UsbSerData,		//< GPRS modem support over USB
-		UsbSerCtrl		//< internally used behind the scenes
+		UsbSerCtrl,		//< internally used behind the scenes
+		RawChannel		//< raw channel
 	};
 
 private:
-	ProbeResult m_result;
-	Usb::Device m_dev;
-	Usb::Interface *m_iface;
-	uint32_t m_pin;
-
-	SocketZero m_zero;
-	SocketRoutingQueue *m_queue;	//< ptr to external object; no delete
+	std::auto_ptr<PrivateControllerData> m_priv;
 
 private:
 	void SetupUsb(const ProbeResult &device);
 
 protected:
 	uint16_t SelectMode(ModeType mode);	// returns mode socket
+	uint16_t SelectMode(ModeType mode, const char *explicitModeName); // returns mode socket
+	SocketHandle OpenSocket(uint16_t socket, const char *password = 0);
+	PrivateControllerData* GetPrivate() { return m_priv.get(); }
 
 public:
-	explicit Controller(const ProbeResult &device);
-	Controller(const ProbeResult &device, SocketRoutingQueue &queue);
+	explicit Controller(const ProbeResult &device,
+		int default_timeout = USBWRAP_DEFAULT_TIMEOUT);
+	Controller(const ProbeResult &device, SocketRoutingQueue &queue,
+		int default_timeout = USBWRAP_DEFAULT_TIMEOUT);
 	~Controller();
 
-	bool HasQueue() const { return m_queue; }
+	bool HasQueue() const;
+	SocketRoutingQueue* GetQueue();	// FIXME - not really ideal to have
+					// this exposed, but oh well
 
-	const ProbeResult& GetProbeResult() const { return m_result; }
+	const ProbeResult& GetProbeResult() const;
 };
 
 } // namespace Barry
