@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ extern std::ostream *LogStream;
 #endif // __BARRY_DEBUG_H__
 
 // data dump output - controlled by command line -v switch
-#define ddout(x)	if(Barry::__data_dump_mode__) { Barry::LogLock lock; (*Barry::LogStream) << x << std::endl; }
+#define ddout(x)	if(::Barry::__data_dump_mode__) { ::Barry::LogLock lock; (*::Barry::LogStream) << x << std::endl; }
 
 #ifdef __DEBUG_MODE__
 	// debugging on
@@ -39,14 +39,14 @@ extern std::ostream *LogStream;
 	#undef eout
 
 	// low level debug output
-	#define dout(x)  	if(Barry::__data_dump_mode__) { Barry::LogLock lock; (*Barry::LogStream) << x << std::endl; }
+	#define dout(x)		if(::Barry::__data_dump_mode__) { ::Barry::LogLock lock; (*::Barry::LogStream) << x << std::endl; }
 //	#define dout(x)
 
 	// exception output
-	#define eout(x)  	{ Barry::LogLock lock; (*Barry::LogStream) << x << std::endl; }
+	#define eout(x)		{ ::Barry::LogLock lock; (*::Barry::LogStream) << x << std::endl; }
 
 	// easy exception output
-	#define eeout(c, r)	{ Barry::LogLock lock; (*Barry::LogStream) << "Sent packet:\n" << c << "\n" << "Response packet:\n" << r << "\n"; }
+	#define eeout(c, r)	{ ::Barry::LogLock lock; (*::Barry::LogStream) << "Sent packet:\n" << c << "\n" << "Response packet:\n" << r << "\n"; }
 
 	// handle assert()
 	#undef NDEBUG
@@ -57,10 +57,11 @@ extern std::ostream *LogStream;
 	#undef eout
 
 	#define dout(x)
-	#define eout(x)  	{ Barry::LogLock lock; (*Barry::LogStream) << x << std::endl; }
-	#define eeout(c, r)	{ Barry::LogLock lock; (*Barry::LogStream) << "Sent packet:\n" << c << "\n" << "Response packet:\n" << r << "\n"; }
+	#define eout(x)		{ ::Barry::LogLock lock; (*::Barry::LogStream) << x << std::endl; }
+	#define eeout(c, r)	{ ::Barry::LogLock lock; (*::Barry::LogStream) << "Sent packet:\n" << c << "\n" << "Response packet:\n" << r << "\n"; }
 
 	// handle assert() as well
+	#undef NDEBUG
 	#define NDEBUG
 #endif
 
