@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include <iostream>
 #include <iomanip>
 #include <string.h>
+#include "ios_state.h"
 
 #define __DEBUG_MODE__
 #include "debug.h"
@@ -78,6 +79,8 @@ const ContactLdif::NameToFunc ContactLdif::FieldMap[] = {
 		&ContactLdif::PublicKey, &ContactLdif::SetPublicKey },
 	{ "Notes", "Notes",
 		&ContactLdif::Notes, &ContactLdif::SetNotes },
+	{ "Image", "Contact photo",
+		&ContactLdif::Image, &ContactLdif::SetImage },
 	{ "WorkPostalAddress", "Mailing Work address (includes address lines, city, province, country, and postal code)",
 		&ContactLdif::WorkPostalAddress, &ContactLdif::SetWorkPostalAddress },
 	{ "HomePostalAddress", "Mailing home address (includes address lines, city, province, country, and postal code)",
@@ -141,6 +144,14 @@ ContactLdif::ContactLdif(const std::string &baseDN)
 	Map("postalAddress", &ContactLdif::WorkPostalAddress, &ContactLdif::SetWorkPostalAddress);
 	Map("homePostalAddress", &ContactLdif::HomePostalAddress, &ContactLdif::SetHomePostalAddress);
 	Map("note", &ContactLdif::Notes, &ContactLdif::SetNotes);
+	// FIXME - jpegPhoto looks like the only LDIF field for photo
+	// images... it is unknown which format will come from the
+	// BlackBerry in the Image field, so we can't guarantee
+	// that Image will be in JPG.  This mapping can be done manually
+	// from the btool command line with "-m jpegPhoto,Image,Image"
+	// Reading photos from LDIF should be fine, since the BlackBerry
+	// seems to handle most formats.
+//	Map("jpegPhoto", &ContactLdif::Image, &ContactLdif::SetImage);
 
 	// add heuristics hooks
 	Hook("cn", &m_cn);
@@ -392,6 +403,11 @@ std::string ContactLdif::Notes(const Barry::Contact &con) const
 	return con.Notes;
 }
 
+std::string ContactLdif::Image(const Barry::Contact &con) const
+{
+	return con.Image;
+}
+
 std::string ContactLdif::WorkPostalAddress(const Barry::Contact &con) const
 {
 	return con.WorkAddress.GetLabel();
@@ -548,6 +564,11 @@ void ContactLdif::SetNotes(Barry::Contact &con, const std::string &val) const
 	con.Notes = val;
 }
 
+void ContactLdif::SetImage(Barry::Contact &con, const std::string &val) const
+{
+	con.Image = val;
+}
+
 void ContactLdif::SetWorkPostalAddress(Barry::Contact &con, const std::string &val) const
 {
 	// FIXME;
@@ -632,12 +653,14 @@ bool ContactLdif::RunHeuristics(Barry::Contact &con)
 void ContactLdif::DumpLdif(std::ostream &os,
 		       const Barry::Contact &con) const
 {
+	ios_format_state state(os);
+
 	// start fresh
 	ClearArrayState();
 
 	// setup stream state
-	std::ios::fmtflags oldflags = os.setf(std::ios::left);
-	char fill = os.fill(' ');
+	os.setf(std::ios::left);
+	os.fill(' ');
 
 	if( FirstName(con).size() == 0 && LastName(con).size() == 0 )
 		return;			// nothing to do
@@ -667,10 +690,6 @@ void ContactLdif::DumpLdif(std::ostream &os,
 
 	// last line must be empty
 	os << "\n";
-
-	// cleanup the stream
-	os.flags(oldflags);
-	os.fill(fill);
 }
 
 bool ContactLdif::ReadLdif(std::istream &is, Barry::Contact &con)
@@ -754,8 +773,10 @@ bool ContactLdif::ReadLdif(std::istream &is, Barry::Contact &con)
 
 void ContactLdif::DumpMap(std::ostream &os) const
 {
-	std::ios::fmtflags oldflags = os.setf(std::ios::left);
-	char fill = os.fill(' ');
+	ios_format_state state(os);
+
+	os.setf(std::ios::left);
+	os.fill(' ');
 
 	os << "ContactLdif Mapping:\n";
 
@@ -777,10 +798,6 @@ void ContactLdif::DumpMap(std::ostream &os) const
 	}
 
 	os << "   >>> DN attribute: " << m_dnAttr.name << "\n";
-
-	// cleanup the stream
-	os.flags(oldflags);
-	os.fill(fill);
 }
 
 std::string ContactLdif::MakeLdifData(const std::string &str)
