@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2008-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2008-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,12 @@
 */
 
 #include "log.h"
+#include "clog.h"
 #include <pthread.h>
+#include <stdio.h>
+#include <stdarg.h>
+#include <string.h>
+#include <iostream>
 
 namespace Barry {
 
@@ -52,3 +57,36 @@ std::ostream* GetLogStream()
 
 } // namespace Barry
 
+// Callable from C:
+
+void BarryLogf(int verbose, const char *msg, ...)
+{
+	va_list vl;
+	va_start(vl, msg);
+	char buffer[2048];
+	char *output = buffer;
+	int buflen = sizeof(buffer);
+	int n = vsnprintf(buffer, buflen, msg, vl);
+	if( n < 0 || n >= buflen ) {
+		buflen = n + 100;
+		output = new char [buflen];
+		n = vsnprintf(output, buflen, msg, vl);
+		if( n < 0 || n >= buflen ) {
+			delete [] output;
+			output = buffer;
+			strcpy(buffer, "BarryLog: (trace error, output too long for buffer)");
+		}
+	}
+	va_end(vl);
+
+	if( verbose ) {
+		barryverbose(output);
+	}
+	else {
+		barrylog(output);
+	}
+
+	if( output != buffer )
+		delete [] output;
+}
+
