@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2008-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2008-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -47,10 +47,10 @@ BXEXPORT std::ostream* GetLogStream();
 
 } // namespace Barry
 
-#define barrylog(x)	{ Barry::LogLock lock; (*Barry::GetLogStream()) << x << std::endl; }
+#define barrylog(x)	{ ::Barry::LogLock lock; (*::Barry::GetLogStream()) << x << std::endl; }
 
 // controlled by command line -v switch
-#define barryverbose(x)	if(Barry::LogVerbose()) { Barry::LogLock lock; (*Barry::GetLogStream()) << x << std::endl; }
+#define barryverbose(x)	if(::Barry::LogVerbose()) { ::Barry::LogLock lock; (*::Barry::GetLogStream()) << x << std::endl; }
 
 #endif // __BARRY_LOG_H__
 
