@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,12 +51,14 @@ void Desktop::SaveDatabaseByType(StorageT &store)
 template <class StorageT>
 void Desktop::LoadDatabaseByName(const std::string &name, StorageT &store)
 {
-	if( name == Contact::GetDBName() )
-		LoadDatabaseByType<Contact>(store);
-	else if( name == Message::GetDBName() )
-		LoadDatabaseByType<Message>(store);
-	else if( name == Calendar::GetDBName() )
-		LoadDatabaseByType<Calendar>(store);
+#undef HANDLE_PARSER
+#define HANDLE_PARSER(rtype) \
+	else if( name == Barry::rtype::GetDBName() ) \
+		LoadDatabaseByType<rtype>(store);
+
+	if( !name.size() )
+		throw Error("Empty database name in LoadDatabaseByName");
+	ALL_KNOWN_PARSER_TYPES
 	else
 		throw Error("Unknown database name in LoadDatabaseByName: " + name);
 }
@@ -64,12 +66,14 @@ void Desktop::LoadDatabaseByName(const std::string &name, StorageT &store)
 template <class StorageT>
 void Desktop::SaveDatabaseByName(const std::string &name, StorageT &store)
 {
-	if( name == Contact::GetDBName() )
-		SaveDatabaseByType<Contact>(store);
-	else if( name == Message::GetDBName() )
-		SaveDatabaseByType<Message>(store);
-	else if( name == Calendar::GetDBName() )
-		SaveDatabaseByType<Calendar>(store);
+#undef HANDLE_BUILDER
+#define HANDLE_BUILDER(rtype) \
+	else if( name == Barry::rtype::GetDBName() ) \
+		SaveDatabaseByType<rtype>(store);
+
+	if( !name.size() )
+		throw Error("Empty database name in SaveDatabaseByName");
+	ALL_KNOWN_BUILDER_TYPES
 	else
 		throw Error("Unknown database name in SaveDatabaseByName: " + name);
 }
