@@ -5,7 +5,7 @@
 ///
 
 /*
-    Copyright (C) 2008-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2008-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 #include "m_ipmodem.h"
 #include "controller.h"
+#include "controllerpriv.h"
 #include "data.h"
 #include "debug.h"
 #include <sstream>
@@ -42,7 +43,7 @@ IpModem::IpModem(Controller &con,
 		DeviceDataCallback callback,
 		void *callback_context)
 	: m_con(con)
-	, m_dev(con.m_dev)
+	, m_dev(con.GetPrivate()->m_dev)
 	, m_continue_reading(false)
 	, m_callback(callback)
 	, m_callback_context(callback_context)
@@ -223,9 +224,11 @@ void IpModem::Open(const char *password)
 		throw Barry::Error(oss.str());
 	}
 
-	// clear halt when starting out
-	m_dev.ClearHalt(pair.read);
-	m_dev.ClearHalt(pair.write);
+	// clear halt when starting out only if needed
+	if( m_con.GetProbeResult().m_needClearHalt ) {
+		m_dev.ClearHalt(pair.read);
+		m_dev.ClearHalt(pair.write);
+	}
 
 	// Send stop command
 	ddout("IPModem: Sending Stop Response:\n");
@@ -337,6 +340,12 @@ void IpModem::Open(const char *password)
 
 		default: 	// ???
 			ddout("IPModem: Unknown AT command response.\n");
+			// treat this unknown data as a serial response
+			if( m_callback ) {
+				(*m_callback)(m_callback_context,
+					data.GetData(),
+					data.GetSize());
+			}
 			break;
 		}
 	}
@@ -406,19 +415,24 @@ void IpModem::Close()
 	memcpy(&end[24], special_flag, sizeof(special_flag));
 	m_dev.BulkWrite(write_ep, end, sizeof(end));
 	m_dev.BulkWrite(write_ep, stop, sizeof(stop));
-	try {
-		m_dev.BulkRead(read_ep, data, 5000);
-		ddout("IPModem: Close read packet:\n" << data);
-	}
-	catch( Usb::Timeout &to ) {
-		// do nothing on timeouts
-		ddout("IPModem: Close Read Timeout");
-	}
+
 	// stop the read thread
 	if( m_continue_reading ) {
 		m_continue_reading = false;
 		pthread_join(m_modem_read_thread, NULL);
 	}
+	else {
+		// otherwise, drain the last read
+		try {
+			m_dev.BulkRead(read_ep, data, 5000);
+			ddout("IPModem: Close read packet:\n" << data);
+		}
+		catch( Usb::Timeout &to ) {
+			// do nothing on timeouts
+			ddout("IPModem: Close Read Timeout");
+		}
+	}
+
 	ddout("IPmodem: Closed!");
 
 }
