@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
     Copyright (C) 2008-2009, Nicolas VIVIEN
 
         Some parts are inspired from m_desktop.h
@@ -39,6 +39,7 @@
 #include <string.h>
 #include <time.h>
 #include <stdio.h>
+#include "ios_state.h"
 
 #include "debug.h"
 
@@ -85,6 +86,8 @@ void JLDirectory::ParseTable(const Data &table_packet)
 
 void JLDirectory::Dump(std::ostream &os) const
 {
+	ios_format_state state(os);
+
 	int indent = m_level * 2;
 
 	os << setfill(' ') << setw(indent) << "";
@@ -150,6 +153,8 @@ void JLDirectoryEntry::Parse(uint16_t id, const Data &entry_packet)
 
 void JLDirectoryEntry::Dump(std::ostream &os) const
 {
+	ios_format_state state(os);
+
 	os << left << setfill(' ') << setw(50) << Name;
 
 	os << "\n";
@@ -282,6 +287,8 @@ std::string JLEventlogEntry::GetFormattedTimestamp() const
 
 void JLEventlogEntry::Dump(std::ostream &os) const
 {
+	ios_format_state state(os);
+
 	static const char *SeverityNames[] = { "Always Log", "Severe Error", "Error",
 		"Warning", "Information", "Debug Info"};
 	static const char *ViewerTypes[] = { "", "Number", "String", "Exception" };
@@ -300,11 +307,13 @@ void JLEventlogEntry::Dump(std::ostream &os) const
 
 void JLDeviceInfo::Dump(std::ostream &os) const
 {
+	ios_format_state state(os);
+
 	os << left << setfill(' ') << setw(17) << "Hardware Id:";
 	os << "0x" << hex << HardwareId << endl;
 
 	os << left << setfill(' ') << setw(17) << "PIN:";
-	os << "0x" << hex << Pin << endl;
+	os << "0x" << Pin.Str() << endl;
 
 	os << left << setfill(' ') << setw(17) << "OS Version:";
 	os << dec << OsVersion.Major << '.' << OsVersion.Minor << '.' << OsVersion.SubMinor << '.' << OsVersion.Build << endl;
@@ -318,7 +327,8 @@ void JLDeviceInfo::Dump(std::ostream &os) const
 	os << left << setfill(' ') << setw(17) << "Vendor ID:";
 	os << dec << VendorId << endl;
 
-	os << left << setfill(' ') << setw(17) << "Active WAFs:";
+	// WAF = Wireless Access Families
+	os << left << setfill(' ') << setw(17) << "Active Wireless Access Families:";
 	os << "0x" << hex << ActiveWafs << endl;
 
 	os << left << setfill(' ') << setw(17) << "OS Metrics:" << endl;
@@ -342,8 +352,16 @@ JavaLoader::JavaLoader(Controller &con)
 
 JavaLoader::~JavaLoader()
 {
-	if( m_StreamStarted )
-		StopStream();
+	try {
+		if( m_StreamStarted )
+			StopStream();
+	}
+	catch( std::exception &e ) {
+		dout("Exception ignored in ~JavaLoader(): " << e.what());
+	}
+	catch( ... ) {
+		dout("Unknown exception in ~JavaLoader()");
+	}
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -441,6 +459,10 @@ void JavaLoader::SendStream(std::istream &input, size_t module_size)
 		packet.PutData(buffer, size);
 		m_socket->Packet(packet);
 
+		if( packet.Command() == SB_COMMAND_JL_NOT_ENOUGH_MEMORY ) {
+			throw Error("JavaLoader::SendStream not enough memory to install the application");
+		}
+
 		if( packet.Command() != SB_COMMAND_JL_ACK ) {
 			ThrowJLError("JavaLoader::SendStream send data", packet.Command());
 		}
@@ -488,7 +510,7 @@ bool JavaLoader::StopStream()
 	m_StreamStarted = false;
 
 	if( packet.Command() == SB_COMMAND_JL_RESET_REQUIRED ) {
-		m_con.m_zero.SetResetOnClose(true);
+		m_socket->ResetOnClose(true);
 		return true;
 	}
 	else if( packet.Command() != SB_COMMAND_JL_ACK &&
@@ -836,10 +858,9 @@ void JavaLoader::SaveData(JLPacket &packet, uint16_t id, CodFileBuilder &builder
 			expect);
 
 		offset += expect;
+		buffer.ReleaseBuffer(offset);
 	}
 
-	buffer.ReleaseBuffer(offset);
-
 	builder.WriteNextHeader(output, buffer.GetData(), buffer.GetSize());
 	output.write((const char *)buffer.GetData(), buffer.GetSize());
 }
