@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2008-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2008-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -52,18 +52,16 @@ Serial::~Serial()
 //////////////////////////////////////////////////////////////////////////////
 // protected API / static functions
 
-void Serial::DataCallback(void *context, Data *data)
+void Serial::DataCallback(Serial &ser, Data *data)
 {
 	ddout("Serial::DataCallback called");
 
-	Serial *ser = (Serial*) context;
-
 	if( data->GetSize() <= 4 )
 		return;	// nothing to do
 
 	// call callback if available
-	if( ser->m_callback ) {
-		(*ser->m_callback)(ser->m_callback_context,
+	if( ser.m_callback ) {
+		(*ser.m_callback)(ser.m_callback_context,
 			data->GetData() + 4,
 			data->GetSize() - 4);
 	}
@@ -73,10 +71,8 @@ void Serial::DataCallback(void *context, Data *data)
 //	}
 }
 
-void Serial::CtrlCallback(void *context, Data *data)
+void Serial::CtrlCallback(Serial &ser, Data *data)
 {
-//	Serial *ser = (Serial*) context;
-
 	// just dump to stdout, and do nothing
 	ddout("CtrlCallback received:\n" << *data);
 }
@@ -99,19 +95,25 @@ void Serial::Open(const char *password)
 	}
 
 	m_ModeSocket = m_con.SelectMode(Controller::UsbSerData);
-	m_data = m_con.m_zero.Open(m_ModeSocket, password);
+	m_data = m_con.OpenSocket(m_ModeSocket, password);
 
 	m_CtrlSocket = m_con.SelectMode(Controller::UsbSerCtrl);
-	m_ctrl = m_con.m_zero.Open(m_CtrlSocket, password);
+	m_ctrl = m_con.OpenSocket(m_CtrlSocket, password);
 
 	// register callback for incoming data, for speed
-	m_data->RegisterInterest(DataCallback, this);
-	m_ctrl->RegisterInterest(CtrlCallback, this);
+	SocketRoutingQueue::SocketDataHandlerPtr data_callback
+		(new SocketRoutingQueue::SimpleSocketDataHandler<Serial>(*this, DataCallback));
+	m_data->UnregisterInterest();
+	m_data->RegisterInterest(data_callback);
+	SocketRoutingQueue::SocketDataHandlerPtr ctrl_callback
+		(new SocketRoutingQueue::SimpleSocketDataHandler<Serial>(*this, CtrlCallback));
+	m_ctrl->UnregisterInterest();
+	m_ctrl->RegisterInterest(ctrl_callback);
 
 	const unsigned char start[] =
 		{ 0, 0, 0x0a, 0, 0x01, 0x01, 0xc2, 0x00, 0x40, 0x00 };
 	Data block(start, sizeof(start));
-	m_ctrl->Send(block);
+	m_ctrl->RawSend(block);
 }
 
 void Serial::Close()
@@ -131,6 +133,7 @@ void Serial::RetryPassword(const char *password)
 	m_ctrl = m_con.m_zero.OpenDBSocket(m_CtrlSocket, password);
 
 	// register callback for incoming data, for speed
+	m_data->UnregisterInterest();
 	m_data->RegisterInterest(DataCallback, this);
 }
 */
@@ -160,7 +163,7 @@ void Serial::Write(const Data &data, int timeout)
 	spack->size = htobs(filtered.GetSize());
 
 	// send via appropriate socket
-	m_data->Send(filtered, timeout);
+	m_data->RawSend(filtered, timeout);
 }
 
 }} // namespace Barry::Mode
