@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2008-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2008-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -61,8 +61,8 @@ private:
 	void *m_callback_context;
 
 protected:
-	static void DataCallback(void *context, Data *data);
-	static void CtrlCallback(void *context, Data *data);
+	static void DataCallback(Serial &context, Data *data);
+	static void CtrlCallback(Serial &context, Data *data);
 
 public:
 	Serial(Controller &con, DeviceDataCallback callback, void *callback_context);
