@@ -6,7 +6,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -49,8 +49,8 @@ namespace Barry {
 ///
 unsigned int Packet::Command() const
 {
-	Protocol::CheckSize(m_receive, SB_PACKET_HEADER_SIZE);
-	MAKE_PACKET(rpack, m_receive);
+	Protocol::CheckSize(*m_receive, SB_PACKET_HEADER_SIZE);
+	MAKE_PACKET(rpack, *m_receive);
 	return rpack->command;
 }
 
@@ -130,53 +130,53 @@ void ZeroPacket::Reset()
 
 unsigned int ZeroPacket::ObjectID() const
 {
-	Protocol::CheckSize(m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
-	MAKE_PACKET(rpack, m_receive);
+	Protocol::CheckSize(*m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
+	MAKE_PACKET(rpack, *m_receive);
 	return btohs(rpack->u.socket.u.fetch.object);
 }
 
 unsigned int ZeroPacket::AttributeID() const
 {
-	Protocol::CheckSize(m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
-	MAKE_PACKET(rpack, m_receive);
+	Protocol::CheckSize(*m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
+	MAKE_PACKET(rpack, *m_receive);
 	return btohs(rpack->u.socket.u.fetch.attribute);
 }
 
 uint32_t ZeroPacket::ChallengeSeed() const
 {
-	Protocol::CheckSize(m_receive, SB_SOCKET_PACKET_HEADER_SIZE +
+	Protocol::CheckSize(*m_receive, SB_SOCKET_PACKET_HEADER_SIZE +
 		PASSWORD_CHALLENGE_SEED_SIZE);
-	MAKE_PACKET(rpack, m_receive);
+	MAKE_PACKET(rpack, *m_receive);
 	return btohl(rpack->u.socket.u.password.u.seed);
 }
 
 unsigned int ZeroPacket::RemainingTries() const
 {
-	Protocol::CheckSize(m_receive, SB_SOCKET_PACKET_HEADER_SIZE +
+	Protocol::CheckSize(*m_receive, SB_SOCKET_PACKET_HEADER_SIZE +
 		PASSWORD_CHALLENGE_HEADER_SIZE);
-	MAKE_PACKET(rpack, m_receive);
+	MAKE_PACKET(rpack, *m_receive);
 	// this is a byte, so no byte swapping needed
 	return rpack->u.socket.u.password.remaining_tries;
 }
 
 unsigned int ZeroPacket::SocketResponse() const
 {
-	Protocol::CheckSize(m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
-	MAKE_PACKET(rpack, m_receive);
+	Protocol::CheckSize(*m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
+	MAKE_PACKET(rpack, *m_receive);
 	return btohs(rpack->u.socket.socket);
 }
 
 unsigned char ZeroPacket::SocketSequence() const
 {
-	Protocol::CheckSize(m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
-	MAKE_PACKET(rpack, m_receive);
+	Protocol::CheckSize(*m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
+	MAKE_PACKET(rpack, *m_receive);
 	return rpack->u.socket.sequence;	// sequence is a byte
 }
 
 uint8_t ZeroPacket::CommandResponse() const
 {
-	Protocol::CheckSize(m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
-	MAKE_PACKET(rpack, m_receive);
+	Protocol::CheckSize(*m_receive, SB_SOCKET_PACKET_HEADER_SIZE);
+	MAKE_PACKET(rpack, *m_receive);
 	return rpack->command;
 }
 
@@ -359,13 +359,13 @@ void DBPacket::GetRecordByIndex(unsigned int dbId, unsigned int stateTableIndex)
 bool DBPacket::SetRecordByIndex(unsigned int dbId, unsigned int stateTableIndex,
 			      Builder &build, const IConverter *ic)
 {
-	// get new data if available
-	if( !build.Retrieve(dbId) )
-		return false;
-
 	// build packet data
+	DBData send(m_send, false);	// send is just a reference to m_send,
+					// so it is safe to use m_send later
+
 	size_t header_size = SB_PACKET_COMMAND_HEADER_SIZE + DBC_INDEXED_UPLOAD_HEADER_SIZE;
-	build.BuildFields(m_send, header_size, ic);
+	if( !build.BuildRecord(send, header_size, ic) )
+		return false;		// no data available
 	size_t total_size = m_send.GetSize();
 
 	// fill in the header values
@@ -411,25 +411,24 @@ void DBPacket::GetRecords(unsigned int dbId)
 }
 
 //
-// SetRecord
+// AddRecord
 //
-/// Builds a command packet in the m_send buffer for the SET_RECORD command
+/// Builds a command packet in the m_send buffer for the ADD_RECORD command
 /// code.
 ///
 /// \return	bool
 ///		- true means success
 ///		- false means no data available from Builder object
 ///
-bool DBPacket::SetRecord(unsigned int dbId, Builder &build, const IConverter *ic)
+bool DBPacket::AddRecord(unsigned int dbId, Builder &build, const IConverter *ic)
 {
-	// get new data if available
-	if( !build.Retrieve(dbId) )
-		return false;
-
 	// build packet data
+	DBData send(m_send, false);	// send is just a reference to m_send,
+					// so it is safe to use m_send later
+
 	size_t header_size = SB_PACKET_COMMAND_HEADER_SIZE + DBC_TAGGED_UPLOAD_HEADER_SIZE;
-	build.BuildHeader(m_send, header_size);
-	build.BuildFields(m_send, header_size, ic);
+	if( !build.BuildRecord(send, header_size, ic) )
+		return false;		// no data available
 	size_t total_size = m_send.GetSize();
 
 	// fill in the header values
@@ -440,15 +439,15 @@ bool DBPacket::SetRecord(unsigned int dbId, Builder &build, const IConverter *ic
 	packet.size = htobs(total_size);
 	packet.command = SB_COMMAND_DB_DATA;
 	packet.u.db.tableCmd = m_con.GetDBCommand(Mode::Desktop::DatabaseAccess);
-	packet.u.db.u.command.operation = SB_DBOP_SET_RECORD;
+	packet.u.db.u.command.operation = SB_DBOP_ADD_RECORD;
 	packet.u.db.u.command.databaseId = htobs(dbId);
-	packet.u.db.u.command.u.tag_upload.rectype = build.GetRecType();
-	packet.u.db.u.command.u.tag_upload.uniqueId = htobl(build.GetUniqueId());
+	packet.u.db.u.command.u.tag_upload.rectype = send.GetRecType();
+	packet.u.db.u.command.u.tag_upload.uniqueId = htobl(send.GetUniqueId());
 	packet.u.db.u.command.u.tag_upload.unknown2 = 1;	// unknown observed value
 
 	m_send.ReleaseBuffer(total_size);
 
-	m_last_dbop = SB_DBOP_SET_RECORD;
+	m_last_dbop = SB_DBOP_ADD_RECORD;
 	return true;
 }
 
@@ -457,8 +456,8 @@ bool DBPacket::SetRecord(unsigned int dbId, Builder &build, const IConverter *ic
 unsigned int DBPacket::ReturnCode() const
 {
 	if( Command() == SB_COMMAND_DB_DONE ) {
-		Protocol::CheckSize(m_receive, SB_PACKET_DBACCESS_HEADER_SIZE + SB_DBACCESS_RETURN_CODE_SIZE);
-		MAKE_PACKET(rpack, m_receive);
+		Protocol::CheckSize(*m_receive, SB_PACKET_DBACCESS_HEADER_SIZE + SB_DBACCESS_RETURN_CODE_SIZE);
+		MAKE_PACKET(rpack, *m_receive);
 		return rpack->u.db.u.return_code;
 	}
 	else {
@@ -475,8 +474,8 @@ unsigned int DBPacket::ReturnCode() const
 ///
 unsigned int DBPacket::DBOperation() const
 {
-	Protocol::CheckSize(m_receive, SB_PACKET_RESPONSE_HEADER_SIZE);
-	MAKE_PACKET(rpack, m_receive);
+	Protocol::CheckSize(*m_receive, SB_PACKET_RESPONSE_HEADER_SIZE);
+	MAKE_PACKET(rpack, *m_receive);
 	return rpack->u.db.u.response.operation;
 }
 
@@ -490,27 +489,59 @@ unsigned int DBPacket::DBOperation() const
 /// \returns	bool	true - packet was recognized and parse was attempted
 ///			false - packet was not recognized
 ///
-bool DBPacket::Parse(Parser &parser, const IConverter *ic)
+bool DBPacket::Parse(Parser &parser, const std::string &dbname,
+			const IConverter *ic)
 {
 	size_t offset = 0;
-	MAKE_PACKET(rpack, m_receive);
+	MAKE_PACKET(rpack, *m_receive);
 
 	switch( m_last_dbop )
 	{
 	case SB_DBOP_OLD_GET_RECORDS:
 	case SB_DBOP_GET_RECORD_BY_INDEX:
-		parser.Clear();
-
 		offset = SB_PACKET_RESPONSE_HEADER_SIZE + DBR_OLD_TAGGED_RECORD_HEADER_SIZE;
-		Protocol::CheckSize(m_receive, offset);
+		Protocol::CheckSize(*m_receive, offset);
+
 		// FIXME - this may need adjustment for email records... they
 		// don't seem to have uniqueID's
-		parser.SetIds(rpack->u.db.u.response.u.tagged.rectype,
-			btohl(rpack->u.db.u.response.u.tagged.uniqueId));
+		{
+			DBData block(DBData::REC_VERSION_1, dbname,
+				rpack->u.db.u.response.u.tagged.rectype,
+				btohl(rpack->u.db.u.response.u.tagged.uniqueId),
+				offset, *m_receive, false);
+			parser.ParseRecord(block, ic);
+		}
+		return true;
 
-		parser.ParseHeader(m_receive, offset);
-		parser.ParseFields(m_receive, offset, ic);
-		parser.Store();
+	default:	// unknown command
+		return false;
+	}
+}
+
+//
+// ParseMeta
+//
+/// Fills DBData's meta data based on its data block, and the last dbop.
+///
+bool DBPacket::ParseMeta(DBData &data)
+{
+	size_t offset = 0;
+	MAKE_PACKET(rpack, data.GetData());
+
+	switch( m_last_dbop )
+	{
+	case SB_DBOP_OLD_GET_RECORDS:
+	case SB_DBOP_GET_RECORD_BY_INDEX:
+		data.SetVersion(DBData::REC_VERSION_1);
+
+		offset = SB_PACKET_RESPONSE_HEADER_SIZE + DBR_OLD_TAGGED_RECORD_HEADER_SIZE;
+		Protocol::CheckSize(data.GetData(), offset);
+		data.SetOffset(offset);
+
+		// FIXME - this may need adjustment for email records... they
+		// don't seem to have uniqueID's
+		data.SetIds(rpack->u.db.u.response.u.tagged.rectype,
+			btohl(rpack->u.db.u.response.u.tagged.uniqueId));
 		return true;
 
 	default:	// unknown command
@@ -537,8 +568,8 @@ JLPacket::~JLPacket()
 
 unsigned int JLPacket::Size()
 {
-	Protocol::CheckSize(m_receive, SB_JLPACKET_HEADER_SIZE + SB_JLRESPONSE_HEADER_SIZE);
-	MAKE_JLPACKET(rpack, m_receive);
+	Protocol::CheckSize(*m_receive, SB_JLPACKET_HEADER_SIZE + SB_JLRESPONSE_HEADER_SIZE);
+	MAKE_JLPACKET(rpack, *m_receive);
 	return btohs(rpack->u.response.expect);
 }
 
@@ -654,5 +685,221 @@ int JLPacket::PutData(const void *data, uint16_t size)
 	return SimpleData(data, size);
 }
 
+
+//////////////////////////////////////////////////////////////////////////////
+// JVMPacket class
+
+JVMPacket::JVMPacket(Data &send, Data &receive)
+	: Packet(send, receive)
+	, m_cmd(send)
+{
+}
+
+JVMPacket::~JVMPacket()
+{
+}
+
+
+unsigned int JVMPacket::Size()
+{
+	MAKE_JVMPACKET(rpack, *m_receive);
+	Protocol::CheckSize(*m_receive, SB_JVMPACKET_HEADER_SIZE + sizeof(rpack->u.expect));
+	return be_btohs(rpack->u.expect);
+}
+
+
+// Command format (param is optionnal) :
+// 00000000: 05 00 07 00 00 01 8a
+//                             ^^ : command
+//                       ^^^^^ : size of commd + param
+//                 ^^^^^ : packet size
+//           ^^^^^ : socket ID
+void JVMPacket::SimpleCmd(uint8_t cmd)
+{
+	// 4 : socket id field + packet size field
+	// 2 : size field
+	// 1 : command field
+	const uint16_t total = 4 + 2 + 1;
+
+	MAKE_JVMPACKETPTR_BUF(cpack, m_cmd.GetBuffer(total));
+	Protocol::JVMPacket &packet = *cpack;
+
+	// socket class sets socket for us
+	packet.size = htobs(total);
+	packet.u.command.size = be_htobs(1);
+	packet.u.command.command = cmd;
+
+	m_cmd.ReleaseBuffer(total);
+}
+
+// Command with parameter format :
+// 00000000: 05 00 0b 00 00 05 8d 00 00 00 00
+//                                ^^^^^^^^^^^ : param
+//                             ^^ : command
+//                       ^^^^^ : size of commd + param
+//                 ^^^^^ : packet size
+//           ^^^^^ : socket ID
+void JVMPacket::ComplexCmd(uint8_t cmd, const void *param, uint16_t size)
+{
+	// 4 : socket id field + packet size field
+	// 2 : size field
+	// 1 : command field
+	uint16_t total = 4 + 2 + 1 + size;
+
+	MAKE_JVMPACKETPTR_BUF(cpack, m_cmd.GetBuffer(total));
+	Protocol::JVMPacket &packet = *cpack;
+
+	// socket class sets socket for us
+	packet.size = htobs(total);
+	packet.u.command.size = be_htobs(1 + size);
+	packet.u.command.command = cmd;
+
+	if ((size > 0) && (param != NULL))
+		memcpy(cpack->u.command.raw, param, size);
+
+	m_cmd.ReleaseBuffer(total);
+}
+
+
+void JVMPacket::Unknown01() {
+	SimpleCmd(SB_COMMAND_JVM_UNKNOWN01);
+}
+
+
+void JVMPacket::Unknown02() {
+	SimpleCmd(SB_COMMAND_JVM_UNKNOWN02);
+}
+
+
+void JVMPacket::Unknown03() {
+	SimpleCmd(SB_COMMAND_JVM_UNKNOWN03);
+}
+
+
+void JVMPacket::Unknown04() {
+	SimpleCmd(SB_COMMAND_JVM_UNKNOWN04);
+}
+
+
+void JVMPacket::Unknown05() {
+	SimpleCmd(SB_COMMAND_JVM_UNKNOWN05);
+}
+
+
+void JVMPacket::Unknown06() {
+	uint32_t param = 0;
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN06, &param, sizeof(param));
+}
+
+
+void JVMPacket::Unknown07() {
+	uint32_t param = 0;
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN07, &param, sizeof(param));
+}
+
+
+void JVMPacket::Unknown08() {
+	uint32_t param = 0;
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN08, &param, sizeof(param));
+}
+
+
+void JVMPacket::Unknown09() {
+	uint32_t param = be_htobl(0x09);
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN09, &param, sizeof(param));
+}
+
+
+void JVMPacket::Unknown10() {
+	uint32_t param = be_htobl(0x01);
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN10, &param, sizeof(param));
+}
+
+
+void JVMPacket::Unknown11(uint32_t id) {
+	id = be_htobl(id);
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN11, &id, sizeof(id));
+}
+
+
+void JVMPacket::Unknown12(uint32_t id) {
+	id = be_htobl(id);
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN12, &id, sizeof(id));
+}
+
+
+void JVMPacket::Unknown13(uint32_t id) {
+	id = be_htobl(id);
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN13, &id, sizeof(id));
+}
+
+
+void JVMPacket::Unknown14(uint32_t id) {
+	id = be_htobl(id);
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN14, &id, sizeof(id));
+}
+
+
+void JVMPacket::Unknown15(uint32_t id) {
+	id = be_htobl(id);
+
+	ComplexCmd(SB_COMMAND_JVM_UNKNOWN15, &id, sizeof(id));
+}
+
+
+void JVMPacket::GetModulesList(uint32_t id) {
+	id = be_htobl(id);
+
+	ComplexCmd(SB_COMMAND_JVM_GET_MODULES_LIST, &id, sizeof(id));
+}
+
+
+void JVMPacket::GetThreadsList() {
+	SimpleCmd(SB_COMMAND_JVM_GET_THREADS_LIST);
+}
+
+
+void JVMPacket::GetConsoleMessage() {
+	SimpleCmd(SB_COMMAND_JVM_GET_CONSOLE_MSG);
+}
+
+
+void JVMPacket::Go()
+{
+	SimpleCmd(SB_COMMAND_JVM_GO);
+}
+
+
+void JVMPacket::Stop()
+{
+	// 4 : socket id field + packet size field
+	// 2 : value field
+	const uint16_t total = 4 + 2;
+
+	MAKE_JVMPACKETPTR_BUF(cpack, m_cmd.GetBuffer(total));
+	Protocol::JVMPacket &packet = *cpack;
+
+	// socket class sets socket for us
+	packet.size = htobs(total);
+	packet.u.value = be_htobs(SB_COMMAND_JVM_STOP);
+
+	m_cmd.ReleaseBuffer(total);
+}
+
+
+void JVMPacket::GetStatus()
+{
+	SimpleCmd(SB_COMMAND_JVM_GET_STATUS);
+}
+
 } // namespace Barry
 
