@@ -6,7 +6,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,7 +26,9 @@
 
 #include <string>
 #include <stdint.h>
+#include <unistd.h>
 #include "protocol.h"
+#include "data.h"
 
 namespace Barry { class Data; }
 
@@ -46,20 +48,27 @@ namespace Mode {
 class Packet
 {
 	friend class SocketZero;
-	friend class Socket;
+	friend class SocketBase;
 
 protected:
-	Data &m_send, &m_receive;
+	Data &m_send;
+	Data *m_receive;
 
 	Data& GetSend() { return m_send; }
-	Data& GetReceive() { return m_receive; }
+	Data& GetReceive() { return *m_receive; }
 
 public:
 	Packet(Data &send, Data &receive)
-		: m_send(send), m_receive(receive)
+		: m_send(send), m_receive(&receive)
 		{}
 	virtual ~Packet() {}
 
+	// allow user to override the receive buffer for
+	// optimization purposes, to reduce copies... be
+	// careful with this, since this new Data object
+	// must outlive any usage of it via this Packet class
+	void SetNewReceive(Data &receive) { m_receive = &receive; }
+
 	//////////////////////////////////
 	// common response analysis
 
@@ -152,7 +161,7 @@ public:
 	void GetRecordByIndex(unsigned int dbId, unsigned int stateTableIndex);
 	bool SetRecordByIndex(unsigned int dbId, unsigned int stateTableIndex, Builder &build, const IConverter *ic);
 	void GetRecords(unsigned int dbId);
-	bool SetRecord(unsigned int dbId, Builder &build, const IConverter *ic);
+	bool AddRecord(unsigned int dbId, Builder &build, const IConverter *ic);
 
 
 	//////////////////////////////////
@@ -162,7 +171,9 @@ public:
 	unsigned int ReturnCode() const;	// throws FIXME if packet doesn't support it
 	unsigned int DBOperation() const; // throws Error on size trouble
 
-	bool Parse(Parser &parser, const IConverter *ic); // switches based on last m_send command
+	bool Parse(Parser &parser, const std::string &dbname,
+		const IConverter *ic); // switches based on last m_send command
+	bool ParseMeta(DBData &data);
 
 	// response parsers
 };
@@ -184,7 +195,7 @@ public:
 ///
 class JLPacket : public Packet
 {
-	friend class Socket;
+	friend class SocketBase;
 
 private:
 	Data &m_cmd, &m_data;
@@ -198,7 +209,7 @@ public:
 	// meta access
 
 	bool HasData() const	{ return m_last_set_size == 2; }
-	Data& GetReceive()	{ return m_receive; }
+	Data& GetReceive()	{ return *m_receive; }
 
 	//////////////////////////////////
 	// packet building
@@ -243,6 +254,74 @@ public:
 };
 
 
+//
+// JVMPacket class
+//
+/// Provides an API for building and analyzing raw JavaDebug protocol packets.
+/// This class relies on 3 external objects:
+/// a command send Data buffer (which can be fairly small), a data
+/// or argument send Data buffer, and a receive data buffer.  Socket and
+/// connection details are retrieved on a readonly basis from the
+/// Mode::JavaDebug object, but all buffers can be modified.
+///
+/// Note that the receive buffer may be modified
+/// during a packet send, and this JVMPacket class provides API helpers
+/// to analyze the results.
+///
+class JVMPacket : public Packet
+{
+	friend class SocketBase;
+
+private:
+	Data &m_cmd;
+
+public:
+	JVMPacket(Data &cmd, Data &receive);
+	~JVMPacket();
+
+	//////////////////////////////////
+	// meta access
+
+	Data& GetReceive()	{ return *m_receive; }
+
+	//////////////////////////////////
+	// packet building
+
+	// commands that correspond to the operation
+	// constants in protocol.h
+
+	// returns 1 or 2 depending on whether cmd or cmd+send are available
+	void SimpleCmd(uint8_t cmd);
+	void ComplexCmd(uint8_t cmd, const void *param, uint16_t size = 0);
+
+	void Unknown01();	// Command 0x53
+	void Unknown02();	// Command 0x01
+	void Unknown03();	// Command 0x6f
+	void Unknown04();	// Command 0x8a
+	void Unknown05();	// Command 0x90
+	void Unknown06();	// Command 0x44
+	void Unknown07();	// Command 0x45
+	void Unknown08();	// Command 0x54
+	void Unknown09();	// Command 0x33
+	void Unknown10();	// Command 0x46
+	void Unknown11(uint32_t id);	// Command 0x0e
+	void Unknown12(uint32_t id);	// Command 0x50
+	void Unknown13(uint32_t id);	// Command 0x0d
+	void Unknown14(uint32_t id);	// Command 0x85
+	void Unknown15(uint32_t id);	// Command 0x84
+	void GetModulesList(uint32_t id);	// Command 0x8d
+	void GetThreadsList();	// Command 0x08
+	void GetConsoleMessage();
+	void Go();	// Command mal formed :)
+	void Stop();	// Command 0x02
+	void GetStatus();	// Command 0x06
+
+	//////////////////////////////////
+	// response analysis
+	unsigned int Size();
+};
+
+
 } // namespace Barry
 
 #endif
