@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,11 +29,16 @@
 #include "packet.h"
 #include "socket.h"
 #include "protocol.h"
+#include "protostructs.h"
 #include "record-internal.h"
 #include "strnlen.h"
+#include "configfile.h"
+#include "platform.h"
 #include <iomanip>
+#include <sstream>
 #include <errno.h>
 #include <string.h>
+#include "ios_state.h"
 
 using namespace Usb;
 
@@ -56,8 +61,8 @@ namespace {
 
 	unsigned int GetSize(const unsigned char *packet)
 	{
-		uint16_t size = *((uint16_t *)&packet[2]);
-		return btohs(size);
+		const Protocol::Packet *pack = (const Protocol::Packet*) packet;
+		return btohs(pack->size);
 	}
 
 	bool Intro(int IntroIndex, const EndpointPair &ep, Device &dev, Data &response)
@@ -104,7 +109,8 @@ bool Probe::ParsePIN(const Data &data, uint32_t &pin)
 		return false;
 
 	// capture the PIN
-	pin = btohl(*((uint32_t *) &pd[16]));
+	memcpy(&pin, &pd[16], sizeof(pin));
+	pin = btohl(pin);
 
 	return true;
 }
@@ -123,8 +129,11 @@ bool Probe::ParseDesc(const Data &data, std::string &desc)
 }
 
 Probe::Probe(const char *busname, const char *devname,
-		const Usb::EndpointPair *epp)
-	: m_fail_count(0)
+		const Usb::EndpointPair *epp,
+		unsigned int log_exceptions,
+		bool auto_dump_log)
+	: m_log_exceptions(log_exceptions)
+	, m_fail_count(0)
 	, m_epp_override(epp)
 {
 	if( m_epp_override ) {
@@ -155,22 +164,47 @@ Probe::Probe(const char *busname, const char *devname,
 
 	// And one more time, for the Blackberry Storm
 	ProbeMatching(VENDOR_RIM, PRODUCT_RIM_STORM, busname, devname);
+
+	// now dump all logged error messages
+	if( auto_dump_log && m_fail_msgs.size() ) {
+		eout("Probe logged " << m_fail_msgs.size() << " exception messages:");
+		for( std::vector<std::string>::const_iterator b = m_fail_msgs.begin();
+			b != m_fail_msgs.end();
+			++b )
+		{
+			eout(*b);
+		}
+	}
 }
 
 void Probe::ProbeMatching(int vendor, int product,
 			const char *busname, const char *devname)
 {
-	Usb::DeviceIDType devid;
+	Usb::DeviceID devid;
 
-	Match match(vendor, product, busname, devname);
-	while( match.next_device(&devid) ) try {
+	Match match(m_devices, vendor, product, busname, devname);
+	while( match.next_device(devid) ) try {
 		ProbeDevice(devid);
 	}
 	catch( Usb::Error &e ) {
+		using namespace std;
+
 		dout("Usb::Error exception caught: " << e.what());
-		if( e.libusb_errcode() == -EBUSY ) {
+		if( ((m_log_exceptions & LOG_BUSY) && e.system_errcode() == -EBUSY) ||
+		    ((m_log_exceptions & LOG_ACCESS) && e.system_errcode() == -EACCES) ||
+		    ((m_log_exceptions & LOG_PERM) && e.system_errcode() == -EPERM) )
+		{
 			m_fail_count++;
-			m_fail_msgs.push_back(e.what());
+			string msg = "Device " + devid.GetUsbName() + ": ";
+			if( e.system_errcode() == -EBUSY )
+				msg += "(EBUSY) ";
+			else if( e.system_errcode() == -EACCES )
+				msg += "(EACCES) ";
+			else if( e.system_errcode() == -EPERM )
+				msg += "(EPERM) ";
+			msg += e.what();
+
+			m_fail_msgs.push_back(msg);
 		}
 		else {
 			throw;
@@ -178,40 +212,48 @@ void Probe::ProbeMatching(int vendor, int product,
 	}
 }
 
-void Probe::ProbeDevice(Usb::DeviceIDType devid)
+void Probe::ProbeDevice(Usb::DeviceID& devid)
 {
 	// skip if we can't properly discover device config
-	DeviceDiscovery discover(devid);
-	ConfigDesc &config = discover.configs[BLACKBERRY_CONFIGURATION];
+	DeviceDescriptor desc(devid);
+	ConfigDescriptor* config = desc[BLACKBERRY_CONFIGURATION];
+	if( !config ) {
+		dout("Probe: No device descriptor for BlackBerry config (config id: "
+			<< BLACKBERRY_CONFIGURATION << ")");
+		return;	// not found
+	}
 
 	// search for interface class
-	InterfaceDiscovery::base_type::iterator idi = config.interfaces.begin();
-	for( ; idi != config.interfaces.end(); idi++ ) {
-		if( idi->second.desc.bInterfaceClass == BLACKBERRY_DB_CLASS )
+	ConfigDescriptor::base_type::iterator idi = config->begin();
+	for( ; idi != config->end(); idi++ ) {
+		if( idi->second->GetClass() == BLACKBERRY_DB_CLASS )
 			break;
 	}
-	if( idi == config.interfaces.end() ) {
+	if( idi == config->end() ) {
 		dout("Probe: Interface with BLACKBERRY_DB_CLASS ("
 			<< BLACKBERRY_DB_CLASS << ") not found.");
 		return;	// not found
 	}
 
-	unsigned char InterfaceNumber = idi->second.desc.bInterfaceNumber;
-	dout("Probe: using InterfaceNumber: " << (unsigned int) InterfaceNumber);
+	unsigned char InterfaceNumber = idi->second->GetNumber();
+	unsigned char InterfaceAltSetting = idi->second->GetAltSetting();
+	dout("Probe: using InterfaceNumber: " << (unsigned int) InterfaceNumber <<
+	     " AltSetting: " << (unsigned int) InterfaceAltSetting);
 
 	// check endpoint validity
-	EndpointDiscovery &ed = config.interfaces[InterfaceNumber].endpoints;
-	if( !ed.IsValid() || ed.GetEndpointPairs().size() == 0 ) {
-		dout("Probe: endpoint invalid.   ed.IsValud() == "
-			<< (ed.IsValid() ? "true" : "false")
-			<< ", ed.GetEndpointPairs().size() == "
-			<< ed.GetEndpointPairs().size());
+	EndpointPairings ep(*(*config)[InterfaceNumber]);
+	if( !ep.IsValid() || ep.size() == 0 ) {
+		dout("Probe: endpoint invalid.   ep.IsValid() == "
+			<< (ep.IsValid() ? "true" : "false")
+			<< ", ep.size() == "
+			<< ep.size());
 		return;
 	}
 
 	ProbeResult result;
 	result.m_dev = devid;
 	result.m_interface = InterfaceNumber;
+	result.m_altsetting = InterfaceAltSetting;
 	result.m_zeroSocketSequence = 0;
 
 	// open device
@@ -224,7 +266,7 @@ void Probe::ProbeDevice(Usb::DeviceIDType devid)
 	if( !dev.GetConfiguration(cfg) )
 		throw Usb::Error(dev.GetLastError(),
 			"Probe: GetConfiguration failed");
-	if( cfg != BLACKBERRY_CONFIGURATION ) {
+	if( cfg != BLACKBERRY_CONFIGURATION || MUST_SET_CONFIGURATION ) {
 		if( !dev.SetConfiguration(BLACKBERRY_CONFIGURATION) )
 			throw Usb::Error(dev.GetLastError(),
 				"Probe: SetConfiguration failed");
@@ -233,61 +275,38 @@ void Probe::ProbeDevice(Usb::DeviceIDType devid)
 	// open interface
 	Interface iface(dev, InterfaceNumber);
 
-	if( m_epp_override ) {
-		// user has given us endpoints to try... so try them
-		uint32_t pin;
-		uint8_t zeroSocketSequence;
-		std::string desc;
-		if( ProbePair(dev, m_epp, pin, desc, zeroSocketSequence) ) {
-			// looks good, finish filling out the result
-			result.m_ep = m_epp;
-			result.m_pin = pin;
-			result.m_description = desc;
-			result.m_zeroSocketSequence = zeroSocketSequence;
-		}
-	}
-	else {
-		// find the first bulk read/write endpoint pair that answers
-		// to our probe commands
-		// Start with second pair, since evidence indicates the later pairs
-		// are the ones we need.
-		size_t i;
-		for(i = ed.GetEndpointPairs().size() > 1 ? 1 : 0;
-		    i < ed.GetEndpointPairs().size();
-		    i++ )
-		{
-			const EndpointPair &ep = ed.GetEndpointPairs()[i];
-			if( ep.type == USB_ENDPOINT_TYPE_BULK ) {
+	// Try the initial probing of endpoints
+	ProbeDeviceEndpoints(dev, ep, result);
 
-				uint32_t pin;
-				uint8_t zeroSocketSequence;
-				std::string desc;
-				if( ProbePair(dev, ep, pin, desc, zeroSocketSequence) ) {
-					result.m_ep = ep;
-					result.m_pin = pin;
-					result.m_description = desc;
-					result.m_zeroSocketSequence = zeroSocketSequence;
-					break;
-				}
-			}
-			else {
-				dout("Probe: Skipping non-bulk endpoint pair (offset: "
-					<< i-1 << ") ");
-			}
-		}
-
-		// check for ip modem endpoints
-		i++;
-		if( i < ed.GetEndpointPairs().size() ) {
-			const EndpointPair &ep = ed.GetEndpointPairs()[i];
-			if( ProbeModem(dev, ep) ) {
-				result.m_epModem = ep;
-			}
-		}
+	if( !result.m_ep.IsComplete() ) {
+		// Probing of end-points failed, so try reprobing
+		// after calling usb_set_altinterface().
+		//
+		// Calling usb_set_altinterface() should be harmless
+		// and can help the host and device to synchronize the
+		// USB state, especially on FreeBSD and Mac OS X.
+		// However it can cause usb-storage URBs to be lost
+		// on some devices, so is only used if necessary.
+		dout("Probe: probing endpoints failed, retrying after setting alternate interface");
+		
+		iface.SetAltInterface(InterfaceAltSetting);
+		result.m_needSetAltInterface = true;
+		ProbeDeviceEndpoints(dev, ep, result);
 	}
 
 	// add to list
 	if( result.m_ep.IsComplete() ) {
+		// before adding to list, try to load the device's
+		// friendly name from the configfile... but don't
+		// fail if we can't do it
+		try {
+			ConfigFile cfg(result.m_pin);
+			result.m_cfgDeviceName = cfg.GetDeviceName();
+		}
+		catch( Barry::ConfigFileError & ) {
+			// ignore...
+		}
+
 		m_results.push_back(result);
 		ddout("Using ReadEndpoint: " << (unsigned int)result.m_ep.read);
 		ddout("      WriteEndpoint: " << (unsigned int)result.m_ep.write);
@@ -297,20 +316,113 @@ void Probe::ProbeDevice(Usb::DeviceIDType devid)
 	}
 }
 
+void Probe::ProbeDeviceEndpoints(Device &dev, EndpointPairings &ed, ProbeResult &result)
+{
+	if( m_epp_override ) {
+		// user has given us endpoints to try... so try them
+		uint32_t pin;
+		uint8_t zeroSocketSequence;
+		std::string desc;
+		bool needClearHalt;
+		if( ProbePair(dev, m_epp, pin, desc, zeroSocketSequence, needClearHalt) ) {
+			// looks good, finish filling out the result
+			result.m_ep = m_epp;
+			result.m_pin = pin;
+			result.m_description = desc;
+			result.m_zeroSocketSequence = zeroSocketSequence;
+			result.m_needClearHalt = needClearHalt;
+		}
+	}
+	else {
+		// find the first bulk read/write endpoint pair that answers
+		// to our probe commands
+		// Start with second pair, since evidence indicates the later pairs
+		// are the ones we need.
+		size_t i;
+		for(i = ed.size() > 1 ? 1 : 0;
+		    i < ed.size();
+		    i++ )
+		{
+			const EndpointPair &ep = ed[i];
+			if( ep.type == Usb::EndpointDescriptor::BulkType ) {
+
+				uint32_t pin;
+				uint8_t zeroSocketSequence;
+				std::string desc;
+				bool needClearHalt;
+				if( ProbePair(dev, ep, pin, desc, zeroSocketSequence, needClearHalt) ) {
+					result.m_ep = ep;
+					result.m_pin = pin;
+					result.m_description = desc;
+					result.m_zeroSocketSequence = zeroSocketSequence;
+					result.m_needClearHalt = needClearHalt;
+					break;
+				}
+			}
+			else {
+				dout("Probe: Skipping non-bulk endpoint pair (offset: "
+				     << i-1 << ") ");
+			}
+		}
+
+		// check for ip modem endpoints
+		i++;
+		if( i < ed.size() ) {
+			const EndpointPair &ep = ed[i];
+			if( ProbeModem(dev, ep) ) {
+				result.m_epModem = ep;
+			}
+		}
+	}
+}
+
 bool Probe::ProbePair(Usb::Device &dev,
 			const Usb::EndpointPair &ep,
 			uint32_t &pin,
 			std::string &desc,
-			uint8_t &zeroSocketSequence)
+			uint8_t &zeroSocketSequence,
+			bool &needClearHalt)
 {
-	dev.ClearHalt(ep.read);
-	dev.ClearHalt(ep.write);
+	// Initially assume that clear halt isn't needed as it causes some
+	// devices to drop packets. The suspicion is that the toggle bits
+	// get out of sync, but this hasn't been confirmed with hardware
+	// tracing.
+	//
+	// It is possible to always use clear halt, as long as SET
+	// INTERFACE has been sent before, via usb_set_altinterface().
+	// However this has the side affect that any outstanding URBs
+	// on other interfaces (i.e. usb-storage) timeout and lose
+	// their data. This is not a good thing as it can corrupt the
+	// file system exposed over usb-storage. This also has the
+	// side-affect that usb-storage issues a port reset after the
+	// 30 second timeout, which kills any current Barry
+	// connection.
+	//
+	// To further complicate matters some devices, such as the
+	// 8830, always need clear halt before they will respond to
+	// probes.
+	//
+	// So to work with all these device quirks the probe is first
+	// attempted without calling clear halt. If that probe fails
+	// then a clear halt is issued followed by a retry on the
+	// probing.
+	needClearHalt = false;
 
 	Data data;
 	dev.BulkDrain(ep.read);
 	if( !Intro(0, ep, dev, data) ) {
-		dout("Probe: Intro(0) failed");
-		return false;
+		// Try clearing halt and then reprobing
+		dout("Probe: Intro(0) failed, retrying after clearing halt");
+		dev.ClearHalt(ep.read);
+		dev.ClearHalt(ep.write);
+		needClearHalt = true;
+		// Retry
+		dev.BulkDrain(ep.read);
+		if( !Intro(0, ep, dev, data) ) {
+			// Still no response so fail the probe
+			dout("Probe: Intro(0) still failed after clearing halt");
+			return false;
+		}
 	}
 
 	SocketZero socket(dev, ep.write, ep.read);
@@ -395,24 +507,39 @@ bool Probe::ProbeModem(Usb::Device &dev, const Usb::EndpointPair &ep)
 //	return false;
 }
 
-int Probe::FindActive(uint32_t pin) const
+int Probe::FindActive(Barry::Pin pin) const
 {
-	for( int i = 0; i < GetCount(); i++ ) {
-		if( Get(i).m_pin == pin )
-			return i;
-	}
-	if( pin == 0 ) {
+	return FindActive(m_results, pin);
+}
+
+int Probe::FindActive(const Barry::Probe::Results &results, Barry::Pin pin)
+{
+	int i = Find(results, pin);
+
+	if( i == -1 && pin == 0 ) {
 		// can we default to a single device?
-		if( GetCount() == 1 )
+		if( results.size() == 1 )
 			return 0;	// yes!
 	}
 
+	return i;
+}
+
+int Probe::Find(const Results &results, Barry::Pin pin)
+{
+	Barry::Probe::Results::const_iterator ci = results.begin();
+	for( int i = 0; ci != results.end(); i++, ++ci ) {
+		if( ci->m_pin == pin )
+			return i;
+	}
 	// PIN not found
 	return -1;
 }
 
 void ProbeResult::DumpAll(std::ostream &os) const
 {
+	ios_format_state state(os);
+
 	os << *this
 	   << ", Interface: 0x" << std::hex << (unsigned int) m_interface
 	   << ", Endpoints: (read: 0x" << std::hex << (unsigned int) m_ep.read
@@ -421,11 +548,24 @@ void ProbeResult::DumpAll(std::ostream &os) const
 	   << ", ZeroSocketSequence: 0x" << std::hex << (unsigned int) m_zeroSocketSequence;
 }
 
+std::string ProbeResult::GetDisplayName() const
+{
+	std::ostringstream oss;
+	oss << m_pin.Str();
+	if( m_cfgDeviceName.size() )
+		oss << " (" << m_cfgDeviceName << ")";
+	return oss.str();
+}
+
 std::ostream& operator<< (std::ostream &os, const ProbeResult &pr)
 {
-	os << "Device ID: " << pr.m_dev
-	   << std::hex << ". PIN: " << pr.m_pin
+	ios_format_state state(os);
+
+	os << "Device ID: " << pr.m_dev.m_impl.get()
+	   << ". PIN: " << pr.m_pin.Str()
 	   << ", Description: " << pr.m_description;
+	if( pr.m_cfgDeviceName.size() )
+		os << ", Name: " << pr.m_cfgDeviceName;
 	return os;
 }
 
