@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,17 +30,21 @@
 
 namespace Barry { namespace Protocol {
 
+// This function is only valid for Packet, JLPacket, and JVMPacket structs,
+// as long as they don't differ from each other in header layout, when
+// it comes to the .size field.  (see protostructs.h)
 void CheckSize(const Data &packet, size_t requiredsize)
 {
 	const Packet *p = (const Packet *) packet.GetData();
 
 	// when packets are larger than 0xFFFF bytes, packet->size is no
 	// longer reliable, so we go with the Data class size
-	if( (btohs(p->size) != packet.GetSize() && packet.GetSize() <= 0xFFFF) ||
+	if( (packet.GetSize() >= 4 && btohs(p->size) != packet.GetSize() && packet.GetSize() <= 0xFFFF) ||
 	    packet.GetSize() < requiredsize )
 
 	{
-		BadSize bs(btohs(p->size), packet.GetSize(), requiredsize);
+		BadSize bs(packet.GetSize() >= 4 ? btohs(p->size) : 0,
+			packet.GetSize(), requiredsize);
 		eout(bs.what());
 		eout(packet);
 		throw bs;
@@ -62,5 +66,18 @@ unsigned int GetSize(const Data &packet)
 	}
 }
 
+bool IsSequencePacket(const Barry::Data &data)
+{
+	if( data.GetSize() == SB_SEQUENCE_PACKET_SIZE ) {
+		MAKE_PACKET(rpack, data);
+		if( rpack->socket == 0 &&
+		    rpack->command == SB_COMMAND_SEQUENCE_HANDSHAKE )
+		{
+			return true;
+		}
+	}
+	return false;	// not a sequence packet
+}
+
 }} // namespace Barry::Protocol
 
