@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,9 +31,11 @@
 #define SB_COMMAND_FETCHED_ATTRIBUTE	0x06
 #define SB_COMMAND_SELECT_MODE		0x07
 #define SB_COMMAND_MODE_SELECTED	0x08
+#define SB_COMMAND_MODE_NOT_SELECTED	0x09
 #define SB_COMMAND_OPEN_SOCKET		0x0a
 #define SB_COMMAND_CLOSE_SOCKET		0x0b
 #define SB_COMMAND_CLOSED_SOCKET	0x0c
+#define SB_COMMAND_REMOTE_CLOSE_SOCKET	0x0d
 #define SB_COMMAND_PASSWORD_CHALLENGE	0x0e
 #define SB_COMMAND_PASSWORD		0x0f
 #define SB_COMMAND_OPENED_SOCKET	0x10
@@ -78,6 +80,7 @@
 #define SB_COMMAND_JL_RESET_REQUIRED	0x78	// Occurs after GOODBYE when handheld reset is required
 #define SB_COMMAND_JL_COD_IN_USE	0x6c	// Perhaps "BUSY" is also a good name?
 #define SB_COMMAND_JL_COD_NOT_FOUND	0x69
+#define SB_COMMAND_JL_NOT_ENOUGH_MEMORY 0x6a	// Occurs when the internal memory isn't enough to install the new COD file
 #define SB_COMMAND_JL_NOT_SUPPORTED	0x71	// Occurs when device does not support a command
 
 // JavaLoader data
@@ -85,6 +88,70 @@
 #define SB_DATA_JL_INVALID		0x68	// Device returns this code if the application isn't valid.
 
 
+// JDWP Command set list
+#define JDWP_CMDSET_VIRTUALMACHINE		1
+#define JDWP_CMDSET_REFERECENTYPE		2
+#define JDWP_CMDSET_CLASSTYPE			3
+#define JDWP_CMDSET_ARRAYTYPE			4
+#define JDWP_CMDSET_INTERFACETYPE		5
+#define JDWP_CMDSET_METHOD			6
+#define JDWP_CMDSET_FIELD			8
+#define JDWP_CMDSET_OBJECTREFERENCE		9
+#define JDWP_CMDSET_STRINGREFERENCE		10
+#define JDWP_CMDSET_THREADREFERENCE		11
+#define JDWP_CMDSET_THREADGROUPREFERENCE	12
+#define JDWP_CMDSET_ARRAYREFERENCE		13
+#define JDWP_CMDSET_CLASSLOADERREFERENCE	14
+#define JDWP_CMDSET_EVENTREQUEST		15
+#define JDWP_CMDSET_STACKFRAME			16
+#define JDWP_CMDSET_CLASSOBJECTREFERENCE	17
+#define JDWP_CMDSET_EVENT			64
+
+// JDWP Command list - VirtualMachine
+#define JDWP_CMD_VERSION			1
+#define JDWP_CMD_ALLCLASSES			3
+#define JDWP_CMD_ALLTHREADS			4
+#define JDWP_CMD_DISPOSE			6
+#define JDWP_CMD_IDSIZES			7
+#define JDWP_CMD_SUSPEND			8
+#define JDWP_CMD_RESUME				9
+#define JDWP_CMD_CLASSPATHS			13
+
+// JDWP Command list - EventRequest
+#define JDWP_CMD_SET				1
+
+// JVMDebug commands
+#define SB_COMMAND_JVM_UNKNOWN01	0x53
+#define SB_COMMAND_JVM_UNKNOWN02	0x01
+#define SB_COMMAND_JVM_UNKNOWN03	0x6f
+#define SB_COMMAND_JVM_UNKNOWN04	0x8a
+#define SB_COMMAND_JVM_UNKNOWN05	0x90
+#define SB_COMMAND_JVM_UNKNOWN06	0x44
+#define SB_COMMAND_JVM_UNKNOWN07	0x45
+#define SB_COMMAND_JVM_UNKNOWN08	0x54
+#define SB_COMMAND_JVM_UNKNOWN09	0x33
+#define SB_COMMAND_JVM_UNKNOWN10	0x46
+
+#define SB_COMMAND_JVM_UNKNOWN11	0x0e
+#define SB_COMMAND_JVM_UNKNOWN12	0x50
+#define SB_COMMAND_JVM_UNKNOWN13	0x0d
+#define SB_COMMAND_JVM_UNKNOWN14	0x85
+#define SB_COMMAND_JVM_UNKNOWN15	0x84
+
+#define SB_COMMAND_JVM_GET_MODULES_LIST	0x8d	// Get all Java modules list with their address and ID
+#define SB_COMMAND_JVM_GET_THREADS_LIST	0x08	// Get all threads currently running in the virtual machine
+#define SB_COMMAND_JVM_GET_CONSOLE_MSG	0x40	// Get console message
+#define SB_COMMAND_JVM_GO		0x02	// Go
+#define SB_COMMAND_JVM_GET_STATUS	0x06	// Get status
+#define SB_COMMAND_JVM_SET_BREAKPOINT	0x21	// Set breakpoint
+#define SB_COMMAND_JVM_RM_BREAKPOINT	0x22	// Remove breakpoint
+
+#define SB_COMMAND_JVM_STOP		0xa502
+
+// JavaDebug response
+#define SB_COMMAND_JVM_GET_DATA_ENTRY	0x06
+
+
 // mode constants
 #define SB_MODE_REQUEST_SOCKET		0x00ff
 
@@ -104,7 +171,7 @@
 
 
 // DB Operation Command
-#define SB_DBOP_SET_RECORD		0x41
+#define SB_DBOP_ADD_RECORD		0x41
 #define SB_DBOP_CLEAR_DATABASE		0x43
 #define SB_DBOP_GET_DBDB		0x4a
 #define SB_DBOP_OLD_GET_DBDB		0x4c
