@@ -12,7 +12,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +32,12 @@
 
 #include <stdint.h>
 #include <sys/types.h>
+#include "platform.h"		// safe to include platform.h here, since
+				// protostructs.h is not installed either
+
+#if USE_PACK_PRAGMA
+#pragma pack(push, 1)
+#endif
 
 // forward declarations
 namespace Barry { class Data; }
@@ -43,7 +49,7 @@ union SizePacket
 {
 	uint16_t size;
 	char buffer[4];
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -53,7 +59,7 @@ struct GroupLink				// used for Contacts records
 {
 	uint32_t	uniqueId;
 	uint16_t	unknown;
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 struct MessageAddress				// used for Message records
 {
@@ -61,7 +67,7 @@ struct MessageAddress				// used for Message records
 	uint8_t		addr[1];	// 2 null terminated strings: first
 					// contains full name, second contains
 					// the email address
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -90,8 +96,8 @@ struct SMSMetaData
 #define SMS_DCS_8BIT 0x01
 #define SMS_DCS_UCS2 0x02
 
-} __attribute__ ((packed));
-#define SMS_METADATA_SIZE	(sizeof(Barry::Protocol::SMSMetaData))
+} ATTRIBUTE_PACKED;
+#define SMS_METADATA_SIZE	(sizeof(::Barry::Protocol::SMSMetaData))
 
 
 
@@ -110,16 +116,17 @@ struct CommonField
 		MessageAddress	addr;
 		SMSMetaData	sms_metadata;
 		uint64_t	timestamp;
+		uint64_t	uint64;
 		uint32_t	uint32;
 		int32_t		min1900;
 		uint16_t	code;
 		uint8_t		raw[1];
 		int16_t		int16;
 
-	} __attribute__ ((packed)) u;
+	} ATTRIBUTE_PACKED u;
 
-} __attribute__ ((packed));
-#define COMMON_FIELD_HEADER_SIZE	(sizeof(Barry::Protocol::CommonField) - sizeof(Barry::Protocol::CommonField::CommonFieldData))
+} ATTRIBUTE_PACKED;
+#define COMMON_FIELD_HEADER_SIZE	(sizeof(::Barry::Protocol::CommonField) - sizeof(::Barry::Protocol::CommonField::CommonFieldData))
 #define COMMON_FIELD_MIN1900_SIZE	(sizeof(int32_t))
 
 struct CommandTableField
@@ -127,8 +134,8 @@ struct CommandTableField
 	uint8_t		size;		// no null terminator
 	uint8_t		code;
 	uint8_t		name[1];
-} __attribute__ ((packed));
-#define COMMAND_FIELD_HEADER_SIZE	(sizeof(Barry::Protocol::CommandTableField) - 1)
+} ATTRIBUTE_PACKED;
+#define COMMAND_FIELD_HEADER_SIZE	(sizeof(::Barry::Protocol::CommandTableField) - 1)
 
 struct OldDBDBField
 {
@@ -140,8 +147,8 @@ struct OldDBDBField
 	uint16_t	unknown2;
 	uint16_t	nameSize;		// includes null terminator
 	uint8_t		name[1];
-} __attribute__ ((packed));
-#define OLD_DBDB_FIELD_HEADER_SIZE	(sizeof(Barry::Protocol::OldDBDBField) - 1)
+} ATTRIBUTE_PACKED;
+#define OLD_DBDB_FIELD_HEADER_SIZE	(sizeof(::Barry::Protocol::OldDBDBField) - 1)
 
 struct DBDBField
 {
@@ -157,8 +164,8 @@ struct DBDBField
 	uint16_t	unknown;		// this comes after the
 						// null terminated name, but
 						// is here for size calcs
-} __attribute__ ((packed));
-#define DBDB_FIELD_HEADER_SIZE	(sizeof(Barry::Protocol::DBDBField) - 1)
+} ATTRIBUTE_PACKED;
+#define DBDB_FIELD_HEADER_SIZE	(sizeof(::Barry::Protocol::DBDBField) - 1)
 
 struct RecordStateTableField
 {
@@ -176,7 +183,7 @@ struct RecordStateTableField
 						// are used
 #define BARRY_RSTF_DIRTY	0x01
 	uint8_t		unknown2[4];
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 struct CalendarRecurrenceDataField  // as documented in the Cassis project spec
 {
@@ -200,14 +207,14 @@ struct CalendarRecurrenceDataField  // as documented in the Cassis project spec
 		struct Day
 		{
 			uint8_t day[6];		// always zeros!
-		} __attribute__ ((packed)) day;
+		} ATTRIBUTE_PACKED day;
 
 		struct MonthByDate
 		{
 			uint8_t monthDay;	// day of month to recur on
 						// (1-31)
 			uint8_t blank[5];
-		} __attribute__ ((packed)) month_by_date;
+		} ATTRIBUTE_PACKED month_by_date;
 
 		struct MonthByDay
 		{
@@ -216,7 +223,7 @@ struct CalendarRecurrenceDataField  // as documented in the Cassis project spec
 						// (1 to 5, first week, second
 						// week, etc)
 			uint8_t blank[4];
-		} __attribute__ ((packed)) month_by_day;
+		} ATTRIBUTE_PACKED month_by_day;
 
 		struct YearByDate
 		{
@@ -225,7 +232,7 @@ struct CalendarRecurrenceDataField  // as documented in the Cassis project spec
 			uint8_t blank;
 			uint8_t month;		// month to recur on (1-12)
 			uint8_t blank_[3];
-		} __attribute__ ((packed)) year_by_date;
+		} ATTRIBUTE_PACKED year_by_date;
 
 		struct YearByDay
 		{
@@ -233,7 +240,7 @@ struct CalendarRecurrenceDataField  // as documented in the Cassis project spec
 			uint8_t week;		// week of month (1 to 5)
 			uint8_t month;		// (1-12)
 			uint8_t blank[3];
-		} __attribute__ ((packed)) year_by_day;
+		} ATTRIBUTE_PACKED year_by_day;
 
 		struct Week
 		{
@@ -247,12 +254,12 @@ struct CalendarRecurrenceDataField  // as documented in the Cassis project spec
 			#define CRDF_WD_SAT	0x40
 
 			uint8_t blank[5];
-		} __attribute__ ((packed)) week;
+		} ATTRIBUTE_PACKED week;
 
-	} __attribute__ ((packed)) u;
+	} ATTRIBUTE_PACKED u;
 
-} __attribute__ ((packed));
-#define CALENDAR_RECURRENCE_DATA_FIELD_SIZE	sizeof(Barry::Protocol::CalendarRecurrenceDataField)
+} ATTRIBUTE_PACKED;
+#define CALENDAR_RECURRENCE_DATA_FIELD_SIZE	sizeof(::Barry::Protocol::CalendarRecurrenceDataField)
 
 //
 // Calendar record: field constants
@@ -308,11 +315,67 @@ struct CalendarRecurrenceDataField  // as documented in the Cassis project spec
 
 #define CLL_PHONETYPE_UNDEFINED		0
 #define CLL_PHONETYPE_OFFICE		1
-#define CLL_PHONETYPE_HOME			2
+#define CLL_PHONETYPE_HOME		2
 #define CLL_PHONETYPE_MOBILE		3
 #define CLL_PHONETYPE_RANGE_LOW		0
 #define CLL_PHONETYPE_RANGE_HIGH	3
 
+//
+// Browser Bookmarks record: field constants
+//
+//
+#define BMK_BROWSER_AUTO		0
+#define BMK_BROWSER_BLACKBERRY		1
+#define BMK_BROWSER_FIREFOX		2
+#define BMK_BROWSER_INTERNET_EXPLORER	3
+
+#define BMK_DISPLAY_AUTO		0
+#define BMK_DISPLAY_COLOMN		1
+#define BMK_DISPLAY_PAGE		2
+
+#define BMK_JAVASCRIPT_AUTO		0
+#define BMK_JAVASCRIPT_ENABLED		1
+#define BMK_JAVASCRIPT_DISABLED		2
+
+struct StringField
+{
+	uint16_t	be_size;	// may or may not have a null term.
+					// big-endian
+	uint8_t		data[1];
+} ATTRIBUTE_PACKED;
+#define STRING_FIELD_HEADER_SIZE	(sizeof(::Barry::Protocol::StringField) - 1)
+
+struct BookmarkId
+{
+	uint32_t	bookmark_id;
+	uint8_t		index;
+} ATTRIBUTE_PACKED;
+#define BOOKMARK_ID_SIZE		(sizeof(::Barry::Protocol::BookmarkId))
+
+struct VarStringField
+{
+	uint8_t		present;
+	uint16_t	be_size;	// big-endian
+	uint8_t		data[1];
+} ATTRIBUTE_PACKED;
+#define VARSTRING_FIELD_HEADER_SIZE	(sizeof(::Barry::Protocol::VarStringField) - 1)
+
+struct BookmarkFolders
+{
+	uint32_t	id1;		// these two fields contain
+	uint32_t	id2;		// b9 fc f8 f6 c2 e3 a4 d5
+					// whenever there is a record in
+					// the Blackberry Bookmarks folder,
+					// even across multiple devices!
+					// this was true for the 9550 and
+					// the oooold 7750, and the 8700
+	uint8_t		unknown;	// may be a section code?  seems to
+					// always be 0x08
+	uint8_t		flag;		// unknown flag
+} ATTRIBUTE_PACKED;
+#define BOOKMARK_FOLDERS_HEADER_SIZE	(sizeof(::Barry::Protocol::BookmarkFolders))
+
+
 //
 // Folder record: field constants
 //
@@ -339,16 +402,16 @@ struct PackedField_02
 	uint8_t		size;
 	uint8_t		type;
 	uint8_t		raw[1];
-} __attribute__ ((packed));
-#define PACKED_FIELD_02_HEADER_SIZE	(sizeof(Barry::Protocol::PackedField_02) - 1)
+} ATTRIBUTE_PACKED;
+#define PACKED_FIELD_02_HEADER_SIZE	(sizeof(::Barry::Protocol::PackedField_02) - 1)
 
 struct PackedField_10
 {
 	uint8_t		type;
 	uint8_t		size;
 	uint8_t		raw[1];
-} __attribute__ ((packed));
-#define PACKED_FIELD_10_HEADER_SIZE	(sizeof(Barry::Protocol::PackedField_10) - 1)
+} ATTRIBUTE_PACKED;
+#define PACKED_FIELD_10_HEADER_SIZE	(sizeof(::Barry::Protocol::PackedField_10) - 1)
 
 
 
@@ -360,8 +423,8 @@ struct ServiceBookConfigField
 {
 	uint8_t		format;
 	uint8_t		fields[1];
-} __attribute__ ((packed));
-#define SERVICE_BOOK_CONFIG_FIELD_HEADER_SIZE (sizeof(Barry::Protocol::ServiceBookConfigField) - 1)
+} ATTRIBUTE_PACKED;
+#define SERVICE_BOOK_CONFIG_FIELD_HEADER_SIZE (sizeof(::Barry::Protocol::ServiceBookConfigField) - 1)
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -371,16 +434,16 @@ struct DBC_Record
 {
 	uint16_t	recordIndex;	// index comes from RecordStateTable
 	uint8_t		data[1];
-} __attribute__ ((packed));
-#define DBC_RECORD_HEADER_SIZE		(sizeof(Barry::Protocol::DBC_Record) - 1)
+} ATTRIBUTE_PACKED;
+#define DBC_RECORD_HEADER_SIZE		(sizeof(::Barry::Protocol::DBC_Record) - 1)
 
 struct DBC_RecordFlags
 {
 	uint8_t		unknown;
 	uint16_t	index;
 	uint8_t		unknown2[5];
-} __attribute__ ((packed));
-#define DBC_RECORD_FLAGS_SIZE		(sizeof(Barry::Protocol::DBC_RecordFlags))
+} ATTRIBUTE_PACKED;
+#define DBC_RECORD_FLAGS_SIZE		(sizeof(::Barry::Protocol::DBC_RecordFlags))
 
 struct DBC_TaggedUpload
 {
@@ -393,16 +456,16 @@ struct DBC_TaggedUpload
 	uint32_t	uniqueId;
 	uint8_t		unknown2;
 	uint8_t		data[1];
-} __attribute__ ((packed));
-#define DBC_TAGGED_UPLOAD_HEADER_SIZE	(sizeof(Barry::Protocol::DBC_TaggedUpload) - 1)
+} ATTRIBUTE_PACKED;
+#define DBC_TAGGED_UPLOAD_HEADER_SIZE	(sizeof(::Barry::Protocol::DBC_TaggedUpload) - 1)
 
 struct DBC_IndexedUpload
 {
 	uint8_t		unknown;	// observed: 00 or 05
 	uint16_t	index;
 	uint8_t		data[1];
-} __attribute__ ((packed));
-#define DBC_INDEXED_UPLOAD_HEADER_SIZE	(sizeof(Barry::Protocol::DBC_IndexedUpload) - 1)
+} ATTRIBUTE_PACKED;
+#define DBC_INDEXED_UPLOAD_HEADER_SIZE	(sizeof(::Barry::Protocol::DBC_IndexedUpload) - 1)
 
 struct PasswordChallenge
 {
@@ -424,20 +487,20 @@ struct PasswordChallenge
 	{
 		uint32_t	seed;
 		uint8_t		hash[20];
-	} __attribute__ ((packed)) u;
+	} ATTRIBUTE_PACKED u;
 
-} __attribute__ ((packed));
-#define PASSWORD_CHALLENGE_HEADER_SIZE	(sizeof(Barry::Protocol::PasswordChallenge) - sizeof(Barry::Protocol::PasswordChallenge::Hash))
+} ATTRIBUTE_PACKED;
+#define PASSWORD_CHALLENGE_HEADER_SIZE	(sizeof(::Barry::Protocol::PasswordChallenge) - sizeof(::Barry::Protocol::PasswordChallenge::Hash))
 #define PASSWORD_CHALLENGE_SEED_SIZE	(PASSWORD_CHALLENGE_HEADER_SIZE + sizeof(uint32_t))
-#define PASSWORD_CHALLENGE_SIZE		(sizeof(Barry::Protocol::PasswordChallenge))
+#define PASSWORD_CHALLENGE_SIZE		(sizeof(::Barry::Protocol::PasswordChallenge))
 
 struct AttributeFetch
 {
 	uint16_t	object;
 	uint16_t	attribute;
 	uint8_t		raw[1];			// used only in response
-} __attribute__ ((packed));
-#define ATTRIBUTE_FETCH_COMMAND_SIZE	(sizeof(Barry::Protocol::AttributeFetch) - 1)
+} ATTRIBUTE_PACKED;
+#define ATTRIBUTE_FETCH_COMMAND_SIZE	(sizeof(::Barry::Protocol::AttributeFetch) - 1)
 
 struct ModeSelect
 {
@@ -445,15 +508,15 @@ struct ModeSelect
 	struct ResponseBlock
 	{
 		uint8_t		unknown[20];
-	} __attribute__ ((packed)) response;
-} __attribute__ ((packed));
+	} ATTRIBUTE_PACKED response;
+} ATTRIBUTE_PACKED;
 
 struct Echo
 {
 	uint64_t	ticks;			// number of microseconds since
 						// host system startup
-} __attribute__ ((packed));
-#define ECHO_COMMAND_SIZE		(sizeof(Barry::Protocol::Echo))
+} ATTRIBUTE_PACKED;
+#define ECHO_COMMAND_SIZE		(sizeof(::Barry::Protocol::Echo))
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -475,17 +538,17 @@ struct SocketCommand
 		uint8_t			raw[1];
 		Echo			echo;
 
-	} __attribute__ ((packed)) u;
-} __attribute__ ((packed));
-#define SOCKET_COMMAND_HEADER_SIZE		(sizeof(Barry::Protocol::SocketCommand) - sizeof(Barry::Protocol::SocketCommand::PacketData))
+	} ATTRIBUTE_PACKED u;
+} ATTRIBUTE_PACKED;
+#define SOCKET_COMMAND_HEADER_SIZE		(sizeof(::Barry::Protocol::SocketCommand) - sizeof(::Barry::Protocol::SocketCommand::PacketData))
 
 struct SequenceCommand
 {
-	uint8_t		unknown1;
+	uint8_t		socket;			// socket number, single byte
 	uint8_t		unknown2;
 	uint8_t		unknown3;
 	uint32_t	sequenceId;
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 struct DBCommand
 {
@@ -501,9 +564,9 @@ struct DBCommand
 		DBC_IndexedUpload	index_upload;
 		uint8_t			raw[1];
 
-	} __attribute__ ((packed)) u;
-} __attribute__ ((packed));
-#define DB_COMMAND_HEADER_SIZE		(sizeof(Barry::Protocol::DBCommand) - sizeof(Barry::Protocol::DBCommand::Parameters))
+	} ATTRIBUTE_PACKED u;
+} ATTRIBUTE_PACKED;
+#define DB_COMMAND_HEADER_SIZE		(sizeof(::Barry::Protocol::DBCommand) - sizeof(::Barry::Protocol::DBCommand::Parameters))
 
 
 
@@ -514,16 +577,16 @@ struct DBR_OldDBDBRecord
 {
 	uint16_t	count;			// number of fields in record
 	OldDBDBField	field[1];
-} __attribute__ ((packed));
-#define OLD_DBDB_RECORD_HEADER_SIZE	(sizeof(Barry::Protocol::DBR_OldDBDBRecord) - sizeof(Barry::Protocol::OldDBDBField))
+} ATTRIBUTE_PACKED;
+#define OLD_DBDB_RECORD_HEADER_SIZE	(sizeof(::Barry::Protocol::DBR_OldDBDBRecord) - sizeof(::Barry::Protocol::OldDBDBField))
 
 struct DBR_DBDBRecord
 {
 	uint16_t	count;
 	uint8_t		unknown[3];
 	DBDBField	field[1];
-} __attribute__ ((packed));
-#define DBDB_RECORD_HEADER_SIZE		(sizeof(Barry::Protocol::DBR_DBDBRecord) - sizeof(Barry::Protocol::DBDBField))
+} ATTRIBUTE_PACKED;
+#define DBDB_RECORD_HEADER_SIZE		(sizeof(::Barry::Protocol::DBR_DBDBRecord) - sizeof(::Barry::Protocol::DBDBField))
 
 // Records with a uniqueId.  This covers the following records:
 //
@@ -541,9 +604,9 @@ struct DBR_OldTaggedRecord
 	union TaggedData
 	{
 		CommonField	field[1];
-	} __attribute__ ((packed)) u;
-} __attribute__ ((packed));
-#define DBR_OLD_TAGGED_RECORD_HEADER_SIZE (sizeof(Barry::Protocol::DBR_OldTaggedRecord) - sizeof(Barry::Protocol::DBR_OldTaggedRecord::TaggedData))
+	} ATTRIBUTE_PACKED u;
+} ATTRIBUTE_PACKED;
+#define DBR_OLD_TAGGED_RECORD_HEADER_SIZE (sizeof(::Barry::Protocol::DBR_OldTaggedRecord) - sizeof(::Barry::Protocol::DBR_OldTaggedRecord::TaggedData))
 
 struct MessageRecord
 {
@@ -599,8 +662,8 @@ struct MessageRecord
 	uint8_t		field33;	// always 'd'
 	uint32_t	timeBlock;	// FIXME
 	CommonField	field[1];
-} __attribute__ ((packed));
-#define MESSAGE_RECORD_HEADER_SIZE (sizeof(Barry::Protocol::MessageRecord) - sizeof(Barry::Protocol::CommonField))
+} ATTRIBUTE_PACKED;
+#define MESSAGE_RECORD_HEADER_SIZE (sizeof(::Barry::Protocol::MessageRecord) - sizeof(::Barry::Protocol::CommonField))
 
 
 
@@ -618,10 +681,10 @@ struct DBResponse
 		DBR_OldDBDBRecord	old_dbdb;
 		DBR_DBDBRecord		dbdb;
 
-	} __attribute__ ((packed)) u;
+	} ATTRIBUTE_PACKED u;
 
-} __attribute__ ((packed));
-#define DB_RESPONSE_HEADER_SIZE		(sizeof(Barry::Protocol::DBResponse) - sizeof(Barry::Protocol::DBResponse::Parameters))
+} ATTRIBUTE_PACKED;
+#define DB_RESPONSE_HEADER_SIZE		(sizeof(::Barry::Protocol::DBResponse) - sizeof(::Barry::Protocol::DBResponse::Parameters))
 
 
 
@@ -641,9 +704,9 @@ struct DBAccess
 		uint8_t			return_code;
 		uint8_t			fragment[1];
 
-	} __attribute__ ((packed)) u;
-} __attribute__ ((packed));
-#define SB_DBACCESS_HEADER_SIZE			(sizeof(Barry::Protocol::DBAccess) - sizeof(Barry::Protocol::DBAccess::DBData))
+	} ATTRIBUTE_PACKED u;
+} ATTRIBUTE_PACKED;
+#define SB_DBACCESS_HEADER_SIZE			(sizeof(::Barry::Protocol::DBAccess) - sizeof(::Barry::Protocol::DBAccess::DBData))
 #define SB_DBACCESS_RETURN_CODE_SIZE		(1)
 
 
@@ -661,24 +724,24 @@ struct JLDirEntry
 	// another string for version, then:
 	// uint32_t	cod_size;
 
-} __attribute__ ((packed));
-#define SB_JLDIRENTRY_HEADER_SIZE		(sizeof(Barry::Protocol::JLDirEntry) - 1)
+} ATTRIBUTE_PACKED;
+#define SB_JLDIRENTRY_HEADER_SIZE		(sizeof(::Barry::Protocol::JLDirEntry) - 1)
 
 struct JLCommand
 {
 	uint8_t		command;
 	uint8_t		unknown;	// nearly always 0, might be top half of command
 	uint16_t	size;
-} __attribute__ ((packed));
-#define SB_JLCOMMAND_HEADER_SIZE		(sizeof(Barry::Protocol::JLCommand))
+} ATTRIBUTE_PACKED;
+#define SB_JLCOMMAND_HEADER_SIZE		(sizeof(::Barry::Protocol::JLCommand))
 
 struct JLResponse
 {
 	uint8_t		command;
 	uint8_t		unknown;
 	uint16_t	expect;
-} __attribute__ ((packed));
-#define SB_JLRESPONSE_HEADER_SIZE		(sizeof(Barry::Protocol::JLResponse))
+} ATTRIBUTE_PACKED;
+#define SB_JLRESPONSE_HEADER_SIZE		(sizeof(::Barry::Protocol::JLResponse))
 
 struct JLScreenInfo
 {
@@ -690,16 +753,16 @@ struct JLScreenInfo
 	uint16_t	unknown4;
 	uint16_t	unknown5;
 	uint16_t	unknown6;
-} __attribute__ ((packed));
-#define SB_JLSCREENINFO_SIZE			(sizeof(Barry::Protocol::JLScreenInfo))
+} ATTRIBUTE_PACKED;
+#define SB_JLSCREENINFO_SIZE			(sizeof(::Barry::Protocol::JLScreenInfo))
 
 struct JLEventlogEntry
 {
 	uint16_t	size;
 	// remainder of packet is variable
 	// it contains the log data as an ASCII (UTF-8?) string
-} __attribute__ ((packed));
-#define SB_JLEVENTLOG_ENTRY_HEADER_SIZE		(sizeof(Barry::Protocol::JLEventlogEntry))
+} ATTRIBUTE_PACKED;
+#define SB_JLEVENTLOG_ENTRY_HEADER_SIZE		(sizeof(::Barry::Protocol::JLEventlogEntry))
 
 struct JLDeviceInfo
 {
@@ -713,8 +776,8 @@ struct JLDeviceInfo
 	// older devices (such as 7130) don't this extra data in the
 	// device info packet and will therefore fail the size check
 	//uint8_t		raw[4];
-} __attribute__ ((packed));
-#define SB_JLDEVICEINFO_SIZE			(sizeof(Barry::Protocol::JLDeviceInfo))
+} ATTRIBUTE_PACKED;
+#define SB_JLDEVICEINFO_SIZE			(sizeof(::Barry::Protocol::JLDeviceInfo))
 
 struct JLPacket
 {
@@ -733,10 +796,205 @@ struct JLPacket
 		uint32_t		cod_size;
 		uint32_t		timestamp;
 		uint16_t		id;
-	} __attribute__ ((packed)) u;
+	} ATTRIBUTE_PACKED u;
 
-} __attribute__ ((packed));
-#define SB_JLPACKET_HEADER_SIZE		(sizeof(Barry::Protocol::JLPacket) - sizeof(Barry::Protocol::JLPacket::PacketData))
+} ATTRIBUTE_PACKED;
+#define SB_JLPACKET_HEADER_SIZE		(sizeof(::Barry::Protocol::JLPacket) - sizeof(::Barry::Protocol::JLPacket::PacketData))
+
+
+///////////////////////////////////////////////////////////////////////////////
+// JavaDebug protocol structures
+
+namespace JDWP {
+
+	// Packet command
+	//----------------
+
+	struct PacketEventRequestSet {
+		uint8_t eventKind;
+		uint8_t suspendPolicy;
+		uint32_t modifiers;
+	} ATTRIBUTE_PACKED;
+
+
+	struct PacketEventRequest {
+		union PacketEventRequestData {
+			PacketEventRequestSet set;
+		} ATTRIBUTE_PACKED u;
+	} ATTRIBUTE_PACKED;
+
+
+	struct PacketCommand {
+		uint8_t commandset;
+		uint8_t command;
+
+		union PacketCommandData {
+			PacketEventRequest eventRequest;
+		} ATTRIBUTE_PACKED u;
+	} ATTRIBUTE_PACKED;
+	#define JDWP_COMMAND_HEADER_SIZE			(sizeof(::Barry::Protocol::JDWP::PacketCommand))
+
+
+	// Packet response
+	//-----------------
+
+	struct PacketVirtualMachineIDSizes {
+		uint32_t fieldIDSize;
+		uint32_t methodIDSize;
+		uint32_t objectIDSize;
+		uint32_t referenceTypeIDSize;
+		uint32_t frameIDSize;
+	} ATTRIBUTE_PACKED;
+
+	#define JDWP_PACKETVIRTUALMACHINEIDSIZES_DATA_SIZE		sizeof(::Barry::Protocol::JDWP::PacketVirtualMachineIDSizes)
+
+
+	struct PacketVirtualMachine {
+		union PacketVirtualMachineData {
+			PacketVirtualMachineIDSizes IDSizes;
+		} ATTRIBUTE_PACKED u;
+	} ATTRIBUTE_PACKED;
+
+
+	struct PacketResponse {
+		uint16_t errorcode;
+
+		union PacketResponseData {
+			PacketVirtualMachine virtualMachine;
+			uint32_t value;
+			uint8_t raw[1];
+		} ATTRIBUTE_PACKED u;
+	} ATTRIBUTE_PACKED;
+	#define JDWP_RESPONSE_HEADER_SIZE			(sizeof(::Barry::Protocol::JDWP::PacketResponse) - sizeof(::Barry::Protocol::JDWP::PacketResponse::PacketResponseData))
+
+
+	// Generic packet
+	//----------------
+
+	struct Packet {
+		uint32_t length;
+		uint32_t id;
+		uint8_t flags;
+
+		union PacketType {
+			PacketCommand command;
+			PacketResponse response;
+		} ATTRIBUTE_PACKED u;
+	} ATTRIBUTE_PACKED;
+	#define JDWP_PACKET_HEADER_SIZE			(sizeof(::Barry::Protocol::JDWP::Packet) - sizeof(::Barry::Protocol::JDWP::Packet::PacketType))
+
+
+	#define MAKE_JDWPPACKET(var, data)		const ::Barry::Protocol::JDWP::Packet *var = (const ::Barry::Protocol::JDWP::Packet *) (data).GetData()
+	#define MAKE_JDWPPACKETPTR_BUF(var, ptr)		::Barry::Protocol::JDWP::Packet *var = (::Barry::Protocol::JDWP::Packet *)ptr
+
+
+} // namespace JDWP
+
+struct JDWField {
+	uint32_t size;
+
+	union JDWFieldData {
+		uint8_t raw[1];
+	} ATTRIBUTE_PACKED u;
+} ATTRIBUTE_PACKED;
+#define JDWP_FIELD_HEADER_SIZE			(sizeof(::Barry::Protocol::JDWField) - sizeof(::Barry::Protocol::JDWField::JDWFieldData))
+
+struct JVMCommand
+{
+	uint16_t	size;
+	uint8_t		command;
+	uint8_t		raw[1];
+} ATTRIBUTE_PACKED;
+#define SB_JVMCOMMAND_HEADER_SIZE		(sizeof(::Barry::Protocol::JVMCommand))
+
+struct JVMResponse
+{
+	uint8_t		command;
+	uint8_t		unknown;
+	uint16_t	expect;
+} ATTRIBUTE_PACKED;
+#define SB_JVMRESPONSE_HEADER_SIZE		(sizeof(::Barry::Protocol::JVMResponse))
+
+struct JVMModulesList
+{
+	uint32_t	nbr;
+	// remainder of packet is variable
+	// it contains the modules list
+} ATTRIBUTE_PACKED;
+#define SB_JVMMODULES_LIST_HEADER_SIZE		(sizeof(::Barry::Protocol::JVMModulesList))
+
+struct JVMModulesEntry
+{
+	uint32_t	id;
+	uint32_t	uniqueId;
+	uint16_t	sizename;
+	// remainder of packet is variable
+	// it contains the module name
+} ATTRIBUTE_PACKED;
+#define SB_JVMMODULES_ENTRY_HEADER_SIZE		(sizeof(::Barry::Protocol::JVMModulesEntry))
+
+struct JVMThreadsList
+{
+	uint32_t	nbr;
+	// remainder of packet is variable
+	// it contains the threads list
+} ATTRIBUTE_PACKED;
+#define SB_JVMTHREADS_LIST_HEADER_SIZE		(sizeof(::Barry::Protocol::JVMThreadsList))
+
+struct JVMUnknown01
+{
+	uint8_t		byte;
+	uint32_t	address;
+} ATTRIBUTE_PACKED;
+#define SB_JVMUNKNOWN01_HEADER_SIZE			(sizeof(::Barry::Protocol::JVMUnknown01))
+
+struct JVMUnknown02
+{
+	uint32_t	address1;
+	uint32_t	address2;
+} ATTRIBUTE_PACKED;
+#define SB_JVMUNKNOWN02_HEADER_SIZE			(sizeof(::Barry::Protocol::JVMUnknown02))
+
+struct JVMPacket
+{
+	uint16_t	socket;
+	uint16_t	size;		// total size of data packet
+
+	union PacketData
+	{
+		JVMCommand		command;
+		JVMResponse		response;
+		JVMModulesList		moduleslist;
+		JVMThreadsList		threadslist;
+		JVMUnknown01		unknown01;
+		JVMUnknown02		unknown02;
+		uint32_t		address;
+		uint16_t		expect;
+		uint16_t		msglength;
+		uint16_t		value;
+		uint8_t			status;
+		uint8_t			raw[1];
+	} ATTRIBUTE_PACKED u;
+
+} ATTRIBUTE_PACKED;
+#define SB_JVMPACKET_HEADER_SIZE		(sizeof(::Barry::Protocol::JVMPacket) - sizeof(::Barry::Protocol::JVMPacket::PacketData))
+
+
+/////////////////////////////////////////////////////////////////////////////
+// Raw channel packet structure
+struct ChannelPacket
+{
+	uint16_t	socket;		// socket ID... 0 exists by default
+	uint16_t	size;		// total size of data packet
+
+	union PacketData
+	{
+		uint8_t			data[1];
+	} ATTRIBUTE_PACKED u;
+} ATTRIBUTE_PACKED;
+#define SB_CHANNELPACKET_HEADER_SIZE		(sizeof(::Barry::Protocol::ChannelPacket) - sizeof(::Barry::Protocol::ChannelPacket::PacketData))
+
+#define SB_CHANNELPACKET_MAX_DATA_SIZE		0x3FFC
 
 ///////////////////////////////////////////////////////////////////////////////
 // Main packet struct
@@ -755,9 +1013,10 @@ struct Packet
 		DBAccess		db;
 		uint8_t			raw[1];
 
-	} __attribute__ ((packed)) u;
-} __attribute__ ((packed));
-#define SB_PACKET_HEADER_SIZE			(sizeof(Barry::Protocol::Packet) - sizeof(Barry::Protocol::Packet::PacketData))
+	} ATTRIBUTE_PACKED u;
+} ATTRIBUTE_PACKED;
+#define SB_PACKET_SOCKET_SIZE			(sizeof(uint16_t)) // size needed to read the socket in a packet
+#define SB_PACKET_HEADER_SIZE			(sizeof(::Barry::Protocol::Packet) - sizeof(::Barry::Protocol::Packet::PacketData))
 
 // WARNING : For JavaLoader we have some packet with 5 size !
 #define MIN_PACKET_SIZE			5
@@ -784,19 +1043,23 @@ struct Packet
 
 #define SB_PACKET_UPLOAD_HEADER_SIZE		(SB_PACKET_DBACCESS_HEADER_SIZE + UPLOAD_HEADER_SIZE)
 
-#define SB_SEQUENCE_PACKET_SIZE			(SB_PACKET_HEADER_SIZE + sizeof(Barry::Protocol::SequenceCommand))
+#define SB_SEQUENCE_PACKET_SIZE			(SB_PACKET_HEADER_SIZE + sizeof(::Barry::Protocol::SequenceCommand))
 #define SB_SOCKET_PACKET_HEADER_SIZE		(SB_PACKET_HEADER_SIZE + SOCKET_COMMAND_HEADER_SIZE)
-#define SB_MODE_PACKET_COMMAND_SIZE		(SB_SOCKET_PACKET_HEADER_SIZE + sizeof(Barry::Protocol::ModeSelect) - sizeof(Barry::Protocol::ModeSelect::ResponseBlock))
-#define SB_MODE_PACKET_RESPONSE_SIZE		(SB_SOCKET_PACKET_HEADER_SIZE + sizeof(Barry::Protocol::ModeSelect))
+#define SB_MODE_PACKET_COMMAND_SIZE		(SB_SOCKET_PACKET_HEADER_SIZE + sizeof(::Barry::Protocol::ModeSelect) - sizeof(::Barry::Protocol::ModeSelect::ResponseBlock))
+#define SB_MODE_PACKET_RESPONSE_SIZE		(SB_SOCKET_PACKET_HEADER_SIZE + sizeof(::Barry::Protocol::ModeSelect))
 
 
 // Macros
-#define COMMAND(data)				(((const Barry::Protocol::Packet *)data.GetData())->command)
+#define COMMAND(data)				(((const ::Barry::Protocol::Packet *)data.GetData())->command)
 #define IS_COMMAND(data, cmd)			(COMMAND(data) == cmd)
-#define MAKE_PACKET(var, data)			const Barry::Protocol::Packet *var = (const Barry::Protocol::Packet *) (data).GetData()
-#define MAKE_JLPACKET(var, data)		const Barry::Protocol::JLPacket *var = (const Barry::Protocol::JLPacket *) (data).GetData()
-#define MAKE_PACKETPTR_BUF(var, ptr)		Barry::Protocol::Packet *var = (Barry::Protocol::Packet *)ptr
-#define MAKE_JLPACKETPTR_BUF(var, ptr)		Barry::Protocol::JLPacket *var = (Barry::Protocol::JLPacket *)ptr
+#define MAKE_PACKET(var, data)			const ::Barry::Protocol::Packet *var = (const ::Barry::Protocol::Packet *) (data).GetData()
+#define MAKE_JLPACKET(var, data)		const ::Barry::Protocol::JLPacket *var = (const ::Barry::Protocol::JLPacket *) (data).GetData()
+#define MAKE_JVMPACKET(var, data)		const ::Barry::Protocol::JVMPacket *var = (const ::Barry::Protocol::JVMPacket *) (data).GetData()
+#define MAKE_CHANNELPACKET(var, data)		const ::Barry::Protocol::ChannelPacket *var = (const ::Barry::Protocol::ChannelPacket *) (data).GetData()
+#define MAKE_PACKETPTR_BUF(var, ptr)		::Barry::Protocol::Packet *var = (::Barry::Protocol::Packet *)ptr
+#define MAKE_JLPACKETPTR_BUF(var, ptr)		::Barry::Protocol::JLPacket *var = (::Barry::Protocol::JLPacket *)ptr
+#define MAKE_JVMPACKETPTR_BUF(var, ptr)		::Barry::Protocol::JVMPacket *var = (::Barry::Protocol::JVMPacket *)ptr
+#define MAKE_CHANNELPACKETPTR_BUF(var, ptr)	::Barry::Protocol::ChannelPacket *var = (::Barry::Protocol::ChannelPacket *)ptr
 #define MAKE_RECORD(type,var,data,off)		type *var = (type *) ((data).GetData() + (off))
 #define MAKE_RECORD_PTR(type,var,data,off)	type *var = (type *) ((data) + (off))
 
@@ -815,7 +1078,13 @@ struct Packet
 void CheckSize(const Barry::Data &packet, size_t requiredsize);
 unsigned int GetSize(const Barry::Data &packet);
 
+bool IsSequencePacket(const Barry::Data &data);
+
 }} // namespace Barry::Protocol
 
+#if USE_PACK_PRAGMA
+#pragma pack(pop)
+#endif
+
 #endif
 
