@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 */
 
 #include "r_calendar.h"
+#include "r_recur_base-int.h"
 #include "record-internal.h"
 #include "protocol.h"
 #include "protostructs.h"
@@ -33,6 +34,7 @@
 #include <time.h>
 #include <string.h>
 #include <stdexcept>
+#include "ios_state.h"
 
 #define __DEBUG_MODE__
 #include "debug.h"
@@ -95,6 +97,7 @@ uint8_t Calendar::ClassFlagRec2Proto(ClassFlagType f)
 #define CALFC_FREEBUSY_FLAG		0x1c
 #define CALFC_TIMEZONE_CODE		0x1e	// only seems to show up if recurring
 #define CALFC_CLASS_FLAG		0x28    // private flag from outlook
+#define CALFC_CALENDAR_ID		0x2b	// Calendar using (new devices have several calendar)
 #define CALFC_ALLDAYEVENT_FLAG		0xff
 #define CALFC_END			0xffff
 
@@ -148,9 +151,9 @@ const unsigned char* Calendar::ParseField(const unsigned char *begin,
 				return begin;	// done!
 			}
 			else if( b->timeMember && btohs(field->size) == 4 ) {
-				time_t &t = this->*(b->timeMember);
+				TimeT &t = this->*(b->timeMember);
 				dout("min1900: " << field->u.min1900);
-				t = min2time(field->u.min1900);
+				t.Time = min2time(field->u.min1900);
 				return begin;
 			}
 			else if( b->addrMember ) {
@@ -231,6 +234,15 @@ const unsigned char* Calendar::ParseField(const unsigned char *begin,
 		FreeBusyFlag = FreeBusyFlagProto2Rec(field->u.raw[0]);
 		return begin;
 
+	case CALFC_CALENDAR_ID:
+		if( btohs(field->size) == 8 ) {
+			CalendarID = btohll(field->u.uint64);
+		}
+		else {
+			throw Error("Calendar::ParseField: size data unknown in calendar field");
+		}
+		return begin;
+
 	case CALFC_CLASS_FLAG:
 		if( field->u.raw[0] > CR_CLASS_RANGE_HIGH ) {
 			throw Error("Calendar::ParseField: ClassFlag out of range" );
@@ -265,6 +277,11 @@ void Calendar::ParseFields(const Data &data, size_t &offset, const IConverter *i
 	offset += finish - (data.GetData() + offset);
 }
 
+void Calendar::Validate() const
+{
+	RecurBase::Validate();
+}
+
 void Calendar::BuildHeader(Data &data, size_t &offset) const
 {
 	// no header in Calendar records
@@ -297,9 +314,9 @@ void Calendar::BuildFields(Data &data, size_t &offset, const IConverter *ic) con
 				BuildField(data, offset, b->type, (b->iconvNeeded && ic) ? ic->ToBB(s) : s);
 		}
 		else if( b->timeMember ) {
-			time_t t = this->*(b->timeMember);
-			if( t > 0 )
-				BuildField1900(data, offset, b->type, t);
+			TimeT t = this->*(b->timeMember);
+			if( t.Time > 0 )
+				BuildField1900(data, offset, b->type, t.Time);
 		}
 		else if( b->addrMember ) {
 			const EmailAddressList &al = this->*(b->addrMember);
@@ -337,7 +354,7 @@ void Calendar::BuildFields(Data &data, size_t &offset, const IConverter *ic) con
 	// handle special cases
 	if( Recurring ) {
 		CalendarRecurrenceDataField recur;
-		BuildRecurrenceData(StartTime, &recur);
+		BuildRecurrenceData(StartTime.Time, &recur);
 		BuildField(data, offset, RecurBase::RecurringFieldType(),
 			&recur, CALENDAR_RECURRENCE_DATA_FIELD_SIZE);
 	}
@@ -348,6 +365,11 @@ void Calendar::BuildFields(Data &data, size_t &offset, const IConverter *ic) con
 	BuildField(data, offset, CALFC_FREEBUSY_FLAG, FreeBusyFlagRec2Proto(FreeBusyFlag));
 	BuildField(data, offset, CALFC_CLASS_FLAG, ClassFlagRec2Proto(ClassFlag));
 
+	// If CalendarID is defined and most of supported !
+	// (by default 0xffff ffff ffff ffff)
+	if( CalendarID != (uint64_t) -1 )
+		BuildField(data, offset, CALFC_CALENDAR_ID, CalendarID);
+
 	// and finally save unknowns
 	UnknownsType::const_iterator
 		ub = Unknowns.begin(), ue = Unknowns.end();
@@ -360,41 +382,122 @@ void Calendar::BuildFields(Data &data, size_t &offset, const IConverter *ic) con
 
 void Calendar::Clear()
 {
+	// clear the base class too
 	RecurBase::Clear();
 
-	RecType = Calendar::GetDefaultRecType();
+	// clear our fields
+	RecType = GetDefaultRecType();
+	RecordId = 0;
 
 	AllDayEvent = false;
 	Subject.clear();
 	Notes.clear();
 	Location.clear();
-	NotificationTime = StartTime = EndTime = 0;
+	NotificationTime.clear();
+	StartTime.clear();
+	EndTime.clear();
+	Organizer.clear();
+	AcceptedBy.clear();
+	Invited.clear();
 
 	FreeBusyFlag = Free;
 	ClassFlag = Public;
 
-	TimeZoneCode = GetTimeZoneCode(0, 0);	// default to GMT
+	CalendarID = btohll((uint64_t) -1);
+
+	TimeZoneCode = GetStaticTimeZoneCode(0, 0);	// default to GMT
 	TimeZoneValid = false;
 
 	Unknowns.clear();
 }
 
-void Calendar::Dump(std::ostream &os) const
+const FieldHandle<Calendar>::ListT& Calendar::GetFieldHandles()
 {
+	static FieldHandle<Calendar>::ListT fhv;
+
+	if( fhv.size() )
+		return fhv;
+
+#undef CONTAINER_OBJECT_NAME
+#define CONTAINER_OBJECT_NAME fhv
+
+#undef RECORD_CLASS_NAME
+#define RECORD_CLASS_NAME Calendar
+
+#define ALL_COMMON_CALENDAR_FIELDS \
+	FHP(RecType, "Record Type Code"); \
+	FHP(RecordId, "Unique Record ID"); \
+ \
+	FHP(AllDayEvent, "All Day Event"); \
+	FHD(Subject, "Subject", CALFC_SUBJECT, true); \
+	FHD(Notes, "Notes", CALFC_NOTES, true); \
+	FHD(Location, "Location", CALFC_LOCATION, true); \
+	FHD(NotificationTime, "Notification Time (0 is off)", \
+				CALFC_NOTIFICATION_TIME, false); \
+	FHD(StartTime, "Start Time", CALFC_START_TIME, false); \
+	FHD(EndTime, "End Time", CALFC_END_TIME, false); \
+	FHD(Organizer, "Organizer", CALFC_ORGANIZER, true); \
+	FHD(AcceptedBy, "Accepted By", CALFC_ACCEPTED_BY, true); \
+	FHD(Invited, "Invited", CALFC_INVITED, true); \
+ \
+	FHE(fbf, FreeBusyFlagType, FreeBusyFlag, "Free or Busy Flag"); \
+	FHE_CONST(fbf, Free, "Free"); \
+	FHE_CONST(fbf, Tentative, "Tentative"); \
+	FHE_CONST(fbf, Busy, "Busy"); \
+	FHE_CONST(fbf, OutOfOffice, "Out of Office"); \
+ \
+	FHE(cf, ClassFlagType, ClassFlag, "Event Class"); \
+	FHE_CONST(cf, Public, "Public"); \
+	FHE_CONST(cf, Confidential, "Confidential"); \
+	FHE_CONST(cf, Private, "Private"); \
+ \
+	FHP(TimeZoneCode, "Time Zone Code"); \
+	FHP(TimeZoneValid, "Time Zone Validity"); \
+ \
+	FHP(Unknowns, "Unknown Fields");
+
+	ALL_COMMON_CALENDAR_FIELDS
+
+	// the fields unique to Calendar, or different in CalendarALL
+	FHD(CalendarID, "Calendar ID", CALFC_CALENDAR_ID, false);
+
+	// and finally, the RecurBase fields
+	RECUR_BASE_FIELD_HANDLES
+
+	return fhv;
+}
+
+std::string Calendar::GetDescription() const
+{
+	return Subject;
+}
+
+void Calendar::DumpSpecialFields(std::ostream &os) const
+{
+	ios_format_state state(os);
+
 	static const char *ClassTypes[] = { "Public", "Confidential", "Private" };
 	static const char *FreeBusy[] = { "Free", "Tentative", "Busy", "Out of Office" };
 
+	os << "   Calendar ID: 0x" << setbase(16) << CalendarID << "\n";
+	os << "   All Day Event: " << (AllDayEvent ? "yes" : "no") << "\n";
+	os << "   Class: " << ClassTypes[ClassFlag] << "\n";
+	os << "   Free/Busy: " << FreeBusy[FreeBusyFlag] << "\n";
+	if( TimeZoneValid )
+		os << "   Time Zone: " << GetStaticTimeZone(TimeZoneCode)->Name << "\n";
+}
+
+void Calendar::Dump(std::ostream &os) const
+{
+	ios_format_state state(os);
+
 // FIXME - need a "check all data" function that make sure that all
 // recurrence data is within range.  Then call that before using
 // the data, such as in Build and in Dump.
 
 	os << "Calendar entry: 0x" << setbase(16) << RecordId
 		<< " (" << (unsigned int)RecType << ")\n";
-	os << "   All Day Event: " << (AllDayEvent ? "yes" : "no") << "\n";
-	os << "   Class: " << ClassTypes[ClassFlag] << "\n";
-	os << "   Free/Busy: " << FreeBusy[FreeBusyFlag] << "\n";
-	if( TimeZoneValid )
-		os << "   Time Zone: " << GetTimeZone(TimeZoneCode)->Name << "\n";
+	DumpSpecialFields(os);
 
 	// cycle through the type table
 	for(	const FieldLink<Calendar> *b = CalendarFieldLinks;
@@ -407,9 +510,9 @@ void Calendar::Dump(std::ostream &os) const
 				os << "   " << b->name << ": " << s << "\n";
 		}
 		else if( b->timeMember ) {
-			time_t t = this->*(b->timeMember);
-			if( t > 0 )
-				os << "   " << b->name << ": " << ctime(&t);
+			TimeT t = this->*(b->timeMember);
+			if( t.Time > 0 )
+				os << "   " << b->name << ": " << t << "\n";
 			else
 				os << "   " << b->name << ": disabled\n";
 		}
@@ -433,6 +536,131 @@ void Calendar::Dump(std::ostream &os) const
 	os << Unknowns;
 }
 
+bool Calendar::operator<(const Calendar &other) const
+{
+	if( StartTime < other.StartTime )
+		return true;
+	else if( other.StartTime < StartTime )
+		return false;
+
+	// times are equal, so secondary sort based on Subject + Location
+	int cmp = Subject.compare(other.Subject);
+	if( cmp == 0 )
+		cmp = Location.compare(other.Location);
+	return cmp < 0;
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+// Calendar-All class
+
+// calendar-all field codes
+#define CALALLFC_CALENDAR_ID		0x02	// Calendar using (new devices have several calendar)
+#define CALALLFC_MAIL_ACCOUNT		0x03
+#define CALALLFC_UNIQUEID			0x05
+#define CALALLFC_CAL_OBJECT			0x0a
+#define CALALLFC_END				0xffff
+
+void CalendarAll::Clear()
+{
+	Calendar::Clear();
+
+	MailAccount.clear();
+}
+
+const FieldHandle<CalendarAll>::ListT& CalendarAll::GetFieldHandles()
+{
+	static FieldHandle<CalendarAll>::ListT fhv;
+
+	if( fhv.size() )
+		return fhv;
+
+#undef CONTAINER_OBJECT_NAME
+#define CONTAINER_OBJECT_NAME fhv
+
+#undef RECORD_CLASS_NAME
+#define RECORD_CLASS_NAME CalendarAll
+
+	ALL_COMMON_CALENDAR_FIELDS
+
+	// Calendar:: field, but with a CalendarAll ID
+	FHD(CalendarID, "Calendar ID", CALALLFC_CALENDAR_ID, false);
+
+	// add the fields specific to CalendarAll
+	FHD(MailAccount, "Mail Account", CALALLFC_MAIL_ACCOUNT, true);
+
+	// and finally, the RecurBase fields
+	RECUR_BASE_FIELD_HANDLES
+
+	return fhv;
+}
+
+void CalendarAll::ParseHeader(const Data &data, size_t &offset)
+{
+	const unsigned char *b = (const unsigned char*) (data.GetData() + offset);
+	const unsigned char *e = (const unsigned char*) (data.GetData() + data.GetSize());
+
+	while( (b + COMMON_FIELD_HEADER_SIZE) < e ) {
+		const CommonField *field = (const CommonField *) b;
+
+		// advance and check size
+		b += COMMON_FIELD_HEADER_SIZE + btohs(field->size);
+		if( b > e )					// if begin==end, we are ok
+			continue;
+
+		if( !btohs(field->size) )	// if field has no size, something's up
+			continue;
+
+		// handle special cases
+		if( field->type == CALALLFC_CAL_OBJECT )
+		{
+			b -= btohs(field->size);
+			// end of header
+			break;
+		}
+
+		switch( field->type )
+		{
+		case CALALLFC_CALENDAR_ID:
+			if( btohs(field->size) == 8 ) {
+				CalendarID = btohll(field->u.uint64);
+			}
+			else {
+				throw Error("CalendarAll::ParseField: size data unknown in calendar field");
+			}
+			continue;
+
+		case CALALLFC_MAIL_ACCOUNT:
+			MailAccount = ParseFieldString(field);
+			continue;
+
+		case CALALLFC_UNIQUEID:
+			if( btohs(field->size) == 4 ) {
+				RecordId = btohl(field->u.uint32);
+			}
+			else {
+				throw Error("CalendarAll::ParseHeader: size data unknown in calendar field");
+			}
+			continue;
+		}
+
+		// if still not handled, add to the Unknowns list
+		UnknownField uf;
+		uf.type = field->type;
+		uf.data.assign((const char*)field->u.raw, btohs(field->size));
+		Unknowns.push_back(uf);
+	}
+
+	offset += b - (data.GetData() + offset);
+}
+
+void CalendarAll::DumpSpecialFields(std::ostream &os) const
+{
+	ios_format_state state(os);
+
+	Calendar::DumpSpecialFields(os);
+	os << "   Mail Account: " << MailAccount << "\n";
+}
 
 } // namespace Barry
 
