@@ -1,10 +1,10 @@
 ///
 /// \file	r_calendar.h
-///		Blackberry database record parser class for calndar records.
+///		Blackberry database record parser class for calendar records.
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ class IConverter;
 class BXEXPORT Calendar : public RecurBase
 {
 public:
-	typedef std::vector<UnknownField>		UnknownsType;
+	typedef Barry::UnknownsType		UnknownsType;
 
 	uint8_t RecType;
 	uint32_t RecordId;
@@ -59,9 +59,9 @@ public:
 	std::string Subject;
 	std::string Notes;
 	std::string Location;
-	time_t NotificationTime;	// 0 means notification is off
-	time_t StartTime;
-	time_t EndTime;
+	Barry::TimeT NotificationTime;	// 0 means notification is off
+	Barry::TimeT StartTime;
+	Barry::TimeT EndTime;
 	EmailAddressList Organizer;
 	EmailAddressList AcceptedBy;
 	EmailAddressList Invited;		// list of invited people (email a
@@ -99,14 +99,21 @@ public:
 
 	ClassFlagType ClassFlag;
 
+	uint64_t CalendarID;	// Calendar ID (usefull if devices have several calendars)
 
-	unsigned short TimeZoneCode;	// the time zone originally used
+	uint16_t TimeZoneCode;		// the time zone originally used
 					// for the recurrence data...
 					// seems to have little use, but
 					// set to your current time zone
 					// as a good default
 	bool TimeZoneValid;		// true if the record contained a
-					// time zone code
+					// time zone code, or in other words,
+					// true if TimeZoneCode contains
+					// valid data.  Be sure to set this to
+					// to true if you desire to write a
+					// calendar item with a time zone
+					// to the device, otherwise, the builder
+					// code will skip the time zone.
 
 	// unknown
 	UnknownsType Unknowns;
@@ -118,6 +125,8 @@ protected:
 	static ClassFlagType ClassFlagProto2Rec(uint8_t f);
 	static uint8_t ClassFlagRec2Proto(ClassFlagType f);
 
+	virtual void DumpSpecialFields(std::ostream &os) const;
+
 public:
 	const unsigned char* ParseField(const unsigned char *begin,
 		const unsigned char *end, const IConverter *ic = 0);
@@ -127,6 +136,7 @@ public:
 	~Calendar();
 
 	// Parser / Builder API (see parser.h / builder.h)
+	void Validate() const;
 	uint8_t GetRecType() const { return RecType; }
 	uint32_t GetUniqueId() const { return RecordId; }
 	void SetIds(uint8_t Type, uint32_t Id) { RecType = Type; RecordId = Id; }
@@ -135,16 +145,20 @@ public:
 	void BuildHeader(Data &data, size_t &offset) const;
 	void BuildFields(Data &data, size_t &offset, const IConverter *ic = 0) const;
 
+	// operations (common among record classes)
 	void Clear();
-
 	void Dump(std::ostream &os) const;
+	std::string GetDescription() const;
 
 	// sorting
-	bool operator<(const Calendar &other) const { return StartTime < other.StartTime; }
+	bool operator<(const Calendar &other) const;
 
 	// database name
 	static const char * GetDBName() { return "Calendar"; }
 	static uint8_t GetDefaultRecType() { return 5; }	// or 0?
+
+	// Generic Field Handle support
+	static const FieldHandle<Calendar>::ListT& GetFieldHandles();
 };
 
 BXEXPORT inline std::ostream& operator<<(std::ostream &os, const Calendar &msg) {
@@ -152,6 +166,35 @@ BXEXPORT inline std::ostream& operator<<(std::ostream &os, const Calendar &msg)
 	return os;
 }
 
+
+class BXEXPORT CalendarAll : public Calendar
+{
+public:
+	std::string MailAccount;
+
+protected:
+	virtual void DumpSpecialFields(std::ostream &os) const;
+
+public:
+	// Parser / Builder API (see parser.h / builder.h)
+	void ParseHeader(const Data &data, size_t &offset);
+
+	void Clear();
+
+public:
+	// database name
+	static const char * GetDBName() { return "Calendar - All"; }
+
+	// Generic Field Handle support
+	static const FieldHandle<CalendarAll>::ListT& GetFieldHandles();
+};
+
+BXEXPORT inline std::ostream& operator<<(std::ostream &os, const CalendarAll &msg) {
+	msg.Dump(os);
+	return os;
+}
+
+
 /// @}
 
 } // namespace Barry
