@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,10 +28,13 @@
 #include "error.h"
 #include "endian.h"
 #include "iconv.h"
-#include <ostream>
+#include "trim.h"
+#include <iostream>
 #include <iomanip>
+#include <sstream>
 #include <time.h>
 #include <stdexcept>
+#include "ios_state.h"
 
 #define __DEBUG_MODE__
 #include "debug.h"
@@ -59,6 +62,8 @@ namespace Barry {
 #define CFC_WORK_PHONE_2	16	// 0x10
 #define CFC_HOME_PHONE_2	17	// 0x11
 #define CFC_OTHER_PHONE		18	// 0x12
+#define CFC_MOBILE_PHONE_2	19	// 0x13
+#define CFC_HOME_FAX		20	// 0x14
 #define CFC_NAME		32	// 0x20 used twice, in first/last name order
 #define CFC_COMPANY		33
 #define CFC_DEFAULT_COMM_METHOD	34
@@ -93,16 +98,21 @@ namespace Barry {
 #define CFC_IMAGE		77	// 0x4D
 #define CFC_BIRTHDAY		82	// 0x52
 #define CFC_ANNIVERSARY		83	// 0x53
+#define CFC_MAYBE_CATEGORYID	84	// 0x54
 #define CFC_UNIQUEID		85	// 0x55
+#define CFC_NICKNAME		86	// 0x56
 #define CFC_INVALID_FIELD	255
 
 // Contact code to field table
 static FieldLink<Contact> ContactFieldLinks[] = {
+   { CFC_NICKNAME,     "Nickname",   0,0,                 &Contact::Nickname, 0, 0, 0, 0, true },
    { CFC_PHONE,        "Phone",      0,0,                 &Contact::Phone, 0, 0, 0, 0, true },
    { CFC_FAX,          "Fax",        "facsimileTelephoneNumber",0, &Contact::Fax, 0, 0, 0, 0, true },
+   { CFC_HOME_FAX,     "HomeFax",    0,0,                 &Contact::HomeFax, 0, 0, 0, 0, true },
    { CFC_WORK_PHONE,   "WorkPhone",  "telephoneNumber",0, &Contact::WorkPhone, 0, 0, 0, 0, true },
    { CFC_HOME_PHONE,   "HomePhone",  "homePhone",0,       &Contact::HomePhone, 0, 0, 0, 0, true },
    { CFC_MOBILE_PHONE, "MobilePhone","mobile",0,          &Contact::MobilePhone, 0, 0, 0, 0, true },
+   { CFC_MOBILE_PHONE_2,"MobilePhone2",0,0,               &Contact::MobilePhone2, 0, 0, 0, 0, true },
    { CFC_PAGER,        "Pager",      "pager",0,           &Contact::Pager, 0, 0, 0, 0, true },
    { CFC_PIN,          "PIN",        0,0,                 &Contact::PIN, 0, 0, 0, 0, true },
    { CFC_RADIO,        "Radio",      0,0,                 &Contact::Radio, 0, 0, 0, 0, true },
@@ -250,6 +260,13 @@ const unsigned char* Contact::ParseField(const unsigned char *begin,
 		Anniversary.FromBBString(astring);
 		}
 		return begin;
+
+	case CFC_UNIQUEID:
+		// this is a duplicate of the UniqueID that comes from
+		// the envelope part of the protocol... just throw this
+		// away, since when we upload it, we need to use a
+		// consistent UniqueID / RecordID from the API
+		return begin;
 	}
 
 	// if still not handled, add to the Unknowns list
@@ -267,7 +284,6 @@ void Contact::ParseHeader(const Data &data, size_t &offset)
 	// no header to parse in Contact records
 }
 
-// this is called by the RecordParser<> class, which checks size for us
 void Contact::ParseFields(const Data &data, size_t &offset, const IConverter *ic)
 {
 	const unsigned char *finish = ParseCommonFields(*this,
@@ -275,6 +291,13 @@ void Contact::ParseFields(const Data &data, size_t &offset, const IConverter *ic
 	offset += finish - (data.GetData() + offset);
 }
 
+void Contact::Validate() const
+{
+	if( !GetFullName().size() && !Company.size() ) {
+		throw Barry::ValidationError("A contact record must contain either a First/Last name, or a Company name.");
+	}
+}
+
 void Contact::BuildHeader(Data &data, size_t &offset) const
 {
 	// no header in Contact records
@@ -391,14 +414,18 @@ void Contact::BuildFields(Data &data, size_t &offset, const IConverter *ic) cons
 
 void Contact::Clear()
 {
-	RecType = Contact::GetDefaultRecType();
+	RecType = GetDefaultRecType();
+	RecordId = 0;
 
 	EmailAddresses.clear();
 	Phone.clear();
+
 	Fax.clear();
+	HomeFax.clear();
 	WorkPhone.clear();
 	HomePhone.clear();
 	MobilePhone.clear();
+	MobilePhone2.clear();
 	Pager.clear();
 	PIN.clear();
 	Radio.clear();
@@ -419,6 +446,7 @@ void Contact::Clear()
 	UserDefined3.clear();
 	UserDefined4.clear();
 	Image.clear();
+	Nickname.clear();
 
 	Birthday.Clear();
 	Anniversary.Clear();
@@ -434,6 +462,106 @@ void Contact::Clear()
 	m_FirstNameSeen = false;
 }
 
+const FieldHandle<Contact>::ListT& Contact::GetFieldHandles()
+{
+	static FieldHandle<Contact>::ListT fhv;
+
+	if( fhv.size() )
+		return fhv;
+
+#undef CONTAINER_OBJECT_NAME
+#define CONTAINER_OBJECT_NAME fhv
+
+#undef RECORD_CLASS_NAME
+#define RECORD_CLASS_NAME Contact
+
+	// first number is priority of fields... 0 being most critical fields
+	FHP(RecType, "Record Type Code");
+	FHP(RecordId, "Unique ID");
+	FHP(EmailAddresses, "Email Addresses");
+
+	FHP(FirstName, "First Name");
+	FHP(LastName, "Last Name");
+	FHL(Company, "Company", CFC_COMPANY, true, "o", 0);
+	FHL(JobTitle, "Job Title", CFC_TITLE, true, "title", 0);
+	FHD(Prefix, "Prefix", CFC_PREFIX, true);
+
+	FHD(Nickname, "Nickname", CFC_NICKNAME, true);
+	FHD(Phone, "Phone (deprecated)", CFC_PHONE, true);
+	FHL(Fax, "Work Fax", CFC_FAX, true, "facsimileTelephoneNumber", 0);
+	FHD(HomeFax, "Home Fax", CFC_HOME_FAX, true);
+	FHL(WorkPhone, "Work Phone", CFC_WORK_PHONE, true,
+		"telephoneNumber", 0);
+	FHD(WorkPhone2, "Work Phone 2", CFC_WORK_PHONE_2, true);
+	FHL(HomePhone, "Home Phone", CFC_HOME_PHONE, true, "homePhone", 0);
+	FHD(HomePhone2, "Home Phone 2", CFC_HOME_PHONE_2, true);
+	FHL(MobilePhone, "Mobile Phone", CFC_MOBILE_PHONE, true, "mobile", 0);
+	FHD(MobilePhone2, "Mobile Phone 2", CFC_MOBILE_PHONE_2, true);
+	FHD(OtherPhone, "Other Phone", CFC_OTHER_PHONE, true);
+	FHL(Pager, "Pager", CFC_PAGER, true, "pager", 0);
+	FHD(PIN, "PIN", CFC_PIN, true);
+	FHD(Radio, "Radio", CFC_RADIO, true);
+	FHD(DefaultCommunicationsMethod, "Default Communications Method",
+		CFC_DEFAULT_COMM_METHOD, true);
+	FHD(PublicKey, "Public Key", CFC_PUBLIC_KEY, false);
+	FHD(URL, "URL", CFC_URL, true);
+	FHD(Notes, "Notes", CFC_NOTES, true);
+	FHD(UserDefined1, "User Defined Field 1", CFC_USER_DEFINED_1, true);
+	FHD(UserDefined2, "User Defined Field 2", CFC_USER_DEFINED_2, true);
+	FHD(UserDefined3, "User Defined Field 3", CFC_USER_DEFINED_3, true);
+	FHD(UserDefined4, "User Defined Field 4", CFC_USER_DEFINED_4, true);
+	FHD(Image, "Image", CFC_IMAGE, false);
+
+	FHD(Birthday, "Birthday", CFC_BIRTHDAY, true);
+	FHD(Anniversary, "Anniversary", CFC_ANNIVERSARY, true);
+
+	FHC(WorkAddress, "Work Address");
+	FHS(WorkAddress, Address1, "Work Address 1",
+					CFC_ADDRESS1, true, 0, 0);
+	FHS(WorkAddress, Address2, "Work Address 2",
+					CFC_ADDRESS2, true, 0, 0);
+	FHS(WorkAddress, Address3, "Work Address 3",
+					CFC_ADDRESS3, true, 0, 0);
+	FHS(WorkAddress, City, "Work City",
+					CFC_CITY, true, "l", 0);
+	FHS(WorkAddress, Province, "Work Province",
+					CFC_PROVINCE, true, "st", 0);
+	FHS(WorkAddress, PostalCode, "Work Postal Code",
+					CFC_POSTAL_CODE, true, "postalCode", 0);
+	FHS(WorkAddress, Country, "Work Country",
+					CFC_COUNTRY, true, "c", "country");
+
+	FHC(HomeAddress, "Home Address");
+	FHS(HomeAddress, Address1, "Home Address 1",
+					CFC_HOME_ADDRESS1, true, 0, 0);
+	FHS(HomeAddress, Address2, "Home Address 2",
+					CFC_HOME_ADDRESS2, true, 0, 0);
+	FHS(HomeAddress, Address3, "Home Address 3",
+					CFC_HOME_ADDRESS3, true, 0, 0);
+	FHS(HomeAddress, City, "Home City",
+					CFC_HOME_CITY, true, 0, 0);
+	FHS(HomeAddress, Province, "Home Province",
+					CFC_HOME_PROVINCE, true, 0, 0);
+	FHS(HomeAddress, PostalCode, "Home Postal Code",
+					CFC_HOME_POSTAL_CODE, true, 0, 0);
+	FHS(HomeAddress, Country, "Home Country",
+					CFC_HOME_COUNTRY, true, 0, 0);
+
+	FHP(Categories, "Categories");
+//	FHP(GroupLinks, "Group Links");
+	FHP(Unknowns, "Unknown Fields");
+
+	return fhv;
+}
+
+std::string Contact::GetDescription() const
+{
+	string desc = GetFullName();
+	if( desc.size() == 0 && Company.size() )
+		return Company;
+	return desc;
+}
+
 //
 // GetFullName
 //
@@ -464,8 +592,10 @@ const std::string& Contact::GetEmail(unsigned int index) const
 
 void Contact::Dump(std::ostream &os) const
 {
-	ios::fmtflags oldflags = os.setf(ios::left);
-	char fill = os.fill(' ');
+	ios_format_state state(os);
+
+	os.setf(ios::left);
+	os.fill(' ');
 
 	os << "Contact: 0x" << setbase(16) << GetID()
 		<< " (" << (unsigned int)RecType << ")\n";
@@ -505,7 +635,7 @@ void Contact::Dump(std::ostream &os) const
 		// print only fields with data
 		if( pField && pField->size() ) {
 			os << "    " << setw(20) << b->name;
-			os << ": " << *pField << "\n";
+			os << ": " << Cr2LfWrapper(*pField) << "\n";
 		}
 	}
 
@@ -541,10 +671,23 @@ void Contact::Dump(std::ostream &os) const
 
 	// and finally print unknowns
 	os << Unknowns;
+}
 
-	// cleanup the stream
-	os.flags(oldflags);
-	os.fill(fill);
+bool Contact::operator<(const Contact &other) const
+{
+	// old sorting mechanism, to put group links at the bottom
+	//return GroupLinks.size() == 0 && other.GroupLinks.size() > 0;
+	// testing - put group links at the top
+	//return GroupLinks.size() > 0 && other.GroupLinks.size() == 0;
+
+	// usually one of these fields is filled in, so compare
+	// them all in a ( LastName + FirstName + Company ) key style
+	int cmp = LastName.compare(other.LastName);
+	if( cmp == 0 )
+		cmp = FirstName.compare(other.FirstName);
+	if( cmp == 0 )
+		cmp = Company.compare(other.Company);
+	return cmp < 0;
 }
 
 void Contact::SplitName(const std::string &full, std::string &first, std::string &last)
@@ -564,5 +707,53 @@ void Contact::SplitName(const std::string &full, std::string &first, std::string
 	}
 }
 
+std::string Contact::Email2CommaString(const EmailList &list)
+{
+	ostringstream oss;
+	for( EmailList::const_iterator i = list.begin(); i!=list.end(); ++i ) {
+		if( i != list.begin() )
+			oss << ", ";
+		oss << *i;
+	}
+	return oss.str();
+}
+
+/// Replaces the EmailAddresses list with the parsed results of
+/// list.  If list is empty, then EmailAddresses will also be empty.
+/// Note that incoming addresses need to be in simple format, not
+/// complex formats like "Name <user@example.com>" but just
+/// "user@example.com".  This is a device limitation.
+///
+/// Any complex email addresses found in the list will be dropped,
+/// with a message sent to the debug output stream.
+void Contact::CommaString2Email(const std::string &list, EmailList &result)
+{
+	// start fresh
+	result.clear();
+
+	// parse the comma separated list
+	istringstream iss(list);
+	string address;
+
+	while( iss >> ws && getline(iss, address, ',') ) {
+		// trim any trailing whitespace in the address
+		Inplace::rtrim(address);
+
+		// is this a complex address?  like:
+		// Chris Frey <cdfrey@foursquare.net>
+		// The device only accepts the plain
+		// "cdfrey@foursquare.net" part here
+		if( address.rfind('>') != string::npos ) {
+			dout("Error: Cannot convert complex name+address to a simple contact email address, skipping: " << address);
+			continue;
+		}
+
+		// add to list if anything left
+		if( address.size() ) {
+			result.push_back(address);
+		}
+	}
+}
+
 } // namespace Barry
 
