@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -65,9 +65,9 @@ public:
 	typedef Barry::CategoryList			CategoryList;
 	typedef ContactGroupLink			GroupLink;
 	typedef std::vector<GroupLink>			GroupLinksType;
-	typedef std::vector<UnknownField>		UnknownsType;
-	typedef std::string				EmailType;
-	typedef std::vector<EmailType>			EmailList;
+	typedef Barry::UnknownsType			UnknownsType;
+	typedef Barry::EmailType			EmailType;
+	typedef Barry::EmailList			EmailList;
 
 	//
 	// Record fields
@@ -89,9 +89,11 @@ public:
 
 	std::string
 		Fax,
+		HomeFax,
 		WorkPhone,
 		HomePhone,
 		MobilePhone,
+		MobilePhone2,
 		Pager,
 		PIN,
 		Radio,
@@ -111,7 +113,8 @@ public:
 		UserDefined2,
 		UserDefined3,
 		UserDefined4,
-		Image;
+		Image,
+		Nickname;
 
 	Date Birthday;
 	Date Anniversary;
@@ -144,6 +147,7 @@ public:
 	const std::string& GetEmail(unsigned int index = 0) const;
 
 	// Parser / Builder API (see parser.h / builder.h)
+	void Validate() const;
 	uint8_t GetRecType() const { return RecType; }
 	uint32_t GetUniqueId() const { return RecordId; }
 	void SetIds(uint8_t Type, uint32_t Id) { RecType = Type; RecordId = Id; }
@@ -152,16 +156,14 @@ public:
 	void BuildHeader(Data &data, size_t &offset) const;
 	void BuildFields(Data &data, size_t &offset, const IConverter *ic = 0) const;
 
+	// operations (common among record classes)
 	void Clear();			// erase everything
-
 	void Dump(std::ostream &os) const;
+	std::string GetDescription() const;
 
-	// sorting - put group links at the end
-	bool operator<(const Contact &other) const {
-		return GroupLinks.size() == 0 && other.GroupLinks.size() > 0;
-//		// testing - put group links at the top
-//		return GroupLinks.size() > 0 && other.GroupLinks.size() == 0;
-	}
+	// Sorting - use enough data to make the sorting as
+	//           consistent as possible
+	bool operator<(const Contact &other) const;
 
 	// database name
 	static const char * GetDBName() { return "Address Book"; }
@@ -169,6 +171,11 @@ public:
 
 	// helpers
 	static void SplitName(const std::string &full, std::string &first, std::string &last);
+	static std::string Email2CommaString(const EmailList &list);
+	static void CommaString2Email(const std::string &list, EmailList &result);
+
+	// Generic Field Handle support
+	static const FieldHandle<Contact>::ListT& GetFieldHandles();
 };
 
 BXEXPORT inline std::ostream& operator<< (std::ostream &os, const Contact &contact) {
