@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
     Copyright (C) 2007, Brian Edginton
 
     This program is free software; you can redistribute it and/or modify
@@ -37,7 +37,8 @@ class IConverter;
 class BXEXPORT Folder
 {
 public:
-	typedef std::vector<UnknownField>			UnknownsType;
+	typedef Barry::UnknownsType			UnknownsType;
+
 	uint8_t RecType;
 	uint32_t RecordId;
 
@@ -72,6 +73,8 @@ public:
 	Folder();
 	~Folder();
 
+	// Parser / Builder API (see parser.h / builder.h)
+	void Validate() const;
 	const unsigned char* ParseField(const unsigned char *begin,
 		const unsigned char *end, const IConverter *ic = 0);
 	uint8_t GetRecType() const { return RecType; }
@@ -80,16 +83,21 @@ public:
 	void ParseHeader(const Data &data, size_t &offset);
 	void ParseFields(const Data &data, size_t &offset, const IConverter *ic = 0);
 	void BuildHeader(Data &data, size_t &offset) const;
+	void BuildFields(Data &data, size_t &offset, const IConverter *ic = 0) const;
 
+	// operations (common among record classes)
 	void Clear();
-
 	void Dump(std::ostream &os) const;
+	std::string GetDescription() const;
+
 	bool operator<(const Folder &other) const { return Name < other.Name; }
 
 	// database name
 	static const char * GetDBName() { return "Folders"; }
 	static uint8_t GetDefaultRecType() { return 0; }
 
+	// Generic Field Handle support
+	static const FieldHandle<Folder>::ListT& GetFieldHandles();
 };
 
 BXEXPORT inline std::ostream& operator<<(std::ostream &os, const Folder &msg) {
