@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
     Copyright (C) 2007, Brian Edginton
 
     This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include "iconv.h"
 #include <ostream>
 #include <iomanip>
+#include "ios_state.h"
 
 using namespace std;
 using namespace Barry::Protocol;
@@ -95,8 +96,8 @@ const unsigned char* Memo::ParseField(const unsigned char *begin,
 				return begin;   // done!
 			}
 			else if( b->timeMember && btohs(field->size) == 4 ) {
-				time_t &t = this->*(b->timeMember);
-				t = min2time(field->u.min1900);
+				TimeT &t = this->*(b->timeMember);
+				t.Time = min2time(field->u.min1900);
 				return begin;
 			}
 		}
@@ -137,6 +138,10 @@ void Memo::ParseFields(const Data &data, size_t &offset, const IConverter *ic)
 }
 
 
+void Memo::Validate() const
+{
+}
+
 void Memo::BuildHeader(Data &data, size_t &offset) const
 {
 	// no header in Memo records
@@ -198,10 +203,23 @@ void Memo::BuildFields(Data &data, size_t &offset, const IConverter *ic) const
 
 void Memo::Dump(std::ostream &os) const
 {
+	ios_format_state state(os);
+
 	os << "Memo entry: 0x" << setbase(16) << RecordId
 	   << " (" << (unsigned int)RecType << ")\n";
 	os << "    Title: " << Title << "\n";
-	os << "    Body: " << Body << "\n";
+	os << "    Body: ";
+
+	// The Body may have '\r' characters... translate them
+	// in the output to make it look more pretty
+	for( string::const_iterator i = Body.begin(); i != Body.end(); ++i ) {
+		if( *i == '\r' )
+			os << "\n          ";
+		else
+			os << *i;
+	}
+	os << "\n";
+
 	if( Categories.size() ) {
 		string display;
 		Categories.CategoryList2Str(display);
@@ -212,8 +230,19 @@ void Memo::Dump(std::ostream &os) const
 	os << "\n\n";
 }
 
+bool Memo::operator<(const Memo &other) const
+{
+	int cmp = Title.compare(other.Title);
+	if( cmp == 0 )
+		cmp = Body.compare(other.Body);
+	return cmp < 0;
+}
+
 void Memo::Clear()
 {
+	RecType = GetDefaultRecType();
+	RecordId = 0;
+
 	Title.clear();
 	Body.clear();
 	Categories.clear();
@@ -221,6 +250,35 @@ void Memo::Clear()
 	Unknowns.clear();
 }
 
+const FieldHandle<Memo>::ListT& Memo::GetFieldHandles()
+{
+	static FieldHandle<Memo>::ListT fhv;
+
+	if( fhv.size() )
+		return fhv;
+
+#undef CONTAINER_OBJECT_NAME
+#define CONTAINER_OBJECT_NAME fhv
+
+#undef RECORD_CLASS_NAME
+#define RECORD_CLASS_NAME Memo
+
+	FHP(RecType, "Record Type Code");
+	FHP(RecordId, "Unique Record ID");
+
+	FHD(Title, "Title", MEMFC_TITLE, true);
+	FHD(Body, "Body", MEMFC_BODY, true);
+	FHD(Categories, "Categories", MEMFC_CATEGORY, true);
+
+	FHP(Unknowns, "Unknown Fields");
+
+	return fhv;
+}
+
+std::string Memo::GetDescription() const
+{
+	return Title;
+}
 
 } // namespace Barry
 
