@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
     Copyright (C) 2007, Brian Edginton
 
     This program is free software; you can redistribute it and/or modify
@@ -37,7 +37,7 @@ class IConverter;
 class BXEXPORT Memo
 {
 public:
-	typedef std::vector<UnknownField>       UnknownsType;
+	typedef Barry::UnknownsType			UnknownsType;
 
 	uint8_t RecType;
 	uint32_t RecordId;
@@ -57,6 +57,7 @@ public:
 	~Memo();
 
 	// Parser / Builder API (see parser.h / builder.h)
+	void Validate() const;
 	uint8_t GetRecType() const { return RecType; }
 	uint32_t GetUniqueId() const { return RecordId; }
 	void SetIds(uint8_t Type, uint32_t Id) { RecType = Type; RecordId = Id; }
@@ -65,14 +66,19 @@ public:
 	void BuildHeader(Data &data, size_t &offset) const;
 	void BuildFields(Data &data, size_t &offset, const IConverter *ic = 0) const;
 
+	// operations (common among record classes)
 	void Clear();
-
 	void Dump(std::ostream &os) const;
+	std::string GetDescription() const;
+
+	bool operator<(const Memo &other) const;
 
-	bool operator<(const Memo &other) const { return Title < other.Title; }
 	// database name
 	static const char * GetDBName() { return "Memos"; }
 	static uint8_t GetDefaultRecType() { return 0; }    // or 0?
+
+	// Generic Field Handle support
+	static const FieldHandle<Memo>::ListT& GetFieldHandles();
 };
 
 BXEXPORT inline std::ostream& operator<<(std::ostream &os, const Memo &msg) {
