@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
     Copyright (C) 2007, Brian Edginton (edge@edginton.net)
 
     This program is free software; you can redistribute it and/or modify
@@ -39,9 +39,26 @@ namespace Barry {
 class BXEXPORT PINMessage : public MessageBase
 {
 public:
+	PINMessage()
+	{
+		// must call this again, since base class can't call ours
+		Clear();
+	}
+
+	void Clear()
+	{
+		MessageBase::Clear();
+
+		RecType = GetDefaultRecType();
+		RecordId = 0;
+	}
+
 	// database name
 	static const char * GetDBName() { return "PIN Messages"; }
 	static uint8_t GetDefaultRecType() { return 0; }
+
+	// Generic Field Handle support
+	static const FieldHandle<PINMessage>::ListT& GetFieldHandles();
 };
 
 BXEXPORT inline std::ostream& operator<<(std::ostream &os, const PINMessage &msg) {
