@@ -1,11 +1,11 @@
 ///
-/// \file	r_save_message.h
+/// \file	r_saved_message.h
 ///		Blackberry database record parser class for saved email
 ///		message records.
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
     Copyright (C) 2007, Brian Edginton (edge@edginton.net)
 
     This program is free software; you can redistribute it and/or modify
@@ -40,9 +40,25 @@ namespace Barry {
 class BXEXPORT SavedMessage : public MessageBase
 {
 public:
+	SavedMessage()
+	{
+		Clear();
+	}
+
+	void Clear()
+	{
+		MessageBase::Clear();
+
+		RecType = GetDefaultRecType();
+		RecordId = 0;
+	}
+
 	// database name
 	static const char * GetDBName() { return "Saved Email Messages"; }
 	static uint8_t GetDefaultRecType() { return 3; }
+
+	// Generic Field Handle support
+	static const FieldHandle<SavedMessage>::ListT& GetFieldHandles();
 };
 
 BXEXPORT inline std::ostream& operator<<(std::ostream &os, const SavedMessage &msg) {
