@@ -5,7 +5,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include <iomanip>
 #include <time.h>
 #include <stdexcept>
+#include "ios_state.h"
 
 #define __DEBUG_MODE__
 #include "debug.h"
@@ -94,7 +95,7 @@ const unsigned char* ServiceBookConfig::ParseField(const unsigned char *begin,
 		break;
 
 	default:
-		eout("------> Unknown packed field format: 0x" << std::hex <<
+		dout("------> Unknown packed field format: 0x" << std::hex <<
 			(unsigned int) Format);
 		throw BadPackedFormat(Format);
 		return begin + 1;
@@ -155,6 +156,10 @@ void ServiceBookConfig::ParseFields(const Data &data, size_t &offset, const ICon
 	offset += finish - (data.GetData() + offset);
 }
 
+void ServiceBookConfig::Validate() const
+{
+}
+
 void ServiceBookConfig::BuildHeader(Data &data, size_t &offset) const
 {
 	// make sure there is enough space
@@ -178,11 +183,14 @@ void ServiceBookConfig::BuildFields(Data &data, size_t &offset, const IConverter
 
 void ServiceBookConfig::Clear()
 {
+	Format = 0;
 	Unknowns.clear();
 }
 
 void ServiceBookConfig::Dump(std::ostream &os) const
 {
+	ios_format_state state(os);
+
 	os << "   ServiceBookConfig Format: " << setbase(16) << (uint16_t)Format << "\n";
 
 	// cycle through the type table
@@ -196,9 +204,9 @@ void ServiceBookConfig::Dump(std::ostream &os) const
 				os << "      " << b->name << ": " << s << "\n";
 		}
 		else if( b->timeMember ) {
-			time_t t = this->*(b->timeMember);
-			if( t > 0 )
-				os << "      " << b->name << ": " << ctime(&t);
+			TimeT t = this->*(b->timeMember);
+			if( t.Time> 0 )
+				os << "      " << b->name << ": " << t << "\n";
 		}
 	}
 
@@ -290,8 +298,8 @@ FieldLink<RecordT>* ParseFieldByTable(RecordT *rec,
 				return links;
 			}
 			else if( b->timeMember && btohs(field->size) == 4 ) {
-				time_t &t = rec->*(b->timeMember);
-				t = min2time(field->u.min1900);
+				TimeT &t = rec->*(b->timeMember);
+				t.Time = min2time(field->u.min1900);
 				return links;
 			}
 		}
@@ -394,6 +402,11 @@ void ServiceBook::ParseFields(const Data &data, size_t &offset, const IConverter
 	offset += finish - (data.GetData() + offset);
 }
 
+void ServiceBook::Validate() const
+{
+	Config.Validate();
+}
+
 void ServiceBook::BuildHeader(Data &data, size_t &offset) const
 {
 	// no header in this record (?)
@@ -416,8 +429,49 @@ void ServiceBook::Clear()
 	Config.Clear();
 }
 
+const FieldHandle<ServiceBook>::ListT& ServiceBook::GetFieldHandles()
+{
+	static FieldHandle<ServiceBook>::ListT fhv;
+
+	if( fhv.size() )
+		return fhv;
+
+#undef CONTAINER_OBJECT_NAME
+#define CONTAINER_OBJECT_NAME fhv
+
+#undef RECORD_CLASS_NAME
+#define RECORD_CLASS_NAME ServiceBook
+
+	FHP(RecType, "Record Type Code");
+	FHP(RecordId, "Unique Record ID");
+
+	FHP(Name, "Name");
+	FHP(HiddenName, "Hidden Name");
+	FHP(Description, "Description");
+	FHP(DSID, "DSID");
+	FHP(BesDomain, "BES Domain");
+	FHP(UniqueId, "Unique ID");
+	FHP(ContentId, "Content ID");
+
+	// FIXME - this config is not yet implmented fully... when it is,
+	// will need to add this as a field to FieldHandle<>, and maybe
+	// implement a ServiceBookConfig::GetFieldHandles()
+	//ServiceBookConfig Config;
+
+	FHP(Unknowns, "Unknown Fields");
+
+	return fhv;
+}
+
+std::string ServiceBook::GetDescription() const
+{
+	return Name;
+}
+
 inline void FormatStr(std::ostream &os, const char *name, const std::string &str)
 {
+	ios_format_state state(os);
+
 	if( str.size() ) {
 		os << "   " << setw(20) << name;
 		os << ": " << str << "\n";
@@ -426,8 +480,10 @@ inline void FormatStr(std::ostream &os, const char *name, const std::string &str
 
 void ServiceBook::Dump(std::ostream &os) const
 {
-	ios::fmtflags oldflags = os.setf(ios::left);
-	char fill = os.fill(' ');
+	ios_format_state state(os);
+
+	os.setf(ios::left);
+	os.fill(' ');
 
 	os << "ServiceBook entry: 0x" << setbase(16) << RecordId
 		<< " (" << (unsigned int)RecType << ")\n";
@@ -444,10 +500,18 @@ void ServiceBook::Dump(std::ostream &os) const
 
 	// print any unknowns
 	os << Unknowns;
+}
 
-	// cleanup the stream
-	os.flags(oldflags);
-	os.fill(fill);
+bool ServiceBook::operator<(const ServiceBook &other) const
+{
+	int cmp = BesDomain.compare(other.BesDomain);
+	if( cmp == 0 )
+		cmp = DSID.compare(other.DSID);
+	if( cmp == 0 )
+		cmp = Name.compare(other.Name);
+	if( cmp == 0 )
+		cmp = UniqueId.compare(other.UniqueId);
+	return cmp < 0;
 }
 
 } // namespace Barry
