@@ -5,7 +5,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ class IConverter;
 class BXEXPORT ServiceBookConfig
 {
 public:
-	typedef std::vector<UnknownField>		UnknownsType;
+	typedef Barry::UnknownsType			UnknownsType;
 
 	uint8_t Format;
 
@@ -65,6 +65,7 @@ public:
 	~ServiceBookConfig();
 
 	// Parser / Builder API (see parser.h / builder.h)
+	void Validate() const;
 	void ParseHeader(const Data &data, size_t &offset);
 	void ParseFields(const Data &data, size_t &offset, const IConverter *ic = 0);
 	void BuildHeader(Data &data, size_t &offset) const;
@@ -112,6 +113,7 @@ public:
 	~ServiceBook();
 
 	// Parser / Builder API (see parser.h / builder.h)
+	void Validate() const;
 	uint8_t GetRecType() const { return RecType; }
 	uint32_t GetUniqueId() const { return RecordId; }
 	void SetIds(uint8_t Type, uint32_t Id) { RecType = Type; RecordId = Id; }
@@ -120,16 +122,20 @@ public:
 	void BuildHeader(Data &data, size_t &offset) const;
 	void BuildFields(Data &data, size_t &offset, const IConverter *ic = 0) const;
 
+	// operations (common among record classes)
 	void Clear();
-
 	void Dump(std::ostream &os) const;
+	std::string GetDescription() const;
 
 	// sorting
-	bool operator<(const ServiceBook &other) const { return RecordId < RecordId; }
+	bool operator<(const ServiceBook &other) const;
 
 	// database name
 	static const char * GetDBName() { return "Service Book"; }
 	static uint8_t GetDefaultRecType() { return 0; }
+
+	// Generic Field Handle support
+	static const FieldHandle<ServiceBook>::ListT& GetFieldHandles();
 };
 
 BXEXPORT inline std::ostream& operator<<(std::ostream &os, const ServiceBook &msg) {
