@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
     Copyright (C) 2007, Brian Edginton
 
     This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 
 #include "r_task.h"
 #include "r_calendar.h"                        // for CAL_* defines
+#include "r_recur_base-int.h"
 #include "record-internal.h"
 #include "protostructs.h"
 #include "data.h"
@@ -31,6 +32,7 @@
 #include <ostream>
 #include <iomanip>
 #include <string.h>
+#include "ios_state.h"
 
 using namespace std;
 using namespace Barry::Protocol;
@@ -97,6 +99,7 @@ uint8_t Task::StatusRec2Proto(StatusFlagType s)
 #define TSKFC_ALARM_TIME	0x0f
 #define TSKFC_TIMEZONE_CODE	0x10
 #define TSKFC_CATEGORIES	0x11
+#define TSKFC_ALARM_FLAG	0x12
 #define TSKFC_END		0xffff
 
 static FieldLink<Task> TaskFieldLinks[] = {
@@ -152,12 +155,13 @@ const unsigned char* Task::ParseField(const unsigned char *begin,
 				return begin;   // done!
 			}
 			else if( b->timeMember && btohs(field->size) == 4 ) {
-				time_t &t = this->*(b->timeMember);
-				t = min2time(field->u.min1900);
+				TimeT &t = this->*(b->timeMember);
+				t.Time = min2time(field->u.min1900);
 				return begin;
 			}
 		}
 	}
+
 	// handle special cases
 	switch( field->type )
 	{
@@ -190,7 +194,17 @@ const unsigned char* Task::ParseField(const unsigned char *begin,
 		return begin;
 
 	case TSKFC_DUE_FLAG:
-		DueDateFlag = field->u.raw[0];
+		// the DueDateFlag is not available on really old devices
+		// such as the 7750, and if the DueTime is available,
+		// we'll just save it.  There is no further need for this
+		// value that we know of yet, so just ignore it for now.
+		return begin;
+
+	case TSKFC_ALARM_FLAG:
+		// the AlarmFlag is not available on really old devices
+		// such as the 7750, and if the AlarmTime is available,
+		// we'll just save it.  There is no further need for this
+		// value that we know of yet, so just ignore it for now.
 		return begin;
 
 	case TSKFC_ALARM_TYPE:
@@ -211,6 +225,7 @@ const unsigned char* Task::ParseField(const unsigned char *begin,
 		}
 		return begin;
 	}
+
 	// base class handles recurring data
 	if( RecurBase::ParseField(field->type, field->u.raw, btohs(field->size), ic) )
 		return begin;
@@ -238,6 +253,11 @@ void Task::ParseFields(const Data &data, size_t &offset, const IConverter *ic)
 }
 
 
+void Task::Validate() const
+{
+	RecurBase::Validate();
+}
+
 void Task::BuildHeader(Data &data, size_t &offset) const
 {
 	// no header in Task records
@@ -253,46 +273,49 @@ void Task::BuildFields(Data &data, size_t &offset, const IConverter *ic) const
 {
 	data.Zap();
 
+	//
+	// Note: we do not use the FieldLink table, since the firmware
+	// on many devices apears flakey, so we try to write as close to
+	// the same order as we see coming from the device.  Unfortunately,
+	// this doesn't really help, for the Tasks corruption bug, but
+	// we'll keep the flexibility for now.
+	//
+
 	// tack on the 't' task type field first
 	BuildField(data, offset, TSKFC_TASK_TYPE, 't');
 
-	BuildField(data, offset, TSKFC_STATUS, StatusRec2Proto(StatusFlag));
-	BuildField(data, offset, TSKFC_PRIORITY, PriorityRec2Proto(PriorityFlag));
-	BuildField(data, offset, TSKFC_ALARM_TYPE, AlarmRec2Proto(AlarmType));
+	// Summary / Title
+	if( Summary.size() ) {
+		std::string s = (ic) ? ic->ToBB(Summary) : Summary;
+		BuildField(data, offset, TSKFC_TITLE, s);
+	}
 
-	if ( DueDateFlag )
-		BuildField(data, offset, TSKFC_DUE_FLAG, (char) 1);
-	else
-		BuildField(data, offset, TSKFC_DUE_FLAG, (char) 0);
+	BuildField(data, offset, TSKFC_STATUS, (uint32_t) StatusRec2Proto(StatusFlag));
+	BuildField(data, offset, TSKFC_PRIORITY, (uint32_t) PriorityRec2Proto(PriorityFlag));
 
-	if( TimeZoneValid )
-		BuildField(data, offset, TSKFC_TIMEZONE_CODE, TimeZoneCode);
+	if( TimeZoneValid ) {
+		// the time zone code field is 4 bytes, but we only use
+		// the first two... pad it with zeros
+		uint32_t code = TimeZoneCode;
+		BuildField(data, offset, TSKFC_TIMEZONE_CODE, code);
+	}
 
-	// cycle through the type table
-	for(	FieldLink<Task> *b = TaskFieldLinks;
-		b->type != TSKFC_END;
-		b++ )
-	{
-		// print only fields with data
-		if( b->strMember ) {
-			const std::string &field = this->*(b->strMember);
-			if( field.size() ) {
-				std::string s = (b->iconvNeeded && ic) ? ic->ToBB(field) : field;
-				BuildField(data, offset, b->type, s);
-			}
-		}
-		else if( b->timeMember ) {
-			time_t t = this->*(b->timeMember);
-			if( t > 0 )
-				BuildField1900(data, offset, b->type, t);
-		}
-		else if( b->postMember && b->postField ) {
-			const std::string &field = (this->*(b->postMember)).*(b->postField);
-			if( field.size() ) {
-				std::string s = (b->iconvNeeded && ic) ? ic->ToBB(field) : field;
-				BuildField(data, offset, b->type, s);
-			}
-		}
+	// make sure StartTime matches DueTime, by writing it manually...
+	/// not sure why StartTime exists, but oh well. :-)
+	if( DueTime.IsValid() ) {
+		// we use DueTime here, with the START_TIME code,
+		// instead of StartTime, since the function is const
+		BuildField1900(data, offset, TSKFC_START_TIME, DueTime);
+
+		// then DueTime, with flag first, then time
+		BuildField(data, offset, TSKFC_DUE_FLAG, (uint32_t) 1);
+		BuildField1900(data, offset, TSKFC_DUE_TIME, DueTime);
+	}
+
+	if( AlarmTime.IsValid() ) {
+		BuildField(data, offset, TSKFC_ALARM_FLAG, (uint32_t) 1);
+		BuildField(data, offset, TSKFC_ALARM_TYPE, AlarmRec2Proto(AlarmType));
+		BuildField1900(data, offset, TSKFC_ALARM_TIME, AlarmTime);
 	}
 
 	// Categories
@@ -302,6 +325,19 @@ void Task::BuildFields(Data &data, size_t &offset, const IConverter *ic) const
 		BuildField(data, offset, TSKFC_CATEGORIES, ic ? ic->ToBB(store) : store);
 	}
 
+	// Notes
+	if( Notes.size() ) {
+		std::string s = (ic) ? ic->ToBB(Notes) : Notes;
+		BuildField(data, offset, TSKFC_NOTES, s);
+	}
+
+	if( Recurring ) {
+		CalendarRecurrenceDataField recur;
+		BuildRecurrenceData(StartTime.Time, &recur);
+		BuildField(data, offset, RecurBase::RecurringFieldType(),
+			&recur, CALENDAR_RECURRENCE_DATA_FIELD_SIZE);
+	}
+
 	// and finally save unknowns
 	UnknownsType::const_iterator
 		ub = Unknowns.begin(), ue = Unknowns.end();
@@ -316,27 +352,92 @@ void Task::BuildFields(Data &data, size_t &offset, const IConverter *ic) const
 
 void Task::Clear()
 {
+	// clear the base class first
 	RecurBase::Clear();
 
+	// our variables...
+	RecType = GetDefaultRecType();
+	RecordId = 0;
+
 	Summary.clear();
 	Notes.clear();
 	Categories.clear();
-	StartTime = DueTime = AlarmTime = 0;
+	UID.clear();
 
-	PriorityFlag = (PriorityFlagType)0;
-	StatusFlag = (StatusFlagType)0;
-	AlarmType = (AlarmFlagType)0;
+	StartTime.clear();
+	DueTime.clear();
+	AlarmTime.clear();
 
-	DueDateFlag = false;
-
-	TimeZoneCode = GetTimeZoneCode( 0, 0 );	// default to GMT
+	TimeZoneCode = GetStaticTimeZoneCode( 0, 0 );	// default to GMT
 	TimeZoneValid = false;
 
+	AlarmType = Date;
+	PriorityFlag = Normal;
+	StatusFlag = NotStarted;
+
 	Unknowns.clear();
 }
 
+const FieldHandle<Task>::ListT& Task::GetFieldHandles()
+{
+	static FieldHandle<Task>::ListT fhv;
+
+	if( fhv.size() )
+		return fhv;
+
+#undef CONTAINER_OBJECT_NAME
+#define CONTAINER_OBJECT_NAME fhv
+
+#undef RECORD_CLASS_NAME
+#define RECORD_CLASS_NAME Task
+
+	FHP(RecType, "Record Type Code");
+	FHP(RecordId, "Unique Record ID");
+
+	FHD(Summary, "Summary", TSKFC_TITLE, true);
+	FHD(Notes, "Notes", TSKFC_NOTES, true);
+	FHD(Categories, "Categories", TSKFC_CATEGORIES, true);
+	FHP(UID, "UID");	// FIXME - not linked to any device field??
+
+	FHD(StartTime, "Start Time", TSKFC_START_TIME, false);
+	FHD(DueTime, "Due Time", TSKFC_DUE_TIME, false);
+	FHD(AlarmTime, "Alarm Time", TSKFC_ALARM_TIME, false);
+	FHD(TimeZoneCode, "Time Zone Code", TSKFC_TIMEZONE_CODE, false);
+	FHP(TimeZoneValid, "Time Zone Code Valid");
+
+	FHE(aft, AlarmFlagType, AlarmType, "Alarm Type");
+	FHE_CONST(aft, Date, "Date");
+	FHE_CONST(aft, Relative, "Relative");
+
+	FHE(pft, PriorityFlagType, PriorityFlag, "Priority");
+	FHE_CONST(pft, High, "High");
+	FHE_CONST(pft, Normal, "Normal");
+	FHE_CONST(pft, Low, "Low");
+
+	FHE(sft, StatusFlagType, StatusFlag, "Status");
+	FHE_CONST(sft, NotStarted, "Not Started");
+	FHE_CONST(sft, InProgress, "In Progress");
+	FHE_CONST(sft, Completed, "Completed");
+	FHE_CONST(sft, Waiting, "Waiting");
+	FHE_CONST(sft, Deferred, "Deferred");
+
+	FHP(Unknowns, "Unknown Fields");
+
+	// and finally, the RecurBase fields
+	RECUR_BASE_FIELD_HANDLES
+
+	return fhv;
+}
+
+std::string Task::GetDescription() const
+{
+	return Summary;
+}
+
 void Task::Dump(std::ostream &os) const
 {
+	ios_format_state state(os);
+
 	static const char *PriorityName[] = { "High", "Normal", "Low" };
 	static const char *StatusName[] = { "Not Started", "In Progress",
 		"Completed", "Waiting", "Deferred" };
@@ -357,23 +458,22 @@ void Task::Dump(std::ostream &os) const
 		if( b->strMember ) {
 			const std::string &s = this->*(b->strMember);
 			if( s.size() )
-				os << "   " << b->name << ": " << s << "\n";
+				os << "   " << b->name << ": " << Cr2LfWrapper(s) << "\n";
 		}
 		else if( b->timeMember ) {
-			time_t t = this->*(b->timeMember);
-			if( t > 0 )
-				os << "   " << b->name << ": " << ctime(&t);
+			TimeT t = this->*(b->timeMember);
+			if( t.IsValid() )
+				os << "   " << b->name << ": " << t << "\n";
 		}
 	}
 
-	os << "   Due Date Flag: " << (DueDateFlag ? "true" : "false") << "\n";
 	os << "   Priority: " << PriorityName[PriorityFlag] << "\n";
 	os << "   Status: " << StatusName[StatusFlag] << "\n";
 	if( AlarmType ) {
 		os << "   Alarm Type: " << AlarmTypeName[AlarmType] << "\n";
 	}
 	if( TimeZoneValid )
-		os << "   Time Zone: " << GetTimeZone(TimeZoneCode)->Name << "\n";
+		os << "   Time Zone: " << GetStaticTimeZone(TimeZoneCode)->Name << "\n";
 
 	// print recurrence data if available
 	os << "   Recurring: " << (Recurring ? "yes" : "no") << "\n";
@@ -437,19 +537,31 @@ void Task::Dump(std::ostream &os) const
 		if( Perpetual )
 			os << "      Ends: never\n";
 		else
-			os << "      Ends: " << ctime(&RecurringEndTime);
+			os << "      Ends: " << RecurringEndTime << "\n";
 	}
 
 	if( Categories.size() ) {
 		string display;
 		Categories.CategoryList2Str(display);
-		os << "    Categories: " << display << "\n";
+		os << "   Categories: " << display << "\n";
 	}
 
 	os << Unknowns;
 	os << "\n\n";
 }
 
+bool Task::operator<(const Task &other) const
+{
+	if( StartTime != other.StartTime )
+		return StartTime < other.StartTime;
+	if( AlarmTime != other.AlarmTime )
+		return AlarmTime < other.AlarmTime;
+
+	int cmp = Summary.compare(other.Summary);
+	if( cmp == 0 )
+		cmp = Notes.compare(other.Notes);
+	return cmp < 0;
+}
 
 } // namespace Barry
 
