@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
     Copyright (C) 2007, Brian Edginton
 
     This program is free software; you can redistribute it and/or modify
@@ -35,10 +35,33 @@ namespace Barry {
 // forward declarations
 class IConverter;
 
+//
+// Task record class
+//
+/// Task record class.  Note that there is a bug in many device firmwares.
+///
+/// If you extract a Tasks record, and then write it back via
+/// SetRecordByIndex(), on many devices that I tested, it ends up
+/// corrupting the record on the device, and the GUI on the device
+/// appears messed up.  (It shows the first few fields twice)
+/// Such a corrupt record also loses the due date.
+///
+/// The workaround, when working with the Tasks database, is to
+/// first DeleteByIndex() and then AddRecord() via the Desktop mode class,
+/// using the same record ID.  This works, but is unfortunately
+/// cumbersome.
+///
+/// See the Desktop GUI and the opensync plugins for examples of this
+/// workaround.
+///
+/// Ideally, we should test a Tasks sync on Windows, and see how
+/// the Windows software handles this.  There may be some protocol
+/// changes that will be needed in future Barry versions.
+///
 class BXEXPORT Task : public RecurBase
 {
 public:
-	typedef std::vector<UnknownField>		UnknownsType;
+	typedef Barry::UnknownsType			UnknownsType;
 
 	uint8_t RecType;
 	uint32_t RecordId;
@@ -48,10 +71,14 @@ public:
 	CategoryList Categories;
 	std::string UID;
 
-	time_t StartTime;
-	time_t DueTime;
-	time_t AlarmTime;
-	unsigned short TimeZoneCode;
+	Barry::TimeT StartTime;		// most devices set this the same as
+					// DueTime... this is a read-only
+					// variable.  By setting DueTime,
+					// the library will write StartTime
+					// for you automatically
+	Barry::TimeT DueTime;
+	Barry::TimeT AlarmTime;
+	uint16_t TimeZoneCode;
 	bool TimeZoneValid;		// true if the record contained a
 					// time zone code
 
@@ -80,8 +107,6 @@ public:
 	};
 	StatusFlagType StatusFlag;
 
-	bool DueDateFlag;	// true if due date is set
-
 	// unknown
 	UnknownsType Unknowns;
 
@@ -99,6 +124,8 @@ public:
 	Task();
 	~Task();
 
+	// Parser / Builder API (see parser.h / builder.h)
+	void Validate() const;
 	const unsigned char* ParseField(const unsigned char *begin,
 		const unsigned char *end, const IConverter *ic = 0);
 	uint8_t GetRecType() const { return RecType; }
@@ -109,15 +136,19 @@ public:
 	void BuildHeader(Data &data, size_t &offset) const;
 	void BuildFields(Data &data, size_t &offset, const IConverter *ic = 0) const;
 
+	// operations (common among record classes)
 	void Clear();
-
 	void Dump(std::ostream &os) const;
-	bool operator<(const Task &other) const { return Summary < other.Summary; }
+	std::string GetDescription() const;
+
+	bool operator<(const Task &other) const;
 
 	// database name
 	static const char * GetDBName() { return "Tasks"; }
 	static uint8_t GetDefaultRecType() { return 2; }
 
+	// Generic Field Handle support
+	static const FieldHandle<Task>::ListT& GetFieldHandles();
 };
 
 BXEXPORT inline std::ostream& operator<<(std::ostream &os, const Task &msg) {
