@@ -8,7 +8,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -54,11 +54,15 @@ const unsigned char*  ParseCommonFields(RecordT &rec,
 	return b;
 }
 
+// Does not exist, so error will be caught by linker instead of at runtime.
+void ConvertHtoB_Not_Implemented();
+
 // Use templates here to guarantee types are converted in the strictest manner.
 template <class SizeT>
 inline SizeT ConvertHtoB(SizeT s)
 {
-	throw Error("Not implemented.");
+	ConvertHtoB_Not_Implemented();
+//	throw Error("Not implemented.");
 }
 
 // specializations for specific sizes
@@ -77,17 +81,22 @@ struct FieldLink
 	const char *objectClass;
 	std::string RecordT::* strMember;	// FIXME - find a more general
 	EmailAddressList RecordT::* addrMember;	// way to do this...
-	time_t RecordT::* timeMember;
+	Barry::TimeT RecordT::* timeMember;
 	PostalAddress RecordT::* postMember;
 	std::string PostalAddress::* postField;
 	bool iconvNeeded;
 };
 
 void BuildField1900(Data &data, size_t &size, uint8_t type, time_t t);
+inline void BuildField1900(Data &data, size_t &size, uint8_t type, const Barry::TimeT &t)
+{
+	BuildField1900(data, size, type, t.Time);
+}
 void BuildField(Data &data, size_t &size, uint8_t type, char c);
 void BuildField(Data &data, size_t &size, uint8_t type, uint8_t c);
 void BuildField(Data &data, size_t &size, uint8_t type, uint16_t value);
 void BuildField(Data &data, size_t &size, uint8_t type, uint32_t value);
+void BuildField(Data &data, size_t &size, uint8_t type, uint64_t value);
 void BuildField(Data &data, size_t &size, uint8_t type, const std::string &str);
 void BuildField(Data &data, size_t &size, uint8_t type, const void *buf, size_t bufsize);
 void BuildField(Data &data, size_t &size, const Barry::UnknownField &field);
@@ -95,6 +104,13 @@ void BuildField(Data &data, size_t &size, uint8_t type, const Barry::Protocol::G
 std::string ParseFieldString(const Barry::Protocol::CommonField *field);
 std::string ParseFieldString(const void *data, uint16_t maxlen);
 
+
+// Functions to help build JDWP command packets
+void AddJDWByte(Barry::Data &data, size_t &size, const uint8_t value);
+void AddJDWInt(Barry::Data &data, size_t &size, const uint32_t value);
+void AddJDWChar(Barry::Data &data, size_t &size, const void *buf, size_t bufsize);
+void AddJDWString(Barry::Data &data, size_t &size, const std::string &str);
+
 } // namespace Barry
 
 #endif
