@@ -1,15 +1,12 @@
 ///
 /// \file	record.cc
-///		Blackberry database record classes.  Help translate data
-///		from data packets to useful structurs, and back.
-///		This header provides the common types and classes
-///		used by the general record parser classes in the
-///		r_*.h files.  Only application-safe API stuff goes in
-///		here.  Internal library types go in record-internal.h
+///		Misc. Blackberry database record helper classes and functions.
+///		Helps translate data from data packets to useful structures,
+///		and back.
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,12 +23,14 @@
 
 #include "record.h"
 #include "record-internal.h"
-#include "protocol.h"
 #include "protostructs.h"
 #include "data.h"
 #include "time.h"
 #include "error.h"
 #include "endian.h"
+#include "trim.h"
+#include "ios_state.h"
+#include "parser.h"
 #include <sstream>
 #include <iomanip>
 #include <time.h>
@@ -39,6 +38,24 @@
 #include <stdio.h>			// for sscanf()
 #include <stdexcept>
 
+// for DBNamedFieldCmp below
+#include "recordtmpl.h"
+#include "r_calendar.h"
+#include "r_calllog.h"
+#include "r_bookmark.h"
+#include "r_contact.h"
+#include "r_cstore.h"
+#include "r_memo.h"
+#include "r_message.h"
+#include "r_servicebook.h"
+#include "r_task.h"
+#include "r_pin_message.h"
+#include "r_saved_message.h"
+#include "r_sms.h"
+#include "r_folder.h"
+#include "r_timezone.h"
+#include "r_hhagent.h"
+
 #define __DEBUG_MODE__
 #include "debug.h"
 
@@ -47,6 +64,35 @@ using namespace Barry::Protocol;
 
 namespace Barry {
 
+std::ostream& operator<< (std::ostream &os, const Cr2LfWrapper &str)
+{
+	for(	std::string::const_iterator i = str.m_str.begin();
+		i != str.m_str.end() && *i;
+		i++)
+	{
+		if( *i == '\r' )
+			os << '\n';
+		else
+			os << *i;
+	}
+	return os;
+}
+
+std::ostream& operator<< (std::ostream &os, const TimeT &t)
+{
+	// strip the trailing newline
+	string output = ctime(&t.Time);
+	while( output.size() &&
+		(output[output.size()-1] == '\n' ||
+			output[output.size()-1] == '\r') )
+	{
+		output.resize(output.size() - 1);
+	}
+
+	os << output;
+	return os;
+}
+
 //////////////////////////////////////////////////////////////////////////////
 // Field builder helper functions
 
@@ -109,7 +155,23 @@ void BuildField(Data &data, size_t &size, uint8_t type, uint32_t value)
 	field->size = htobl(strsize);
 	field->type = type;
 
-	uint32_t store = htobs(value);
+	uint32_t store = htobl(value);
+	memcpy(field->u.raw, &store, strsize);
+
+	size += fieldsize;
+}
+
+void BuildField(Data &data, size_t &size, uint8_t type, uint64_t value)
+{
+	size_t strsize = 8;
+	size_t fieldsize = COMMON_FIELD_HEADER_SIZE + strsize;
+	unsigned char *pd = data.GetBuffer(size + fieldsize) + size;
+	CommonField *field = (CommonField *) pd;
+
+	field->size = htobl(strsize);
+	field->type = type;
+
+	uint64_t store = htobll(value);
 	memcpy(field->u.raw, &store, strsize);
 
 	size += fieldsize;
@@ -178,337 +240,12 @@ std::string ParseFieldString(const void *data, uint16_t maxlen)
 
 
 ///////////////////////////////////////////////////////////////////////////////
-// CommandTable class
-
-CommandTable::CommandTable()
-{
-}
-
-CommandTable::~CommandTable()
-{
-}
-
-const unsigned char* CommandTable::ParseField(const unsigned char *begin,
-					      const unsigned char *end)
-{
-	// check if there is enough data for a header
-	const unsigned char *headend = begin + sizeof(CommandTableField);
-	if( headend > end )
-		return headend;
-
-	const CommandTableField *field = (const CommandTableField *) begin;
-
-	// advance and check size
-	begin += COMMAND_FIELD_HEADER_SIZE + field->size;	// size is byte
-	if( begin > end )		// if begin==end, we are ok
-		return begin;
-
-	if( !field->size )		// if field has no size, something's up
-		return begin;
-
-	Command command;
-	command.Code = field->code;
-	command.Name.assign((const char *)field->name, field->size);
-	Commands.push_back(command);
-	return begin;
-}
-
-void CommandTable::Parse(const Data &data, size_t offset)
-{
-	if( offset >= data.GetSize() )
-		return;
-
-	const unsigned char *begin = data.GetData() + offset;
-	const unsigned char *end = data.GetData() + data.GetSize();
-
-	while( begin < end )
-		begin = ParseField(begin, end);
-}
-
-void CommandTable::Clear()
-{
-	Commands.clear();
-}
-
-unsigned int CommandTable::GetCommand(const std::string &name) const
-{
-	CommandArrayType::const_iterator b = Commands.begin();
-	for( ; b != Commands.end(); b++ )
-		if( b->Name == name )
-			return b->Code;
-	return 0;
-}
-
-void CommandTable::Dump(std::ostream &os) const
-{
-	CommandArrayType::const_iterator b = Commands.begin();
-	os << "Command table:\n";
-	for( ; b != Commands.end(); b++ ) {
-		os << "    Command: 0x" << setbase(16) << b->Code
-		   << " '" << b->Name << "'\n";
-	}
-}
-
-
-
-///////////////////////////////////////////////////////////////////////////////
-// RecordStateTable class
-
-RecordStateTable::RecordStateTable()
-	: m_LastNewRecordId(1)
-{
-}
-
-RecordStateTable::~RecordStateTable()
-{
-}
-
-const unsigned char* RecordStateTable::ParseField(const unsigned char *begin,
-						  const unsigned char *end)
-{
-	const RecordStateTableField *field = (const RecordStateTableField *) begin;
-
-	// advance and check size
-	begin += sizeof(RecordStateTableField);
-	if( begin > end )		// if begin==end, we are ok
-		return begin;
-
-	State state;
-	state.Index = btohs(field->index);
-	state.RecordId = btohl(field->uniqueId);
-	state.Dirty = (field->flags & BARRY_RSTF_DIRTY) != 0;
-	state.RecType = field->rectype;
-	state.Unknown2.assign((const char*)field->unknown2, sizeof(field->unknown2));
-	StateMap[state.Index] = state;
-
-	return begin;
-}
-
-void RecordStateTable::Parse(const Data &data)
-{
-	size_t offset = 12;	// skipping the unknown 2 bytes at start
-
-	if( offset >= data.GetSize() )
-		return;
-
-	const unsigned char *begin = data.GetData() + offset;
-	const unsigned char *end = data.GetData() + data.GetSize();
-
-	while( begin < end )
-		begin = ParseField(begin, end);
-}
-
-void RecordStateTable::Clear()
-{
-	StateMap.clear();
-	m_LastNewRecordId = 1;
-}
-
-// Searches the StateMap table for RecordId, and returns the "index"
-// in the map if found.  Returns true if found, false if not.
-// pFoundIndex can be null if only the existence of the index is desired
-bool RecordStateTable::GetIndex(uint32_t RecordId, IndexType *pFoundIndex) const
-{
-	StateMapType::const_iterator i = StateMap.begin();
-	for( ; i != StateMap.end(); ++i ) {
-		if( i->second.RecordId == RecordId ) {
-			if( pFoundIndex )
-				*pFoundIndex = i->first;
-			return true;
-		}
-	}
-	return false;
-}
-
-// Generate a new RecordId that is not in the state table.
-// Starts at 1 and keeps incrementing until a free one is found.
-uint32_t RecordStateTable::MakeNewRecordId() const
-{
-	// start with next Id
-	m_LastNewRecordId++;
-
-	// make sure it doesn't already exist
-	StateMapType::const_iterator i = StateMap.begin();
-	while( i != StateMap.end() ) {
-		if( m_LastNewRecordId == i->second.RecordId ) {
-			m_LastNewRecordId++;		// try again
-			i = StateMap.begin();		// start over
-		}
-		else {
-			++i;				// next State
-		}
-	}
-	return m_LastNewRecordId;
-}
-
-void RecordStateTable::Dump(std::ostream &os) const
-{
-	ios::fmtflags oldflags = os.setf(ios::right);
-	char fill = os.fill(' ');
-	bool bPrintAscii = Data::PrintAscii();
-	Data::PrintAscii(false);
-
-	os << "  Index  RecordId    Dirty  RecType" << endl;
-	os << "-------  ----------  -----  -------" << endl;
-
-	StateMapType::const_iterator b, e = StateMap.end();
-	for( b = StateMap.begin(); b != e ; ++b ) {
-		const State &state = b->second;
-
-		os.fill(' ');
-		os << setbase(10) << setw(7) << state.Index;
-		os << "  0x" << setbase(16) << setfill('0') << setw(8) << state.RecordId;
-		os << "  " << setfill(' ') << setw(5) << (state.Dirty ? "yes" : "no");
-		os << "     0x" << setbase(16) << setfill('0') << setw(2) << state.RecType;
-		os << "   " << Data(state.Unknown2.data(), state.Unknown2.size());
-	}
-
-	// cleanup the stream
-	os.flags(oldflags);
-	os.fill(fill);
-	Data::PrintAscii(bPrintAscii);
-}
-
-
-
-///////////////////////////////////////////////////////////////////////////////
-// DatabaseDatabase class
-
-DatabaseDatabase::DatabaseDatabase()
-{
-}
-
-DatabaseDatabase::~DatabaseDatabase()
-{
-}
-
-template <class RecordType, class FieldType>
-void DatabaseDatabase::ParseRec(const RecordType &rec, const unsigned char *end)
-{
-}
-
-template <class FieldType>
-const unsigned char* DatabaseDatabase::ParseField(const unsigned char *begin,
-						  const unsigned char *end)
-{
-	// check if there is enough data for a header
-	const unsigned char *headend = begin + sizeof(FieldType);
-	if( headend > end )
-		return headend;
-
-	// get our header
-	const FieldType *field = (const FieldType *) begin;
-
-	// advance and check size
-	begin += sizeof(FieldType) - sizeof(field->name) + ConvertHtoB(field->nameSize);
-	if( begin > end )		// if begin==end, we are ok
-		return begin;
-
-	if( !ConvertHtoB(field->nameSize) ) // if field has no size, something's up
-		return begin;
-
-	Database db;
-	db.Number = ConvertHtoB(field->dbNumber);
-	db.RecordCount = ConvertHtoB(field->dbRecordCount);
-	db.Name.assign((const char *)field->name, ConvertHtoB(field->nameSize) - 1);
-	Databases.push_back(db);
-	return begin;
-}
-
-void DatabaseDatabase::Parse(const Data &data)
-{
-	// check size to make sure we have up to the DBAccess operation byte
-	if( data.GetSize() < (SB_PACKET_DBACCESS_HEADER_SIZE + 1) )
-		return;
-
-	MAKE_PACKET(pack, data);
-	const unsigned char *begin = 0;
-	const unsigned char *end = data.GetData() + data.GetSize();
-
-	switch( pack->u.db.u.response.operation )
-	{
-	case SB_DBOP_GET_DBDB:
-		// using the new protocol
-		if( data.GetSize() > SB_PACKET_DBDB_HEADER_SIZE ) {
-			begin = (const unsigned char *)
-				&pack->u.db.u.response.u.dbdb.field[0];
-
-			// this while check is ok, since ParseField checks
-			// for header size
-			while( begin < end )
-				begin = ParseField<DBDBField>(begin, end);
-		}
-		else
-			dout("DatabaseDatabase: not enough data for parsing");
-		break;
-
-	case SB_DBOP_OLD_GET_DBDB:
-		// using the old protocol
-		if( data.GetSize() > SB_PACKET_OLD_DBDB_HEADER_SIZE ) {
-			begin = (const unsigned char *)
-				&pack->u.db.u.response.u.old_dbdb.field[0];
-
-			// this while check is ok, since ParseField checks
-			// for header size
-			while( begin < end )
-				begin = ParseField<OldDBDBField>(begin, end);
-		}
-		else
-			dout("DatabaseDatabase: not enough data for parsing");
-		break;
-
-	default:
-		// unknown protocol
-		dout("Unknown protocol");
-		break;
-	}
-
-
-}
-
-void DatabaseDatabase::Clear()
-{
-	Databases.clear();
-}
-
-bool DatabaseDatabase::GetDBNumber(const std::string &name,
-				   unsigned int &number) const
-{
-	DatabaseArrayType::const_iterator b = Databases.begin();
-	for( ; b != Databases.end(); b++ )
-		if( b->Name == name ) {
-			number = b->Number;
-			return true;
-		}
-	return false;
-}
-
-bool DatabaseDatabase::GetDBName(unsigned int number,
-				 std::string &name) const
-{
-	DatabaseArrayType::const_iterator b = Databases.begin();
-	for( ; b != Databases.end(); b++ )
-		if( b->Number == number ) {
-			name = b->Name;
-			return true;
-		}
-	return false;
-}
-
-void DatabaseDatabase::Dump(std::ostream &os) const
-{
-	DatabaseArrayType::const_iterator b = Databases.begin();
-	os << "Database database:\n";
-	for( ; b != Databases.end(); b++ ) {
-		os << "    Database: 0x" << setbase(16) << b->Number
-		   << " '" << b->Name << "' (records: "
-		   << setbase(10) << b->RecordCount << ")\n";
-	}
-}
-
+// UnknownField
 
 std::ostream& operator<< (std::ostream &os, const std::vector<UnknownField> &unknowns)
 {
+	ios_format_state state(os);
+
 	std::vector<UnknownField>::const_iterator
 		ub = unknowns.begin(), ue = unknowns.end();
 	if( ub != ue )
@@ -522,16 +259,88 @@ std::ostream& operator<< (std::ostream &os, const std::vector<UnknownField> &unk
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+// EmailList typedef
+
+std::ostream& operator<< (std::ostream &os, const EmailList &list)
+{
+	for( EmailList::const_iterator b = list.begin(), e = list.end();
+		b != e;
+		++b )
+	{
+		if( b != list.begin() )
+			os << ", ";
+		os << *b;
+	}
+	return os;
+}
 
 ///////////////////////////////////////////////////////////////////////////////
 // EmailAddress class
 
-std::ostream& operator<<(std::ostream &os, const EmailAddress &msga) {
-	os << msga.Name << " <" << msga.Email << ">";
+EmailAddress::EmailAddress(const std::string &complex_address)
+{
+	size_t end = complex_address.rfind('>');
+	size_t start = complex_address.rfind('<');
+	if( start == string::npos || end == string::npos || start > end ) {
+		// simple address, add it
+		Email = complex_address;
+		Inplace::trim(Email);
+	}
+	else {
+		Name = complex_address.substr(0, start);
+		Inplace::trim(Name);
+
+		Email = complex_address.substr(start+1, end - start - 1);
+		Inplace::trim(Email);
+	}
+}
+
+std::ostream& operator<<(std::ostream &os, const EmailAddress &msga)
+{
+	if( msga.Name.size() )
+		os << msga.Name << " <";
+	os << msga.Email;
+	if( msga.Name.size() )
+		os << ">";
 	return os;
 }
 
-std::ostream& operator<<(std::ostream &os, const EmailAddressList &elist) {
+
+///////////////////////////////////////////////////////////////////////////////
+// EmailAddressList class
+
+std::string EmailAddressList::ToCommaSeparated() const
+{
+	std::ostringstream oss;
+	oss << *this;
+	return oss.str();
+}
+
+/// Adds every email address found in the comma separated list.
+/// Does not clear() first.
+void EmailAddressList::AddCommaSeparated(const std::string &list)
+{
+	istringstream iss(list);
+	string address;
+	iss >> ws;
+
+	while( getline(iss, address, ',') ) {
+		// trim any trailing whitespace in the address
+		size_t len = address.size();
+		while( len && ::isspace(address[len-1]) )
+			address.resize(len-1);
+
+		// add to list if anything left
+		if( address.size() ) {
+			EmailAddress ea(address);
+			push_back(ea);
+		}
+	}
+}
+
+std::ostream& operator<<(std::ostream &os, const EmailAddressList &elist)
+{
 	for( EmailAddressList::const_iterator i = elist.begin(); i != elist.end(); ++i ) {
 		if( i != elist.begin() )
 			os << ", ";
@@ -621,9 +430,9 @@ std::string Date::ToYYYYMMDD() const
 {
 	std::ostringstream oss;
 	// setfill and setw not sticky.
-	oss	<< setw(4) << setfill('0') << Year
-		<< setw(2) << setfill('0') << Month + 1
-		<< setw(2) << setfill('0') << Day;
+	oss	<< setw(4) << setfill('0') << dec << Year
+		<< setw(2) << setfill('0') << dec << (Month + 1)
+		<< setw(2) << setfill('0') << dec << Day;
 	return oss.str();
 }
 
@@ -637,14 +446,17 @@ std::string Date::ToBBString() const
 {
 	std::ostringstream oss;
 	// setw() ain't 'sticky'!
-	oss	<< setw(2) << setfill('0') << Day << '/'
-		<< setw(2) << setfill('0') << Month + 1 << '/'
-		<< setw(2) << setfill('0') << Year;
+	oss	<< setw(2) << setfill('0') << dec << Day << '/'
+		<< setw(2) << setfill('0') << dec << (Month + 1) << '/'
+		<< setw(2) << setfill('0') << dec << Year;
 	return oss.str();
 }
 
 bool Date::FromTm(const struct tm *timep)
 {
+	if( !timep )
+		throw std::logic_error("NULL time pointer passed to Date::FromTm");
+
 	Year = timep->tm_year + 1900;
 	Month = timep->tm_mon;
 	Day = timep->tm_mday;
@@ -677,9 +489,15 @@ bool Date::FromYYYYMMDD(const std::string &str)
 
 std::ostream& operator<<(std::ostream &os, const Date &date)
 {
-	os	<< setw(4) << date.Year << '/'
-		<< setw(2) << date.Month << '/'
-		<< setw(2) << date.Day;
+	ios_format_state state(os);
+
+	os.setf(ios::right);
+	os.fill('0');
+
+	os	<< setw(4) << dec << date.Year << '/'
+		<< setw(2) << dec << (date.Month + 1) << '/'
+		<< setw(2) << dec << date.Day;
+
 	return os;
 }
 
@@ -740,6 +558,93 @@ void CategoryList::CategoryList2Str(std::string &str) const
 	}
 }
 
+std::ostream& operator<<(std::ostream &os, const CategoryList &cl)
+{
+	string buf;
+	cl.CategoryList2Str(buf);
+	os << buf;
+	return os;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// EnumConstants class
+
+void EnumConstants::AddConstant(const char *name,
+				const std::string &display,
+				int val)
+{
+	m_constants.push_back(EnumConstant(name, display, val));
+}
+
+const EnumConstants::EnumConstant& EnumConstants::GetConstant(int value) const
+{
+	for( EnumConstantList::const_iterator b = m_constants.begin(), e = m_constants.end();
+		b != e;
+		++b )
+	{
+		if( b->Value == value )
+			return *b;
+	}
+
+	// not found in list
+	throw std::logic_error("Enum value not found in constant list");
+}
+
+const char* EnumConstants::GetName(int value) const
+{
+	return GetConstant(value).Name;
+}
+
+const std::string& EnumConstants::GetDisplayName(int value) const
+{
+	return GetConstant(value).DisplayName;
+}
+
+bool EnumConstants::IsConstantValid(int value) const
+{
+	for( EnumConstantList::const_iterator b = m_constants.begin(), e = m_constants.end();
+		b != e;
+		++b )
+	{
+		if( b->Value == value )
+			return true;
+	}
+
+	// not found in list, so not a valid constant
+	return false;
+}
+
+
+//////////////////////////////////////////////////////////////////////////////
+// DBNamedFieldCmp class
+
+DBNamedFieldCmp::DBNamedFieldCmp(const std::string &field_name,
+					const Barry::IConverter *ic)
+	: m_name(field_name)
+	, m_ic(ic)
+{
+}
+
+bool DBNamedFieldCmp::operator() (const Barry::DBData &a,
+				const Barry::DBData &b) const
+{
+#undef HANDLE_PARSER
+#define HANDLE_PARSER(tname) \
+	else if( tname::GetDBName() == a.GetDBName() ) { \
+		tname rec1, rec2; \
+		ParseDBData(a, rec1, m_ic); \
+		ParseDBData(b, rec2, m_ic); \
+		return NamedFieldCmp<tname>(m_name).operator()(rec1, rec2); \
+	}
+
+	if( a.GetDBName() != b.GetDBName() ) {
+		throw logic_error("Different database types in DBNamedFieldCmp");
+	}
+	// fall through and use else's
+	ALL_KNOWN_PARSER_TYPES
+
+	throw logic_error("Unknown database in DBNamedFieldCmp::operator()");
+}
 
 } // namespace Barry
 
