@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2008, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,8 +29,8 @@
 #include <stdexcept>
 #include <pthread.h>
 #include "dataqueue.h"
-
-namespace Usb { class Device; }
+#include "error.h"
+#include "usbwrap.h"
 
 namespace Barry {
 
@@ -41,16 +41,53 @@ class BXEXPORT SocketRoutingQueue
 	friend class DataHandle;
 
 public:
-	typedef void (*SocketDataHandler)(void *ctx, Data*);	//< See RegisterInterest() for information on this callback.
+	// Interface class for socket data callbacks
+	// See RegisterInterest() for more information.
+	class BXEXPORT SocketDataHandler
+	{
+	public:
+		// Called when data is received on the socket
+		// for which interest has been registered.
+		//
+		// The lifetime of the data parameter is only valid
+		// for the duration of this method call.
+		virtual void DataReceived(Data& data) = 0;
+
+		// Called when an error has occured on the socket
+		// for which interest has been registered.
+		//
+		// The lifetime of the error parameter is only valid
+		// for the lifetime of this method call.
+		virtual void Error(Barry::Error &error);
+
+		virtual ~SocketDataHandler();
+	};
+
+	typedef std::tr1::shared_ptr<SocketDataHandler> SocketDataHandlerPtr;
+
+	// Simple wrapper template class for SocketDataHandler which provides a basic data recieved callback
+	template<typename T> class SimpleSocketDataHandler : public SocketDataHandler
+	{
+		void (*m_callback)(T&, Data*);
+		T& m_context;
+	public:
+		SimpleSocketDataHandler<T>(T& context, void (*callback)(T& context, Data* data))
+			: m_callback(callback)
+			, m_context(context)
+		{}
+		virtual void DataReceived(Data& data)
+		{
+			m_callback(m_context, &data);
+		}
+	};
+
 	struct QueueEntry
 	{
-		SocketDataHandler m_handler;
-		void *m_context;
+		SocketDataHandlerPtr m_handler;
 		DataQueue m_queue;
 
-		QueueEntry(SocketDataHandler h, void *c)
+		QueueEntry(SocketDataHandlerPtr h)
 			: m_handler(h)
-			, m_context(c)
 			{}
 	};
 	typedef std::tr1::shared_ptr<QueueEntry>	QueueEntryPtr;
@@ -70,11 +107,15 @@ private:
 
 	pthread_mutex_t m_readwaitMutex;
 	pthread_cond_t m_readwaitCond;
+	bool m_seen_usb_error;
+	SocketDataHandlerPtr m_usb_error_dev_callback;
 
 	DataQueue m_free;
 	DataQueue m_default;
 	SocketQueueMap m_socketQueues;
 
+	int m_timeout;
+
 	// thread state
 	pthread_t m_usb_read_thread;
 	volatile bool m_continue_reading;// set to true when the thread is created,
@@ -88,21 +129,47 @@ protected:
 	// from its destructor.
 	void ReturnBuffer(Data *buf);
 
+	// Helper function to add a buffer to a socket queue
+	// Returns false if no queue is available for that socket
+	// Also empties the DataHandle on success.
+	bool QueuePacket(SocketId socket, DataHandle &buf);
+	bool QueuePacket(DataQueue &queue, DataHandle &buf);
+	bool RouteOrQueuePacket(SocketId socket, DataHandle &buf);
+
 	// Thread function for the simple read behaviour... thread is
 	// created in the SpinoffSimpleReadThread() member below.
 	static void *SimpleReadThread(void *userptr);
 
+	void DumpSocketQueue(SocketId socket, const DataQueue &dq);
+
 public:
-	SocketRoutingQueue(int prealloc_buffer_count = 4);
+	SocketRoutingQueue(int prealloc_buffer_count = 4,
+		int default_read_timeout = USBWRAP_DEFAULT_TIMEOUT);
 	~SocketRoutingQueue();
 
+	//
+	// data access
+	//
+	int GetWriteEp() const { return m_writeEp; }
+	int GetReadEp() const { return m_readEp; }
+
+
 	// These functions connect the router to an external Usb::Device
 	// object.  Normally this is handled automatically by the
 	// Controller class, but are public here in case they are needed.
-	void SetUsbDevice(Usb::Device *dev, int writeEp, int readEp);
+	//
+	// If DoRead encounters an error, it sets a flag and stops
+	// reading.  To recover, you should handle the Error() call in
+	// the callback, fix the USB device, and then call
+	// ClearUsbError() to clear the flag.
+	//
+	void SetUsbDevice(Usb::Device *dev, int writeEp, int readEp,
+		SocketDataHandlerPtr callback = SocketDataHandlerPtr());
 	void ClearUsbDevice();
 	bool UsbDeviceReady();
 	Usb::Device* GetUsbDevice() { return m_dev; }
+	void ClearUsbError();
+
 
 	// This class starts out with no buffers, and will grow one buffer
 	// at a time if needed.  Call this to allocate count buffers
@@ -114,6 +181,9 @@ public:
 	// Returns false (or null pointer) on timeout and no data.
 	// With the return version of the function, there is no
 	// copying performed.
+	//
+	// Timeout is in milliseconds.  Default timeout set by constructor
+	// is used if set to -1.
 	bool DefaultRead(Data &receive, int timeout = -1);
 	DataHandle DefaultRead(int timeout = -1);
 
@@ -123,11 +193,11 @@ public:
 	// and must be read by DefaultRead()
 	// If not null, handler is called when new data is read.  It will
 	// be called in the same thread instance that DoRead() is called from.
-	// Handler is passed the DataQueue Data pointer, and so no
+	// Handler is passed the DataQueue Data object, and so no
 	// copying is done.  Once the handler returns, the data is
 	// considered processed and not added to the interested queue,
 	// but instead returned to m_free.
-	void RegisterInterest(SocketId socket, SocketDataHandler handler = 0, void *context = 0);
+	void RegisterInterest(SocketId socket, SocketDataHandlerPtr handler);
 
 	// Unregisters interest in data from the given socket, and discards
 	// any existing data in its interest queue.  Any new incoming data
@@ -140,6 +210,9 @@ public:
 	// Returns false (or null pointer) on timeout and no data.
 	// With the return version of the function, there is no
 	// copying performed.
+	//
+	// Timeout is in milliseconds.  Default timeout set by constructor
+	// is used if set to -1.
 	bool SocketRead(SocketId socket, Data &receive, int timeout = -1);
 	DataHandle SocketRead(SocketId socket, int timeout = -1);
 
@@ -153,11 +226,8 @@ public:
 	// called... it just doesn't do anything if there is no usb
 	// device to work with.
 	//
-	// Timeout is in milliseconds.
-	//
-	// Returns false in the case of USB errors, and puts the error
-	// message in msg.
-	bool DoRead(std::string &msg, int timeout = -1);
+	// Timeout is in milliseconds.  Default is default USB timeout.
+	void DoRead(int timeout = -1);
 
 	// Utility function to make it easier for the user to create the
 	// USB pure-read thread.  If the user wants anything more complicated
@@ -216,13 +286,20 @@ public:
 		return m_data;
 	}
 
-	Data* release()	// no longer owns the pointer
+	Data* release()	// no longer owns the pointer, and does not free it
 	{
 		Data *ret = m_data;
 		m_data = 0;
 		return ret;
 	}
 
+	// frees current pointer, and takes ownership of new one
+	void reset(Data *next = 0)
+	{
+		clear();
+		m_data = next;
+	}
+
 	Data* operator->()
 	{
 		return m_data;
