@@ -7,7 +7,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #ifndef __BARRY_S11N_BOOST_H__
 #define __BARRY_S11N_BOOST_H__
 
+#include "dll.h"
 #include "record.h"
 #include <boost/serialization/vector.hpp>
 
@@ -45,6 +46,12 @@
 namespace boost {
 namespace serialization {
 
+template <class ArchiveT>
+void serialize(ArchiveT &ar, Barry::TimeT &t, const unsigned int ver)
+{
+	ar & make_nvp("Barry::TimeT::Time", t.Time);
+}
+
 template <class ArchiveT>
 void serialize(ArchiveT &ar, Barry::UnknownField &uf, const unsigned int ver)
 {
@@ -66,6 +73,13 @@ void serialize(ArchiveT &ar, Barry::CategoryList &cl, const unsigned int ver)
 	ar & make_nvp("CategoryList", sl);
 }
 
+template <class ArchiveT>
+void serialize(ArchiveT &ar, Barry::EmailList &el, const unsigned int ver)
+{
+	std::vector<std::string> &sl = el;
+	ar & make_nvp("EmailList", sl);
+}
+
 template <class ArchiveT>
 void serialize(ArchiveT &ar, Barry::Contact &c, const unsigned int ver)
 {
@@ -75,9 +89,11 @@ void serialize(ArchiveT &ar, Barry::Contact &c, const unsigned int ver)
 	ar & make_nvp("EmailAddresses", c.EmailAddresses);
 	ar & make_nvp("Phone", c.Phone);
 	ar & make_nvp("Fax", c.Fax);
+	ar & make_nvp("HomeFax", c.HomeFax);
 	ar & make_nvp("WorkPhone", c.WorkPhone);
 	ar & make_nvp("HomePhone", c.HomePhone);
 	ar & make_nvp("MobilePhone", c.MobilePhone);
+	ar & make_nvp("MobilePhone2", c.MobilePhone2);
 	ar & make_nvp("Pager", c.Pager);
 	ar & make_nvp("PIN", c.PIN);
 	ar & make_nvp("Radio", c.Radio);
@@ -113,6 +129,7 @@ void serialize(ArchiveT &ar, Barry::Contact &c, const unsigned int ver)
 	ar & make_nvp("HomePostalCode", c.HomeAddress.PostalCode);
 	ar & make_nvp("HomeCountry", c.HomeAddress.Country);
 	ar & make_nvp("Image", c.Image);
+	ar & make_nvp("Nickname", c.Nickname);
 
 	if( ver < BARRY_POD_MAP_VERSION ) {
 		ar & make_nvp("GroupLinks", c.GroupLinks);
@@ -127,6 +144,13 @@ void serialize(ArchiveT &ar, Barry::EmailAddress &a, const unsigned int ver)
 	ar & make_nvp("Email", a.Email);
 }
 
+template <class ArchiveT>
+void serialize(ArchiveT &ar, Barry::EmailAddressList &eal, const unsigned int ver)
+{
+	std::vector<Barry::EmailAddress> &v = eal;
+	ar & make_nvp("EmailAddressList", v);
+}
+
 template <class ArchiveT>
 void serialize(ArchiveT &ar, Barry::Message &m, const unsigned int ver)
 {
@@ -140,8 +164,8 @@ void serialize(ArchiveT &ar, Barry::Message &m, const unsigned int ver)
 	ar & make_nvp("Attachment", m.Attachment);
 	ar & make_nvp("MessageRecordId", m.MessageRecordId);
 	ar & make_nvp("MessageReplyTo", m.MessageReplyTo);
-	ar & make_nvp("MessageDateSent", m.MessageDateSent);
-	ar & make_nvp("MessageDateReceived", m.MessageDateReceived);
+	ar & make_nvp("MessageDateSent", m.MessageDateSent.Time);
+	ar & make_nvp("MessageDateReceived", m.MessageDateReceived.Time);
 
 	ar & make_nvp("MessageTruncated", m.MessageTruncated);
 	ar & make_nvp("MessageRead", m.MessageRead);
@@ -169,9 +193,9 @@ void serialize(ArchiveT &ar, Barry::Calendar &c, const unsigned int ver)
 	ar & make_nvp("Notes", c.Notes);
 	ar & make_nvp("Location", c.Location);
 
-	ar & make_nvp("NotificationTime", c.NotificationTime);
-	ar & make_nvp("StartTime", c.StartTime);
-	ar & make_nvp("EndTime", c.EndTime);
+	ar & make_nvp("NotificationTime", c.NotificationTime.Time);
+	ar & make_nvp("StartTime", c.StartTime.Time);
+	ar & make_nvp("EndTime", c.EndTime.Time);
 
 	ar & make_nvp("Organizer", c.Organizer);
 	ar & make_nvp("AcceptedBy", c.AcceptedBy);
@@ -183,8 +207,9 @@ void serialize(ArchiveT &ar, Barry::Calendar &c, const unsigned int ver)
 	ar & make_nvp("Recurring", c.Recurring);
 	ar & make_nvp("RecurringType", c.RecurringType);
 	ar & make_nvp("Interval", c.Interval);
-	ar & make_nvp("RecurringEndTime", c.RecurringEndTime);
+	ar & make_nvp("RecurringEndTime", c.RecurringEndTime.Time);
 	ar & make_nvp("Perpetual", c.Perpetual);
+	ar & make_nvp("CalendarID", c.CalendarID);
 	ar & make_nvp("TimeZoneCode", c.TimeZoneCode);
 	ar & make_nvp("TimeZoneValid", c.TimeZoneValid);
 
@@ -199,6 +224,14 @@ void serialize(ArchiveT &ar, Barry::Calendar &c, const unsigned int ver)
 	}
 }
 
+template <class ArchiveT>
+void serialize(ArchiveT &ar, Barry::CalendarAll &c, const unsigned int ver)
+{
+	serialize(ar, static_cast<Barry::Calendar&>(c), ver);
+
+	ar & make_nvp("MailAccount", c.MailAccount);
+}
+
 template <class ArchiveT>
 void serialize(ArchiveT &ar, Barry::CallLog &c, const unsigned int ver)
 {
@@ -220,6 +253,26 @@ void serialize(ArchiveT &ar, Barry::CallLog &c, const unsigned int ver)
 	}
 }
 
+template <class ArchiveT>
+void serialize(ArchiveT &ar, Barry::Bookmark &c, const unsigned int ver)
+{
+	ar & make_nvp("RecType", c.RecType);
+	ar & make_nvp("RecordId", c.RecordId);
+	ar & make_nvp("Index", c.Index);
+
+	ar & make_nvp("Name", c.Name);
+	ar & make_nvp("Icon", c.Icon);
+	ar & make_nvp("Url", c.Url);
+
+	ar & make_nvp("BrowserIdentity", c.BrowserIdentity);
+	ar & make_nvp("DisplayMode", c.DisplayMode);
+	ar & make_nvp("JavaScriptMode", c.JavaScriptMode);
+
+	if( ver < BARRY_POD_MAP_VERSION ) {
+		ar & make_nvp("Unknowns", c.Unknowns);
+	}
+}
+
 template <class ArchiveT>
 void serialize(ArchiveT &ar, Barry::ServiceBookConfig &c, const unsigned int ver)
 {
@@ -276,9 +329,9 @@ void serialize(ArchiveT &ar, Barry::Task &t, const unsigned int ver)
 	ar & make_nvp("Categories", t.Categories);
 	ar & make_nvp("UID", t.UID);
 
-	ar & make_nvp("StartTime", t.StartTime);
-	ar & make_nvp("DueTime", t.DueTime);
-	ar & make_nvp("AlarmTime", t.AlarmTime);
+	ar & make_nvp("StartTime", t.StartTime.Time);
+	ar & make_nvp("DueTime", t.DueTime.Time);
+	ar & make_nvp("AlarmTime", t.AlarmTime.Time);
 
 	ar & make_nvp("TimeZoneCode", t.TimeZoneCode);
 	ar & make_nvp("TimeZoneValid", t.TimeZoneValid);
@@ -286,7 +339,7 @@ void serialize(ArchiveT &ar, Barry::Task &t, const unsigned int ver)
 	ar & make_nvp("AlarmType", t.AlarmType);
 	ar & make_nvp("Interval", t.Interval);
 	ar & make_nvp("RecurringType", t.RecurringType);
-	ar & make_nvp("RecurringEndTime", t.RecurringEndTime);
+	ar & make_nvp("RecurringEndTime", t.RecurringEndTime.Time);
 	ar & make_nvp("DayOfWeek", t.DayOfWeek);
 	ar & make_nvp("WeekOfMonth", t.WeekOfMonth);
 	ar & make_nvp("DayOfMonth", t.DayOfMonth);
@@ -297,7 +350,6 @@ void serialize(ArchiveT &ar, Barry::Task &t, const unsigned int ver)
 	ar & make_nvp("StatusFlag", t.StatusFlag);
 	ar & make_nvp("Recurring", t.Recurring);
 	ar & make_nvp("Perpetual", t.Perpetual);
-	ar & make_nvp("DueDateFlag", t.DueDateFlag);
 
 	if( ver < BARRY_POD_MAP_VERSION ) {
 		ar & make_nvp( "Unknowns", t.Unknowns);
@@ -318,8 +370,8 @@ void serialize(ArchiveT &ar, Barry::PINMessage &p, const unsigned int ver)
 	ar & make_nvp("Body", p.Body);
 	ar & make_nvp("MessageRecordId", p.MessageRecordId);
 	ar & make_nvp("MessageReplyTo", p.MessageReplyTo);
-	ar & make_nvp("MessageDateSent", p.MessageDateSent);
-	ar & make_nvp("MessageDateReceived", p.MessageDateReceived);
+	ar & make_nvp("MessageDateSent", p.MessageDateSent.Time);
+	ar & make_nvp("MessageDateReceived", p.MessageDateReceived.Time);
 
 	ar & make_nvp("MessageTruncated", p.MessageTruncated);
 	ar & make_nvp("MessageRead", p.MessageRead);
@@ -352,8 +404,8 @@ void serialize(ArchiveT &ar, Barry::SavedMessage &m, const unsigned int ver)
 	ar & make_nvp("Attachment", m.Attachment);
 	ar & make_nvp("MessageRecordId", m.MessageRecordId);
 	ar & make_nvp("MessageReplyTo", m.MessageReplyTo);
-	ar & make_nvp("MessageDateSent", m.MessageDateSent);
-	ar & make_nvp("MessageDateReceived", m.MessageDateReceived);
+	ar & make_nvp("MessageDateSent", m.MessageDateSent.Time);
+	ar & make_nvp("MessageDateReceived", m.MessageDateReceived.Time);
 
 	ar & make_nvp("MessageTruncated", m.MessageTruncated);
 	ar & make_nvp("MessageRead", m.MessageRead);
@@ -415,29 +467,322 @@ void serialize(ArchiveT &ar, Barry::Folder &f, const unsigned int ver)
 }
 
 template <class ArchiveT>
-void serialize(ArchiveT &ar, Barry::Timezone &t, const unsigned int ver)
+void serialize(ArchiveT &ar, Barry::TimeZone &t, const unsigned int ver)
 {
 	ar & make_nvp("RecType", t.RecType);
 	ar & make_nvp("RecordId", t.RecordId);
 
-	ar & make_nvp("TZType", t.TZType);
-	ar & make_nvp("DSTOffset", t.DSTOffset);
+	ar & make_nvp("Name", t.Name);
 	ar & make_nvp("Index", t.Index);
-	ar & make_nvp("Offset", t.Offset);
-	ar & make_nvp("OffsetFraction", t.OffsetFraction);
+	ar & make_nvp("UTCOffset", t.UTCOffset);
+
+	ar & make_nvp("UseDST", t.UseDST);
+	ar & make_nvp("DSTOffset", t.DSTOffset);
 	ar & make_nvp("StartMonth", t.StartMonth);
 	ar & make_nvp("EndMonth", t.EndMonth);
-	ar & make_nvp("Left", t.Left);
-	ar & make_nvp("UseDST", t.UseDST);
 
-	ar & make_nvp("TimeZoneName", t.TimeZoneName);
+	ar & make_nvp("TZType", t.TZType);
 
 	if( ver < BARRY_POD_MAP_VERSION ) {
 		ar & make_nvp("Unknowns", t.Unknowns);
 	}
 }
 
+template <class ArchiveT>
+void serialize(ArchiveT &ar, Barry::ContentStore &c, const unsigned int ver)
+{
+	ar & make_nvp("RecType", c.RecType);
+	ar & make_nvp("RecordId", c.RecordId);
+
+	ar & make_nvp("Filename", c.Filename);
+	ar & make_nvp("FolderFlag", c.FolderFlag);
+	ar & make_nvp("FileContent", c.FileContent);
+	ar & make_nvp("FileDescriptor", c.FileDescriptor);
+
+	// FileSize is not used for building, so no need to save it here
+
+	if( ver < BARRY_POD_MAP_VERSION ) {
+		ar & make_nvp("Unknowns", c.Unknowns);
+	}
+}
+
+template <class ArchiveT>
+void serialize(ArchiveT &ar, Barry::HandheldAgent &h, const unsigned int ver)
+{
+	ar & make_nvp("RecType", h.RecType);
+	ar & make_nvp("RecordId", h.RecordId);
+
+	ar & make_nvp("MEID", h.MEID);
+	ar & make_nvp("Model", h.Model);
+	ar & make_nvp("Bands", h.Bands);
+	ar & make_nvp("Pin", h.Pin);
+	ar & make_nvp("Version", h.Version);
+	ar & make_nvp("PlatformVersion", h.PlatformVersion);
+	ar & make_nvp("Manufacturer", h.Manufacturer);
+	ar & make_nvp("Network", h.Network);
+
+	if( ver < BARRY_POD_MAP_VERSION ) {
+		ar & make_nvp("Unknowns", h.Unknowns);
+	}
+}
+
 }} // namespace boost::serialization
 
+
+//////////////////////////////////////////////////////////////////////////////
+// Helper wrapper templates for loading and saving records to an iostream
+
+namespace Barry {
+
+// Can be used as a Storage class for RecordBuilder<>
+template <class RecordT>
+class BoostLoader
+{
+public:
+	typedef RecordT				rec_type;
+	typedef std::vector<rec_type>		list_type;
+
+private:
+	list_type m_records;
+	typename list_type::iterator rec_it;
+
+public:
+	explicit BoostLoader(std::istream &is)
+	{
+		boost::archive::text_iarchive ia(is);
+		ia >> m_records;
+		rec_it = m_records.begin();
+	}
+
+	list_type& GetRecords() { return m_records; }
+	const list_type& GetRecords() const { return m_records; }
+
+	// retrieval operator
+	bool operator()(RecordT &rec, Builder &builder)
+	{
+		if( rec_it == m_records.end() )
+			return false;
+		rec = *rec_it;
+		++rec_it;
+		return true;
+	}
+};
+
+// Can be used as a Storage class for RecordParser<>
+template <class RecordT>
+class BoostSaver
+{
+public:
+	typedef RecordT				rec_type;
+	typedef std::vector<rec_type>		list_type;
+
+private:
+	std::ostream &m_os;
+	list_type m_records;
+	typename list_type::iterator rec_it;
+
+public:
+	explicit BoostSaver(std::ostream &os)
+		: m_os(os)
+	{
+	}
+
+	~BoostSaver()
+	{
+		WriteArchive();
+	}
+
+	void WriteArchive() const
+	{
+		// write dbname first, so parsing is possible
+		m_os << RecordT::GetDBName() << std::endl;
+
+		// write boost archive of all records
+		boost::archive::text_oarchive oa(m_os);
+
+		// boost is fussy that the vector must be const
+		// we do this explicitly, for documentation's sake
+		const list_type &recs = m_records;
+		oa << recs;
+		m_os << std::endl;
+	}
+
+	list_type& GetRecords() { return m_records; }
+	const list_type& GetRecords() const { return m_records; }
+
+	// storage operator
+	void operator()(const RecordT &rec)
+	{
+		m_records.push_back(rec);
+	}
+};
+
+//
+// BoostParser
+//
+/// This Parser turns incoming records (which can be of any record type
+/// included in ALL_KNOWN_PARSER_TYPES) into a Boost Serialization stream
+/// on the given iostream.
+///
+/// This class is defined completely in the header, so that it is
+/// optional for applications to link against the boost libraries.
+///
+class BXEXPORT BoostParser : public Barry::Parser
+{
+	std::auto_ptr<Barry::Parser> m_parser;
+	std::ofstream *m_ofs;
+	std::ostream &m_os;	// references either an external object,
+				// or *m_ifs... this is the reference to
+				// use in the entire class... the constructor
+				// sets it up
+
+	std::string m_current_db;
+
+public:
+	explicit BoostParser(const std::string &filename)
+		: m_ofs( new std::ofstream(filename.c_str()) )
+		, m_os(*m_ofs)
+	{
+	}
+
+	explicit BoostParser(std::ostream &os)
+		: m_ofs(0)
+		, m_os(os)
+	{
+	}
+
+	~BoostParser()
+	{
+		// flush any remaining parser output
+		// (note this still potentially uses m_ofs, so do this first)
+		m_parser.reset();
+
+		// cleanup the stream
+		delete m_ofs;
+	}
+
+	void StartDB(const std::string &dbname)
+	{
+		// done with current parser, flush it's output
+		m_parser.reset();
+
+#undef HANDLE_PARSER
+#define HANDLE_PARSER(tname) \
+		if( dbname == tname::GetDBName() ) { \
+			m_parser.reset( \
+				new RecordParser<tname, BoostSaver<tname> >( \
+					new BoostSaver<tname>(m_os) ) ); \
+			return; \
+		}
+
+		ALL_KNOWN_PARSER_TYPES
+
+		// if we make it here, we don't have a record parser
+		// for this dbname, so just dump it to stderr (not stdout,
+		// since the user might be sending normal output there)
+		m_parser.reset( new HexDumpParser(std::cerr) );
+	}
+
+	void ParseRecord(const DBData &data, const IConverter *ic)
+	{
+		if( m_current_db != data.GetDBName() ) {
+			StartDB(data.GetDBName());
+			m_current_db = data.GetDBName();
+		}
+
+		m_parser->ParseRecord(data, ic);
+	}
+};
+
+//
+// BoostBuilder
+//
+/// This Builder class reads a boost serialization stream, and converts
+/// them into DBData records.  Can only produce records for record types
+/// in ALL_KNOWN_BUILDER_TYPES.
+///
+class BXEXPORT BoostBuilder : public Barry::Builder
+{
+	std::auto_ptr<Builder> m_builder;
+	std::ifstream *m_ifs;
+
+	std::istream &m_is;	// references either an external object,
+				// or *m_ifs... this is the reference to
+				// use in the entire class... the constructor
+				// sets it up
+
+public:
+	explicit BoostBuilder(const std::string &filename)
+		: m_ifs( new std::ifstream(filename.c_str()) )
+		, m_is(*m_ifs)
+	{
+		FinishDB();
+	}
+
+	explicit BoostBuilder(std::istream &is)
+		: m_ifs(0)
+		, m_is(is)
+	{
+		FinishDB();
+	}
+
+	~BoostBuilder()
+	{
+		delete m_ifs;
+	}
+
+	void FinishDB()
+	{
+		// done with current builder
+		m_builder.reset();
+
+		// read the next DBName
+		std::string dbName;
+		while( getline(m_is, dbName) ) {
+
+#undef HANDLE_BUILDER
+#define HANDLE_BUILDER(tname) \
+			if( dbName == tname::GetDBName() ) { \
+				m_builder.reset( \
+					new RecordBuilder<tname, BoostLoader<tname> >( \
+						new BoostLoader<tname>(m_is) ) ); \
+				return; \
+			}
+
+			ALL_KNOWN_BUILDER_TYPES
+		}
+	}
+
+	bool BuildRecord(DBData &data, size_t &offset, const IConverter *ic)
+	{
+		if( !m_builder.get() )
+			return false;
+
+		bool ret = m_builder->BuildRecord(data, offset, ic);
+		if( !ret )
+			FinishDB();
+		return ret;
+	}
+
+	bool FetchRecord(DBData &data, const IConverter *ic)
+	{
+		if( !m_builder.get() )
+			return false;
+
+		bool ret = m_builder->FetchRecord(data, ic);
+		if( !ret )
+			FinishDB();
+		return ret;
+	}
+
+	bool EndOfFile() const
+	{
+		return m_builder.get() ? false : true;
+	}
+};
+
+
+} // namespace Barry
+
 #endif
 
