@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -43,34 +43,34 @@ namespace Barry {
 SocketZero::SocketZero(	SocketRoutingQueue &queue,
 			int writeEndpoint,
 			uint8_t zeroSocketSequenceStart)
-	: m_dev(0),
-	m_queue(&queue),
-	m_writeEp(writeEndpoint),
-	m_readEp(0),
-	m_zeroSocketSequence(zeroSocketSequenceStart),
-	m_sequenceId(0),
-	m_halfOpen(false),
-	m_challengeSeed(0),
-	m_remainingTries(0),
-	m_hideSequencePacket(true),
-	m_resetOnClose(false)
+	: m_dev(0)
+	, m_queue(&queue)
+	, m_writeEp(writeEndpoint)
+	, m_readEp(0)
+	, m_zeroSocketSequence(zeroSocketSequenceStart)
+	, m_sequenceId(0)
+	, m_halfOpen(false)
+	, m_challengeSeed(0)
+	, m_remainingTries(0)
+	, m_modeSequencePacketSeen(false)
+	, m_pushback(false)
 {
 }
 
 SocketZero::SocketZero(	Device &dev,
 			int writeEndpoint, int readEndpoint,
 			uint8_t zeroSocketSequenceStart)
-	: m_dev(&dev),
-	m_queue(0),
-	m_writeEp(writeEndpoint),
-	m_readEp(readEndpoint),
-	m_zeroSocketSequence(zeroSocketSequenceStart),
-	m_sequenceId(0),
-	m_halfOpen(false),
-	m_challengeSeed(0),
-	m_remainingTries(0),
-	m_hideSequencePacket(true),
-	m_resetOnClose(false)
+	: m_dev(&dev)
+	, m_queue(0)
+	, m_writeEp(writeEndpoint)
+	, m_readEp(readEndpoint)
+	, m_zeroSocketSequence(zeroSocketSequenceStart)
+	, m_sequenceId(0)
+	, m_halfOpen(false)
+	, m_challengeSeed(0)
+	, m_remainingTries(0)
+	, m_modeSequencePacketSeen(false)
+	, m_pushback(false)
 {
 }
 
@@ -214,20 +214,17 @@ void SocketZero::SendOpen(uint16_t socket, Data &receive)
 	try {
 		RawSend(send);
 		RawReceive(receive);
+		if( Protocol::IsSequencePacket(receive) ) {
+			m_modeSequencePacketSeen = true;
+			// during open, we could get a sequence packet in
+			// the middle, from the SelectMode operation
+			RawReceive(receive);
+		}
 	} catch( Usb::Error & ) {
 		eeout(send, receive);
 		throw;
 	}
 
-	// check sequence ID
-	Protocol::CheckSize(receive, SB_PACKET_HEADER_SIZE);
-	if( IS_COMMAND(receive, SB_COMMAND_SEQUENCE_HANDSHAKE) ) {
-		CheckSequence(0, receive);
-
-		// still need our ACK
-		RawReceive(receive);
-	}
-
 	// receive now holds the Open response
 }
 
@@ -278,8 +275,9 @@ void SocketZero::SendPasswordHash(uint16_t socket, const char *password, Data &r
 	send.Zap();
 
 	// check sequence ID
-	Protocol::CheckSize(receive, SB_PACKET_HEADER_SIZE);
-	if( IS_COMMAND(receive, SB_COMMAND_SEQUENCE_HANDSHAKE) ) {
+	if( Protocol::IsSequencePacket(receive) ) {
+		m_modeSequencePacketSeen = true;
+
 		CheckSequence(0, receive);
 
 		// still need our ACK
@@ -292,6 +290,8 @@ void SocketZero::SendPasswordHash(uint16_t socket, const char *password, Data &r
 void SocketZero::RawSend(Data &send, int timeout)
 {
 	Usb::Device *dev = m_queue ? m_queue->GetUsbDevice() : m_dev;
+	if( !dev )
+		throw Error("SocketZero: No device available for RawSend");
 
 	// Special case: it seems that sending packets with a size that's an
 	// exact multiple of 0x40 causes the device to get confused.
@@ -306,59 +306,40 @@ void SocketZero::RawSend(Data &send, int timeout)
 		packet.buffer[2] = 0;		// zero the top byte
 		Data sizeCommand(&packet, 3);
 
-		dev->BulkWrite(m_writeEp, sizeCommand);
+		dev->BulkWrite(m_writeEp, sizeCommand, timeout);
 	}
 
-	dev->BulkWrite(m_writeEp, send);
+	dev->BulkWrite(m_writeEp, send, timeout);
 }
 
 void SocketZero::RawReceive(Data &receive, int timeout)
 {
-	do {
-		if( m_queue ) {
-			if( !m_queue->DefaultRead(receive, timeout) )
-				throw Timeout("SocketZero::RawReceive: queue DefaultRead returned false (likely a timeout)");
-		}
-		else {
-			m_dev->BulkRead(m_readEp, receive, timeout);
-		}
-		ddout("SocketZero::RawReceive: Endpoint " << m_readEp
-			<< "\nReceived:\n" << receive);
-	} while( SequencePacket(receive) );
+	if( m_pushback ) {
+		receive = m_pushback_buffer;
+		m_pushback = false;
+		return;
+	}
+
+	if( m_queue ) {
+		if( !m_queue->DefaultRead(receive, timeout) )
+			throw Timeout("SocketZero::RawReceive: queue DefaultRead returned false (likely a timeout)");
+	}
+	else {
+		m_dev->BulkRead(m_readEp, receive, timeout);
+	}
+
+	ddout("SocketZero::RawReceive: Endpoint "
+		<< (m_queue ? m_queue->GetReadEp() : m_readEp)
+		<< "\nReceived:\n" << receive);
 }
 
-//
-// SequencePacket
-//
-/// Returns true if this is a sequence packet that should be ignored.
-/// This function is used in SocketZero::RawReceive() in order
-/// to determine whether to keep reading or not.  By default,
-/// this function checks whether the packet is a sequence packet
-/// or not, and returns true if so.  Also, if it is a sequence
-/// packet, it checks the validity of the sequence number.
-///
-/// If sequence packets become important in the future, this
-/// function could be changed to call a user-defined callback,
-/// in order to handle these things out of band.
-///
-bool SocketZero::SequencePacket(const Data &data)
+void SocketZero::Pushback(const Data &buf)
 {
-	// Begin -- Test quiet durty :(
-	if (m_hideSequencePacket == false) {
-		return false;
-	}
-	// End -- Test quiet durty :(
+	if( m_pushback )
+		throw Error("Multiple pushbacks");
 
-	if( data.GetSize() >= MIN_PACKET_SIZE ) {
-		MAKE_PACKET(rpack, data);
-		if( rpack->socket == 0 &&
-		    rpack->command == SB_COMMAND_SEQUENCE_HANDSHAKE )
-		{
-			CheckSequence(0, data);
-			return true;
-		}
-	}
-	return false;	// not a sequence packet
+	m_pushback = true;
+	m_pushback_buffer = buf;
 }
 
 
@@ -403,7 +384,7 @@ void SocketZero::Send(Data &send, Data &receive, int timeout)
 
 void SocketZero::Send(Barry::Packet &packet, int timeout)
 {
-	Send(packet.m_send, packet.m_receive, timeout);
+	Send(packet.m_send, *packet.m_receive, timeout);
 }
 
 void SocketZero::Receive(Data &receive, int timeout)
@@ -452,6 +433,14 @@ SocketHandle SocketZero::Open(uint16_t socket, const char *password)
 		// starting fresh
 		m_remainingTries = 0;
 
+		// this gets set to true if we see a starting sequence packet
+		// during any of our open and password commands... After
+		// a mode command (like "RIM Desktop", etc.) a starting
+		// sequence packet is sent, and may arrive before or after
+		// the socket open handshake.
+		m_modeSequencePacketSeen = false;
+
+		// go for it!
 		SendOpen(socket, receive);
 
 		// check for password challenge, or success
@@ -500,6 +489,22 @@ SocketHandle SocketZero::Open(uint16_t socket, const char *password)
 		// fall through to success check...
 	}
 
+	// If the device thinks that the socket was already open then
+	// it will tell us by sending an SB_COMMAND_CLOSE_SOCKET.
+	//
+	// This happens most commonly when using raw channels which
+	// haven't been cleanly closed (such as by killing the process
+	// running Barry) and so the device still thinks the socket
+	// is open.
+	//
+	// Retrying the open will usually succeed, but relies on the
+	// device software re-creating the channel after it's closed
+	// so return an error here instead of automatically retrying.
+	if( packet.Command() == SB_COMMAND_CLOSE_SOCKET )
+	{
+		throw SocketCloseOnOpen("Socket: Device closed socket when trying to open (can be caused by the wrong password, or if the device thinks the socket is already open... please try again)");
+	}
+
 	if( packet.Command() != SB_COMMAND_OPENED_SOCKET ||
 	    packet.SocketResponse() != socket ||
 	    packet.SocketSequence() != closeFlag )
@@ -508,8 +513,44 @@ SocketHandle SocketZero::Open(uint16_t socket, const char *password)
 		throw Error("Socket: Bad OPENED packet in Open");
 	}
 
+	// if no sequence packet has yet arrived, wait for it here
+	if( !m_modeSequencePacketSeen ) {
+		Data sequence;
+		RawReceive(sequence);
+		if( !Protocol::IsSequencePacket(sequence) ) {
+			// if this is not the sequence packet, then it might
+			// just be out of order (some devices do this when
+			// opening the JavaLoader mode), so as a last
+			// ditch effort, do one more read with a short
+			// timeout, and check that as well
+			Data late_sequence;
+			RawReceive(late_sequence, 500);
+			if( !Protocol::IsSequencePacket(late_sequence) ) {
+				throw Error("Could not find mode's starting sequence packet");
+			}
+
+			// ok, so our ditch effort worked, but now we have
+			// a leftover packet on our hands... do a temporary
+			// pushback
+			Pushback(sequence);
+		}
+	}
+
 	// success!  save the socket
-	return SocketHandle(new Socket(*this, socket, closeFlag));
+	Socket *sock = new Socket(*this, socket, closeFlag);
+	SocketHandle sh(sock);
+
+	// if we are running with a routing queue, register the
+	// socket's interest in all its own data.  By default, this
+	// data will be queued without a callback handler.
+	// If other application code needs to intercept this with
+	// its own handler, it must call UnregisterInterest() and
+	// re-register its own handler.
+	if( m_queue ) {
+		sock->RegisterInterest();
+	}
+
+	return sh;
 }
 
 //
@@ -549,8 +590,7 @@ void SocketZero::Close(Socket &socket)
 	}
 
 	// starting fresh, reset sequence ID
-	Protocol::CheckSize(response, SB_PACKET_HEADER_SIZE);
-	if( IS_COMMAND(response, SB_COMMAND_SEQUENCE_HANDSHAKE) ) {
+	if( Protocol::IsSequencePacket(response) ) {
 		CheckSequence(0, response);
 
 		// still need our ACK
@@ -559,7 +599,15 @@ void SocketZero::Close(Socket &socket)
 
 	Protocol::CheckSize(response, SB_SOCKET_PACKET_HEADER_SIZE);
 	MAKE_PACKET(rpack, response);
-	if( rpack->command != SB_COMMAND_CLOSED_SOCKET ||
+	// The reply will be SB_COMMAND_CLOSED_SOCKET if the device
+	// has closed the socket in response to our request.
+	//
+	// It's also possible for the reply to be
+	// SB_COMMAND_REMOTE_CLOSE_SOCKET if the device wanted to
+	// close the socket at the same time, such as if the channel
+	// API is being used by the device.
+	if( ( rpack->command != SB_COMMAND_CLOSED_SOCKET &&
+	      rpack->command != SB_COMMAND_REMOTE_CLOSE_SOCKET ) ||
 	    btohs(rpack->u.socket.socket) != socket.GetSocket() ||
 	    rpack->u.socket.sequence != socket.GetCloseFlag() )
 	{
@@ -570,7 +618,7 @@ void SocketZero::Close(Socket &socket)
 		throw BadPacket(rpack->command, "Socket: Bad CLOSED packet in Close");
 	}
 
-	if( m_resetOnClose ) {
+	if( socket.IsResetOnClose() ) {
 		Data send, receive;
 		ZeroPacket reset_packet(send, receive);
 		reset_packet.Reset();
@@ -598,118 +646,262 @@ void SocketZero::Close(Socket &socket)
 
 
 
-
 //////////////////////////////////////////////////////////////////////////////
-// Socket class
+// SocketBase class
 
-Socket::Socket( SocketZero &zero,
-		uint16_t socket,
-		uint8_t closeFlag)
-	: m_zero(&zero)
-	, m_socket(socket)
-	, m_closeFlag(closeFlag)
-	, m_registered(false)
+SocketBase::~SocketBase()
 {
 }
 
-Socket::~Socket()
+void SocketBase::CheckSequence(const Data &seq)
 {
-	// trap exceptions in the destructor
-	try {
-		// a non-default socket has been opened, close it
-		Close();
-	}
-	catch( std::runtime_error &re ) {
-		// do nothing... log it?
-		dout("Exception caught in ~Socket: " << re.what());
-	}
+	// FIXME - needs implementation
 }
 
-
-////////////////////////////////////
-// Socket protected API
-
-void Socket::CheckSequence(const Data &seq)
-{
-	m_zero->CheckSequence(m_socket, seq);
-}
-
-void Socket::ForceClosed()
-{
-	m_socket = 0;
-	m_closeFlag = 0;
-}
-
-
-////////////////////////////////////
-// Socket public API
-
-void Socket::Close()
-{
-	UnregisterInterest();
-	m_zero->Close(*this);
-}
-
-
 //
-// Send
+// DBFragSend
 //
-/// Sends 'send' data to device, no receive.
+/// Sends a fragmented Desktop / Database command packet.
+/// Assumes that 'send' contains a valid packet, which may or may not
+/// need fragmentation.  If it does, fragmentation will be done
+/// automatically.
 ///
-/// \returns	void
-///
-/// \exception	Usb::Error on underlying bus errors.
-///
-void Socket::Send(Data &send, int timeout)
+void SocketBase::DBFragSend(Data &send, int timeout)
 {
-	// force the socket number to this socket
-	if( send.GetSize() >= SB_PACKET_HEADER_SIZE ) {
-		MAKE_PACKETPTR_BUF(spack, send.GetBuffer());
-		spack->socket = htobs(m_socket);
+	MAKE_PACKET(spack, send);
+	if( send.GetSize() < MIN_PACKET_SIZE ||
+	    (spack->command != SB_COMMAND_DB_DATA &&
+	     spack->command != SB_COMMAND_DB_DONE) )
+	{
+		// we don't do that around here
+		eout("unknown send data in DBFragSend(): " << send);
+		throw std::logic_error("Socket: unknown send data in DBFragSend()");
 	}
-	m_zero->RawSend(send, timeout);
+
+	if( send.GetSize() <= MAX_PACKET_SIZE ) {
+		// send non-fragmented
+		SyncSend(send, timeout);
+	}
+	else {
+		// send fragmented
+		unsigned int offset = 0;
+		Data outFrag;
+
+		do {
+			offset = SocketZero::MakeNextFragment(send, outFrag, offset);
+			SyncSend(outFrag, timeout);
+		} while( offset > 0 );
+	}
+
 }
 
 //
 // Send
 //
-/// Sends 'send' data to device, and waits for response.
+/// SyncSends 'send' data to device, and waits for response.
 ///
 /// \returns	void
 ///
 /// \exception	Usb::Error on underlying bus errors.
 ///
-void Socket::Send(Data &send, Data &receive, int timeout)
+void SocketBase::Send(Data &send, Data &receive, int timeout)
 {
-	Send(send, timeout);
+	SyncSend(send, timeout);
 	Receive(receive, timeout);
 }
 
-void Socket::Send(Barry::Packet &packet, int timeout)
+void SocketBase::Send(Barry::Packet &packet, int timeout)
 {
-	Send(packet.m_send, packet.m_receive, timeout);
+	Send(packet.m_send, *packet.m_receive, timeout);
 }
 
-void Socket::Receive(Data &receive, int timeout)
+// sends the send packet down to the device, fragmenting if
+// necessary, and returns the response in receive, defragmenting
+// if needed
+// Blocks until response received or timed out in Usb::Device
+//
+// This is primarily for Desktop Database packets... Javaloader
+// packets use PacketData().
+//
+void SocketBase::Packet(Data &send, Data &receive, int timeout)
 {
-	if( m_registered ) {
-		if( m_zero->m_queue ) {
-			if( !m_zero->m_queue->SocketRead(m_socket, receive, timeout) )
-				throw Timeout("Socket::Receive: queue SocketRead returned false (likely a timeout)");
+	// assume the common case of no fragmentation,
+	// and use the receive buffer for input... allocate a frag buffer
+	// later if necessary
+	Data *inputBuf = &receive;
+	receive.Zap();
+
+	DBFragSend(send, timeout);
+	Receive(*inputBuf, timeout);
+
+	std::auto_ptr<Data> inFrag;
+	bool done = false, frag = false;
+	int blankCount = 0;
+	while( !done ) {
+		MAKE_PACKET(rpack, *inputBuf);
+
+		// check the packet's validity
+		if( inputBuf->GetSize() > 0 ) {
+			blankCount = 0;
+
+			Protocol::CheckSize(*inputBuf, SB_PACKET_HEADER_SIZE);
+
+			switch( rpack->command )
+			{
+			case SB_COMMAND_SEQUENCE_HANDSHAKE:
+				CheckSequence(*inputBuf);
+				break;
+
+			case SB_COMMAND_DB_DATA:
+				if( frag ) {
+					SocketZero::AppendFragment(receive, *inputBuf);
+				}
+				else {
+					// no copy needed, already in receive,
+					// since inputBuf starts out that way
+				}
+				done = true;
+				break;
+
+			case SB_COMMAND_DB_FRAGMENTED:
+				// only copy if frag is true, since the
+				// first time through, receive == inputBuf
+				if( frag ) {
+					SocketZero::AppendFragment(receive, *inputBuf);
+				}
+				frag = true;
+				break;
+
+			case SB_COMMAND_DB_DONE:
+				// no copy needed, already in receive
+				done = true;
+				break;
+
+			default: {
+				std::ostringstream oss;
+				oss << "Socket: (read) unhandled packet in Packet(): 0x" << std::hex << (unsigned int)rpack->command;
+				eout(oss.str());
+				throw Error(oss.str());
+				}
+				break;
+			}
 		}
 		else {
-			throw std::logic_error("NULL queue pointer in a registered socket read.");
+			blankCount++;
+			//std::cerr << "Blank! " << blankCount << std::endl;
+			if( blankCount == 10 ) {
+				// only ask for more data on stalled sockets
+				// for so long
+				throw Error("Socket: 10 blank packets received");
+			}
+		}
+
+		if( !done ) {
+			// not done yet, ask for another read, and
+			// create new buffer for fragmented reads
+			if( frag && !inFrag.get() ) {
+				inFrag.reset( new Data );
+				inputBuf = inFrag.get();
+			}
+			Receive(*inputBuf);
 		}
-	}
-	else {
-		m_zero->RawReceive(receive, timeout);
 	}
 }
 
+void SocketBase::Packet(Barry::Packet &packet, int timeout)
+{
+	Packet(packet.m_send, *packet.m_receive, timeout);
+}
+
+void SocketBase::Packet(Barry::JLPacket &packet, int timeout)
+{
+	if( packet.HasData() ) {
+		SyncSend(packet.m_cmd);
+		PacketData(packet.m_data, *packet.m_receive, false, timeout);
+	}
+	else {
+		PacketData(packet.m_cmd, *packet.m_receive, false, timeout);
+	}
+}
+
+void SocketBase::Packet(Barry::JVMPacket &packet, int timeout)
+{
+	PacketJVM(packet.m_cmd, *packet.m_receive, timeout);
+}
 
 // sends the send packet down to the device
 // Blocks until response received or timed out in Usb::Device
-void Socket::PacketData(Data &send, Data &receive, int timeout)
+//
+// This function is used to send packet to JVM
+void SocketBase::PacketJVM(Data &send, Data &receive, int timeout)
+{
+	if( ( send.GetSize() < MIN_PACKET_DATA_SIZE ) ||
+		( send.GetSize() > MAX_PACKET_DATA_SIZE ) ) {
+		// we don't do that around here
+		throw std::logic_error("Socket: unknown send data in PacketJVM()");
+	}
+
+	Data &inFrag = receive;
+	receive.Zap();
+
+	// send non-fragmented
+	RawSend(send, timeout);
+	Receive(inFrag, timeout);
+
+	bool done = false;
+	int blankCount = 0;
+
+	while( !done ) {
+		// check the packet's validity
+		if( inFrag.GetSize() > 6 ) {
+			MAKE_PACKET(rpack, inFrag);
+
+			blankCount = 0;
+
+			Protocol::CheckSize(inFrag, SB_PACKET_HEADER_SIZE);
+
+			switch( rpack->command )
+			{
+			case SB_COMMAND_SEQUENCE_HANDSHAKE:
+				CheckSequence(inFrag);
+				break;
+
+			default: {
+				std::ostringstream oss;
+				oss << "Socket: (read) unhandled packet in Packet(): 0x" << std::hex << (unsigned int)rpack->command;
+				eout(oss.str());
+				throw Error(oss.str());
+				}
+				break;
+			}
+		}
+		else if( inFrag.GetSize() == 6 ) {
+			done = true;
+		}
+		else {
+			blankCount++;
+
+			//std::cerr << "Blank! " << blankCount << std::endl;
+			if( blankCount == 10 ) {
+				// only ask for more data on stalled sockets
+				// for so long
+				throw Error("Socket: 10 blank packets received");
+			}
+		}
+
+		if( !done ) {
+			// not done yet, ask for another read
+			Receive(inFrag, timeout);
+		}
+	}
+}
+
+// sends the send packet down to the device
+// Blocks until response received or timed out in Usb::Device
+void SocketBase::PacketData(Data &send,
+				Data &receive,
+				bool done_on_sequence,
+				int timeout)
 {
 	if( ( send.GetSize() < MIN_PACKET_DATA_SIZE ) ||
 		( send.GetSize() > MAX_PACKET_DATA_SIZE ) ) {
@@ -721,7 +913,8 @@ void Socket::PacketData(Data &send, Data &receive, int timeout)
 	receive.Zap();
 
 	// send non-fragmented
-	Send(send, inFrag, timeout);
+	SyncSend(send, timeout);
+	Receive(inFrag, timeout);
 
 	bool done = false;
 	int blankCount = 0;
@@ -738,8 +931,8 @@ void Socket::PacketData(Data &send, Data &receive, int timeout)
 			switch( rpack->command )
 			{
 			case SB_COMMAND_SEQUENCE_HANDSHAKE:
-				CheckSequence(inFrag);
-				if (!m_zero->IsSequencePacketHidden())
+//				CheckSequence(inFrag);
+				if( done_on_sequence )
 					done = true;
 				break;
 
@@ -786,175 +979,9 @@ void Socket::PacketData(Data &send, Data &receive, int timeout)
 	}
 }
 
-// sends the send packet down to the device, fragmenting if
-// necessary, and returns the response in receive, defragmenting
-// if needed
-// Blocks until response received or timed out in Usb::Device
-//
-// This is primarily for Desktop Database packets... Javaloader
-// packets use PacketData().
-//
-void Socket::Packet(Data &send, Data &receive, int timeout)
-{
-	MAKE_PACKET(spack, send);
-	if( send.GetSize() < MIN_PACKET_SIZE ||
-	    (spack->command != SB_COMMAND_DB_DATA &&
-	     spack->command != SB_COMMAND_DB_DONE) )
-	{
-		// we don't do that around here
-		eout("unknown send data in Packet(): " << send);
-		throw std::logic_error("Socket: unknown send data in Packet()");
-	}
-
-	Data inFrag;
-	receive.Zap();
-
-	if( send.GetSize() <= MAX_PACKET_SIZE ) {
-		// send non-fragmented
-		Send(send, inFrag, timeout);
-	}
-	else {
-		// send fragmented
-		unsigned int offset = 0;
-		Data outFrag;
-
-		// You haven't to sequence packet while the whole packet isn't sent
-		//  a) No sequence received packet
-		//  b) 1°) Sent framgment 1/N
-		//     2°) Sent framgment 2/N
-		//         ...
-		//     N°) Before sent fragment N/N, I enable the sequence packet process.
-		//         Sent framgment N/N
-		HideSequencePacket(false);
-
-		do {
-			offset = SocketZero::MakeNextFragment(send, outFrag, offset);
-
-			// Is last packet ?
-			MAKE_PACKET(spack, outFrag);
-
-			if (spack->command != SB_COMMAND_DB_FRAGMENTED)
-				HideSequencePacket(true);
-
-			Send(outFrag, inFrag, timeout);
-
-			// only process sequence handshakes... once we
-			// get to the last fragment, we fall through to normal
-			// processing below
-			if (spack->command != SB_COMMAND_DB_FRAGMENTED) {
-				MAKE_PACKET(rpack, inFrag);
-
-				if( offset && inFrag.GetSize() > 0 ) {
-					Protocol::CheckSize(inFrag, SB_PACKET_HEADER_SIZE);
-
-					switch( rpack->command )
-					{
-					case SB_COMMAND_SEQUENCE_HANDSHAKE:
-						CheckSequence(inFrag);
-						break;
-
-					default: {
-						std::ostringstream oss;
-						oss << "Socket: (send) unhandled packet in Packet(): 0x" << std::hex << (unsigned int)rpack->command;
-						eout(oss.str());
-						throw Error(oss.str());
-						}
-						break;
-					}
-				}
-			}
-
-		} while( offset > 0 );
-
-		// To be sure that it's clean...
-		HideSequencePacket(true);
-	}
-
-	bool done = false, frag = false;
-	int blankCount = 0;
-	while( !done ) {
-		MAKE_PACKET(rpack, inFrag);
-
-		// check the packet's validity
-		if( inFrag.GetSize() > 0 ) {
-			blankCount = 0;
-
-			Protocol::CheckSize(inFrag, SB_PACKET_HEADER_SIZE);
-
-			switch( rpack->command )
-			{
-			case SB_COMMAND_SEQUENCE_HANDSHAKE:
-				CheckSequence(inFrag);
-				break;
-
-			case SB_COMMAND_DB_DATA:
-				if( frag ) {
-					SocketZero::AppendFragment(receive, inFrag);
-				}
-				else {
-					receive = inFrag;
-				}
-				done = true;
-				break;
-
-			case SB_COMMAND_DB_FRAGMENTED:
-				SocketZero::AppendFragment(receive, inFrag);
-				frag = true;
-				break;
-
-			case SB_COMMAND_DB_DONE:
-				receive = inFrag;
-				done = true;
-				break;
-
-			default: {
-				std::ostringstream oss;
-				oss << "Socket: (read) unhandled packet in Packet(): 0x" << std::hex << (unsigned int)rpack->command;
-				eout(oss.str());
-				throw Error(oss.str());
-				}
-				break;
-			}
-		}
-		else {
-			blankCount++;
-			//std::cerr << "Blank! " << blankCount << std::endl;
-			if( blankCount == 10 ) {
-				// only ask for more data on stalled sockets
-				// for so long
-				throw Error("Socket: 10 blank packets received");
-			}
-		}
-
-		if( !done ) {
-			// not done yet, ask for another read
-			Receive(inFrag);
-		}
-	}
-}
-
-void Socket::Packet(Barry::Packet &packet, int timeout)
-{
-	Packet(packet.m_send, packet.m_receive, timeout);
-}
-
-void Socket::Packet(Barry::JLPacket &packet, int timeout)
-{
-	if( packet.HasData() ) {
-		HideSequencePacket(false);
-		PacketData(packet.m_cmd, packet.m_receive, timeout);
-		HideSequencePacket(true);
-		PacketData(packet.m_data, packet.m_receive, timeout);
-	}
-	else {
-		PacketData(packet.m_cmd, packet.m_receive, timeout);
-	}
-}
-
-void Socket::NextRecord(Data &receive)
+void SocketBase::NextRecord(Data &receive)
 {
 	Barry::Protocol::Packet packet;
-	packet.socket = htobs(GetSocket());
 	packet.size = htobs(7);
 	packet.command = SB_COMMAND_DB_DONE;
 	packet.u.db.tableCmd = 0;
@@ -964,20 +991,52 @@ void Socket::NextRecord(Data &receive)
 	Packet(command, receive);
 }
 
-void Socket::RegisterInterest(SocketRoutingQueue::SocketDataHandler handler,
-				void *context)
+
+
+//////////////////////////////////////////////////////////////////////////////
+// Socket class
+
+Socket::Socket( SocketZero &zero,
+		uint16_t socket,
+		uint8_t closeFlag)
+	: m_zero(&zero)
+	, m_socket(socket)
+	, m_closeFlag(closeFlag)
+	, m_registered(false)
+	, m_sequence(new Data)
 {
-	if( !m_zero->m_queue )
-		throw std::logic_error("SocketRoutingQueue required in SocketZero in order to call Socket::RegisterInterest()");
-
-	if( m_registered )
-		throw std::logic_error("Socket already registered in Socket::RegisterInterest()!");
-
-	m_zero->m_queue->RegisterInterest(m_socket, handler, context);
-	m_registered = true;
 }
 
-void Socket::UnregisterInterest()
+Socket::~Socket()
+{
+	// trap exceptions in the destructor
+	try {
+		// a non-default socket has been opened, close it
+		LocalClose();
+	}
+	catch( std::runtime_error &re ) {
+		// do nothing... log it?
+		dout("Exception caught in ~Socket: " << re.what());
+	}
+}
+
+
+////////////////////////////////////
+// Socket protected API
+
+void Socket::ForceClosed()
+{
+	m_socket = 0;
+	m_closeFlag = 0;
+}
+
+void Socket::LocalClose()
+{
+	LocalUnregisterInterest();
+	m_zero->Close(*this);
+}
+
+void Socket::LocalUnregisterInterest()
 {
 	if( m_registered ) {
 		if( m_zero->m_queue )
@@ -987,5 +1046,69 @@ void Socket::UnregisterInterest()
 }
 
 
+//
+// Send
+//
+/// Sends 'send' data to device, no receive.
+///
+/// \returns	void
+///
+/// \exception	Usb::Error on underlying bus errors.
+///
+void Socket::RawSend(Data &send, int timeout)
+{
+	// force the socket number to this socket
+	if( send.GetSize() >= SB_PACKET_HEADER_SIZE ) {
+		MAKE_PACKETPTR_BUF(spack, send.GetBuffer());
+		spack->socket = htobs(m_socket);
+	}
+	m_zero->RawSend(send, timeout);
+}
+
+void Socket::SyncSend(Data &send, int timeout)
+{
+	RawSend(send, timeout);
+	Receive(*m_sequence, timeout);
+	if( !Protocol::IsSequencePacket(*m_sequence) )
+		throw Barry::Error("Non-sequence packet in Socket::SyncSend()");
+	CheckSequence(*m_sequence);
+}
+
+void Socket::Receive(Data &receive, int timeout)
+{
+	if( m_registered ) {
+		if( m_zero->m_queue ) {
+			if( !m_zero->m_queue->SocketRead(m_socket, receive, timeout) )
+				throw Timeout("Socket::Receive: queue SocketRead returned false (likely a timeout)");
+		}
+		else {
+			throw std::logic_error("NULL queue pointer in a registered socket read.");
+		}
+		ddout("Socket::Receive: Endpoint "
+			<< (m_zero->m_queue ? m_zero->m_queue->GetReadEp() : m_zero->m_readEp)
+			<< "\nReceived:\n" << receive);
+	}
+	else {
+		m_zero->RawReceive(receive, timeout);
+	}
+}
+
+
+void Socket::RegisterInterest(SocketRoutingQueue::SocketDataHandlerPtr handler)
+{
+	if( !m_zero->m_queue )
+		throw std::logic_error("SocketRoutingQueue required in SocketZero in order to call Socket::RegisterInterest()");
+
+	if( m_registered ) {
+		std::ostringstream oss;
+		oss << "Socket (" << m_socket << ") already registered in Socket::RegisterInterest()!";
+		throw std::logic_error(oss.str());
+	}
+
+	m_zero->m_queue->RegisterInterest(m_socket, handler);
+	m_registered = true;
+}
+
+
 } // namespace Barry
 
