@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,23 +27,26 @@
 #include <queue>
 #include <memory>
 #include "router.h"
+#include "data.h"
 
 // forward declarations
 namespace Usb { class Device; }
 namespace Barry {
-	class Data;
 	class Packet;
 	class JLPacket;
+	class JVMPacket;
 	class SocketRoutingQueue;
 }
 
 namespace Barry {
 
+class SocketBase;
 class Socket;
-typedef std::auto_ptr<Socket>	SocketHandle;
+typedef std::auto_ptr<SocketBase>	SocketHandle;
 
 class BXEXPORT SocketZero
 {
+	friend class SocketBase;
 	friend class Socket;
 
 	Usb::Device *m_dev;
@@ -57,10 +60,11 @@ class BXEXPORT SocketZero
 	bool m_halfOpen;
 	uint32_t m_challengeSeed;
 	unsigned int m_remainingTries;
+	bool m_modeSequencePacketSeen;
 
-	bool m_hideSequencePacket;
-
-	bool m_resetOnClose;
+	// pushback for out of order sequence packets
+	Data m_pushback_buffer;
+	bool m_pushback;
 
 private:
 	static void AppendFragment(Data &whole, const Data &fragment);
@@ -76,13 +80,12 @@ private:
 	void RawSend(Data &send, int timeout = -1);
 	void RawReceive(Data &receive, int timeout = -1);
 
+	void Pushback(const Data &buf);
+
 protected:
-	bool SequencePacket(const Data &data);
-	bool IsSequencePacketHidden() { return m_hideSequencePacket; }
+	bool IsSequencePacketHidden() { return false; }
 
 public:
-	void SetResetOnClose(bool flag) { m_resetOnClose = flag; }
-	void HideSequencePacket(bool flag) { m_hideSequencePacket = flag; }
 	explicit SocketZero(SocketRoutingQueue &queue, int writeEndpoint,
 		uint8_t zeroSocketSequenceStart = 0);
 	SocketZero(Usb::Device &dev, int writeEndpoint, int readEndpoint,
@@ -109,6 +112,70 @@ public:
 	void Close(Socket &socket);
 };
 
+class BXEXPORT SocketBase
+{
+	bool m_resetOnClose;
+
+protected:
+	void CheckSequence(const Data &seq);
+
+public:
+	SocketBase()
+		: m_resetOnClose(false)
+	{
+	}
+
+	virtual ~SocketBase();
+
+	//
+	// Virtual Socket API
+	//
+	virtual void Close() = 0;
+
+	// FIXME - do I need RawSend?  Or just a good fragmenter?
+	virtual void RawSend(Data &send, int timeout = -1) = 0;
+	virtual void SyncSend(Data &send, int timeout = -1) = 0;
+	virtual void Receive(Data &receive, int timeout = -1) = 0;
+
+	virtual void RegisterInterest(Barry::SocketRoutingQueue::SocketDataHandlerPtr handler = Barry::SocketRoutingQueue::SocketDataHandlerPtr()) = 0;
+	virtual void UnregisterInterest() = 0;
+
+	void ResetOnClose(bool reset = true) { m_resetOnClose = reset; }
+	bool IsResetOnClose() const { return m_resetOnClose; }
+
+	//
+	// Convenience functions that just call the virtuals above
+	//
+	void DBFragSend(Data &send, int timeout = -1);
+	void Send(Data &send, Data &receive, int timeout = -1);
+	void Send(Barry::Packet &packet, int timeout = -1);
+
+	//
+	// Protocol based functions... all use the above virtual functions
+	//
+
+	// sends the send packet down to the device, fragmenting if
+	// necessary, and returns the response in receive, defragmenting
+	// if needed
+	// Blocks until response received or timed out in Usb::Device
+	void Packet(Data &send, Data &receive, int timeout = -1);
+	void Packet(Barry::Packet &packet, int timeout = -1);
+	void Packet(Barry::JLPacket &packet, int timeout = -1);
+	void Packet(Barry::JVMPacket &packet, int timeout = -1);
+
+	// Use this function to send packet to JVM instead of Packet function
+	// FIXME
+	void PacketJVM(Data &send, Data &receive, int timeout = -1);
+
+	// Use this function to send data packet instead of Packet function
+	// Indeed, Packet function is used to send command (and not data)
+	// FIXME
+	void PacketData(Data &send, Data &receive, bool done_on_sequence,
+		int timeout = -1);
+
+	// some handy wrappers for the Packet() interface
+	void NextRecord(Data &receive);
+};
 
 //
 // Socket class
@@ -124,7 +191,7 @@ public:
 ///
 /// Requires an active Usb::Device object to work on.
 ///
-class BXEXPORT Socket
+class BXEXPORT Socket : public SocketBase
 {
 	friend class SocketZero;
 
@@ -134,55 +201,44 @@ class BXEXPORT Socket
 
 	bool m_registered;
 
+	// buffer data
+	std::auto_ptr<Data> m_sequence;
+
 protected:
-	void CheckSequence(const Data &seq);
 	void ForceClosed();
 
+	// These "Local" function are non-virtual worker functions,
+	// which are safe to be called from the destructor.
+	// If you override the virtual versions in your derived class,
+	// make sure your "Local" versions call us.
+	void LocalClose();
+	void LocalUnregisterInterest();
+
+	uint16_t GetSocket() const { return m_socket; }
+	uint8_t GetCloseFlag() const { return m_closeFlag; }
+
 	Socket(SocketZero &zero, uint16_t socket, uint8_t closeFlag);
 
 public:
 	~Socket();
 
-	uint16_t GetSocket() const { return m_socket; }
-	uint8_t GetCloseFlag() const { return m_closeFlag; }
-
-	void Close();
-
-	// Send and Receive are available before Open...
-	// an unopened socket defaults to socket 0, which you need
-	// in order to set the blackberry mode
-	// The send function will overwrite the zeroSocketSequence byte
-	// *inside* the packet, if the current m_socket is 0.
-	void Send(Data &send, int timeout = -1);	// send only
-	void Send(Data &send, Data &receive, int timeout = -1); // send+recv
-	void Send(Barry::Packet &packet, int timeout = -1);
+	//
+	// virtual overrides
+	//
+	void Close() { LocalClose(); }
+	void RawSend(Data &send, int timeout = -1);
+	void SyncSend(Data &send, int timeout = -1);
 	void Receive(Data &receive, int timeout = -1);
 
-	// sends the send packet down to the device, fragmenting if
-	// necessary, and returns the response in receive, defragmenting
-	// if needed
-	// Blocks until response received or timed out in Usb::Device
-	void Packet(Data &send, Data &receive, int timeout = -1);
-	void Packet(Barry::Packet &packet, int timeout = -1);
-	void Packet(Barry::JLPacket &packet, int timeout = -1);
-
-	// Use this function to send data packet instead of Packet function
-	// Indeed, Packet function is used to send command (and not data)
-	void PacketData(Data &send, Data &receive, int timeout = -1);
-
-	// some handy wrappers for the Packet() interface
-	void NextRecord(Data &receive);
 
 	// Register a callback for incoming data from the device.
 	// This function assumes that this socket is based on a socketZero
 	// that has a SocketRoutingQueue, otherwise throws logic_error.
-	void RegisterInterest(SocketRoutingQueue::SocketDataHandler handler, void *context);
-	void UnregisterInterest();
-
-
-	// This function is quickly written
-	// It's very durty :( (but it's usefull to test...)
-	void HideSequencePacket(bool flag) { m_zero->HideSequencePacket(flag); }
+	// It also assumes that nothing has been registered before.
+	// If you wish to re-register, call UnregisterInterest() first,
+	// which is safe to call as many times as you like.
+	void RegisterInterest(Barry::SocketRoutingQueue::SocketDataHandlerPtr handler = Barry::SocketRoutingQueue::SocketDataHandlerPtr());
+	void UnregisterInterest() { LocalUnregisterInterest(); }
 };
 
 
