@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,9 +22,13 @@
 #ifndef __BARRY_STRNLEN_H__
 #define __BARRY_STRNLEN_H__
 
-#include "config.h"
+#include "config.h"		// strnlen.h is not installed, so this is safe
 #include <string.h>
 
+#ifdef WIN32
+#define HAVE_WORKING_STRNLEN 1
+#endif
+
 // this is always defined by configure, if not, then the autoconf
 // sources changed (likely in /usr/share/autoconf/autoconf/functions.m4)
 // and configure.ac is no longer accurate
