@@ -6,7 +6,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -53,31 +53,42 @@ typedef long min1900_t;
 BXEXPORT min1900_t time2min(time_t t);
 BXEXPORT time_t min2time(min1900_t m);
 
-// FIXME - turn TimeZone into a C typedef and wrap this in extern "C"
+// FIXME - turn StaticTimeZone into a C typedef and wrap this in extern "C"
 // so the data can be used in both C and C++ libraries
-struct BXEXPORT TimeZone
+//
+// This is named StaticTimeZone since the time zone table is hard coded
+// in the library.  If you want to know what the device's idea of time zones
+// is, then extract the time zone database using the TimeZone record class.
+//
+// See also the TimeZones class, which unifies access to the static and
+// dynamic time zone tables in one class API.
+//
+struct BXEXPORT StaticTimeZone
 {
-	unsigned short Code;
+	uint16_t Code;
 	signed short HourOffset;
 	signed short MinOffset;
 	const char *Name;
 };
 
 // FIXME - put this somewhere for both C and C++
-#define TIME_ZONE_CODE_ERR	0xffff
+#define STATIC_TIME_ZONE_CODE_ERR	0xffff
 
-BXEXPORT const TimeZone* GetTimeZoneTable();
-BXEXPORT const TimeZone* GetTimeZone(unsigned short Code);
-BXEXPORT unsigned short GetTimeZoneCode(signed short HourOffset,
+BXEXPORT const StaticTimeZone* GetStaticTimeZoneTable();
+BXEXPORT const StaticTimeZone* GetStaticTimeZone(uint16_t Code);
+BXEXPORT unsigned short GetStaticTimeZoneCode(signed short HourOffset,
 	signed short MinOffset = 0);
 
 // Message time conversion stuff
-BXEXPORT time_t DayToDate( unsigned short Day );
+BXEXPORT time_t DayToDate( uint16_t Day );
 BXEXPORT time_t Message2Time(uint16_t r_date, uint16_t r_time);
 
 // Thread timeout creation
 BXEXPORT struct timespec* ThreadTimeout(int timeout_ms, struct timespec *spec);
 
+// Utility functions
+BXEXPORT int DaysInMonth(struct tm &t);
+
 } // namespace Barry
 
 #endif
