@@ -3,7 +3,7 @@
 //
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <string>
 #include <string.h>
 #include <barry/data.h>
+#include "i18n.h"
 
 using namespace std;
 
@@ -109,6 +110,8 @@ void SplitHex(const char *logmarker, const char *str, Barry::Data &data)
 
 int main(int argc, char *argv[])
 {
+	INIT_I18N(PACKAGE);
+
 	cout.sync_with_stdio(false);
 
 	if( argc < 2 ) {
