@@ -4,7 +4,7 @@
 ///
 ///		This file is part of the Barry project:
 ///
-///		http://www.netdirect.ca/software/packages/barry/index.php
+///		http://www.netdirect.ca/software/packages/barry
 ///		http://sourceforge.net/projects/barry
 ///
 ///		Compile with the following command (needs libusb):
@@ -13,7 +13,7 @@
 ///
 
 /*
-    Copyright (C) 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <usb.h>
 #include <stdio.h>
 #include <unistd.h>
+#include "i18n.h"
 
 #define VENDOR_RIM		0x0fca
 #define PRODUCT_RIM_BLACKBERRY	0x0001
@@ -56,12 +57,14 @@ int main()
 {
 	struct usb_bus *busses;
 
+	INIT_I18N(PACKAGE);
+
 	usb_init();
 	usb_find_busses();
 	usb_find_devices();
 	busses = usb_get_busses();
 
-	printf("Scanning for Blackberry devices...\n");
+	printf(_("Scanning for Blackberry devices...\n"));
 	int found = 0;
 
 	struct usb_bus *bus;
@@ -75,16 +78,16 @@ int main()
 			     dev->descriptor.idProduct == PRODUCT_RIM_PEARL ||
 			     dev->descriptor.idProduct == PRODUCT_RIM_PEARL_8120 ||
 			     dev->descriptor.idProduct == PRODUCT_RIM_PEARL_DUAL ) ) {
-			    	printf("Found...");
-				printf("attempting to reset.\n");
+				printf(_("Found..."));
+				printf(_("attempting to reset.\n"));
 				if( reset(dev) )
 					found++;
 				else
-					printf("Can't reset device on bus %s, devnum %u\n", bus->dirname, (unsigned int) dev->devnum);
+					printf(_("Can't reset device on bus %s, devnum %u\n"), bus->dirname, (unsigned int) dev->devnum);
 			}
 		}
 	}
 
-	printf("%d device%s reset.\n", found, found > 1 ? "s" : "");
+	printf(_("%d device%s reset.\n"), found, found > 1 ? "s" : "");
 }
 
