@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,30 @@
     root directory of this project for more details.
 */
 
+//
+// This define is used in barry/barry.h to signal inclusion of Boost
+// serialization headers.  It is intended to be used by applications,
+// so we shouldn't mess with it.
+//
+// But all actual Boost related code is now stuffed into util.cc, safely
+// locked away from other code.  So we don't need the Boost headers, we
+// just need a flag for our own functionality.  So translate this define
+// into our own, and undef to skip the Boost headers, and the compile speed
+// slowdown that it creates.
+//
+#ifdef __BARRY_BOOST_MODE__
+#define __BTOOL_BOOST_MODE__
+#endif
+#undef __BARRY_BOOST_MODE__
+
 #include <barry/barry.h>
+#ifdef __BARRY_SYNC_MODE__
+#include <barry/barrysync.h>
+#endif
+#ifdef __BARRY_BACKUP_MODE__
+#include <barry/barrybackup.h>
+#endif
+
 #include <iomanip>
 #include <iostream>
 #include <fstream>
@@ -27,34 +50,44 @@
 #include <vector>
 #include <string>
 #include <algorithm>
-#include <getopt.h>
+#include <stdlib.h>
+#include <tr1/memory>
+#include "i18n.h"
+#include "util.h"
+#include "boostwrap.h"
 
+#include "barrygetopt.h"
 
 using namespace std;
+using namespace std::tr1;
 using namespace Barry;
 
+std::map<std::string, std::string> SortKeys;
+
 void Usage()
 {
-   int major, minor;
-   const char *Version = Barry::Version(major, minor);
+   int logical, major, minor;
+   const char *Version = Barry::Version(logical, major, minor);
 
    cerr
    << "btool - Command line USB Blackberry Test Tool\n"
-   << "        Copyright 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)\n"
+   << "        Copyright 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)\n"
    << "        Using: " << Version << "\n"
    << "        Compiled "
-#ifdef __BARRY_BOOST_MODE__
+#ifdef __BTOOL_BOOST_MODE__
    << "with"
 #else
    << "without"
 #endif
    << " Boost support\n"
    << "\n"
+   << "   -b file   Filename to save or load a Barry Backup to (tar.gz)\n"
    << "   -B bus    Specify which USB bus to search on\n"
    << "   -N dev    Specify which system device, using system specific string\n"
    << "\n"
-   << "   -a db     Clear database 'db' FROM device\n"
-   << "             Can be used multiple times to clear more than one DB\n"
+   << "   -a db     Erase / clear database 'db' FROM device, deleting all\n"
+   << "             its records.  Can be used multiple times to clear more\n"
+   << "             than one DB.\n"
    << "   -c dn     Convert address book database to LDIF format, using the\n"
    << "             specified baseDN\n"
    << "   -C dnattr LDIF attribute name to use when building the FQDN\n"
@@ -66,12 +99,18 @@ void Usage()
    << "             endpoint pair.  Example: -e 83,5\n"
    << "             Note: Endpoints are specified in hex.\n"
    << "             You should never need to use this option.\n"
-#ifdef __BARRY_BOOST_MODE__
+#ifdef __BTOOL_BOOST_MODE__
    << "   -f file   Filename to save or load handheld data to/from\n"
 #endif
+   << "   -F sort   Field name by which to sort the output.  Note that the\n"
+   << "             format of this field is special: 'DBName:field1,field2'\n"
+   << "             with no spaces unless the spaces are part of the name.\n"
+   << "             Can be used multiple times, to match your -d options.\n"
+   << "             Example: -F 'Address Book:Company,LastName,FirstName'\n"
    << "   -h        This help\n"
    << "   -i cs     International charset for string conversions\n"
    << "             Valid values here are available with 'iconv --list'\n"
+   << "   -I        Sort records before output\n"
    << "   -l        List devices\n"
    << "   -L        List Contact field names\n"
    << "   -m        Map LDIF name to Contact field / Unmap LDIF name\n"
@@ -83,10 +122,14 @@ void Usage()
    << "             If only one device is plugged in, this flag is optional\n"
    << "   -P pass   Simplistic method to specify device password\n"
    << "   -s db     Save database 'db' TO device from data loaded from -f file\n"
-   << "   -S        Show list of supported database parsers\n"
+   << "   -S        Show list of supported database parsers.  Use twice to\n"
+   << "             display fields names as well.\n"
    << "   -t        Show database database table\n"
    << "   -T db     Show record state table for given database\n"
    << "   -v        Dump protocol data during operation\n"
+#ifdef __BARRY_SYNC_MODE__
+   << "   -V        Dump records using MIME vformats where possible\n"
+#endif
    << "   -X        Reset device\n"
    << "   -z        Use non-threaded sockets\n"
    << "   -Z        Use threaded socket router (default)\n"
@@ -123,220 +166,262 @@ struct Store
 	mutable typename std::vector<Record>::const_iterator rec_it;
 	std::string filename;
 	bool load;
-	int count;
+	bool immediate_display;
+	bool vformat_mode;
+	int from_device_count;
+	mutable int to_device_count;
 
-	Store(const string &filename, bool load)
+	Store(const string &filename, bool load, bool immediate_display,
+			bool vformat_mode)
 		: rec_it(records.end()),
 		filename(filename),
 		load(load),
-		count(0)
+		immediate_display(immediate_display && !SortKeys.size()),
+		vformat_mode(vformat_mode),
+		from_device_count(0),
+		to_device_count(0)
 	{
-#ifdef __BARRY_BOOST_MODE__
-		try {
-
-			if( load && filename.size() ) {
-				// filename is available, attempt to load
-				cout << "Loading: " << filename << endl;
-				ifstream ifs(filename.c_str());
-				std::string dbName;
-				getline(ifs, dbName);
-				boost::archive::text_iarchive ia(ifs);
-				ia >> records;
-				cout << records.size()
-				     << " records loaded from '"
-				     << filename << "'" << endl;
-				sort(records.begin(), records.end());
-				rec_it = records.begin();
-
-				// debugging aid
-				typename std::vector<Record>::const_iterator beg = records.begin(), end = records.end();
-				for( ; beg != end; beg++ ) {
-					cout << (*beg) << endl;
-				}
+#ifdef __BTOOL_BOOST_MODE__
+		if( load && filename.size() ) {
+			// filename is available, attempt to load
+			cout << "Loading: " << filename << endl;
+			string errmsg, dbName;
+			if( !LoadBoostFile(filename, records, dbName, errmsg) ) {
+				cerr << errmsg << endl;
 			}
+			cout << records.size()
+			     << " records loaded from '"
+			     << filename << "'" << endl;
+			sort(records.begin(), records.end());
+			rec_it = records.begin();
 
-		} catch( boost::archive::archive_exception &ae ) {
-			cerr << "Archive exception in ~Store(): "
-			     << ae.what() << endl;
+			// debugging aid
+			typename std::vector<Record>::const_iterator beg = records.begin(), end = records.end();
+			for( ; beg != end; beg++ ) {
+				cout << (*beg) << endl;
+			}
 		}
 #endif
 	}
+
 	~Store()
 	{
-		cout << "Store counted " << dec << count << " records." << endl;
-#ifdef __BARRY_BOOST_MODE__
-		try {
-
-			if( !load && filename.size() ) {
-				// filename is available, attempt to save
-				cout << "Saving: " << filename << endl;
-				const std::vector<Record> &r = records;
-				ofstream ofs(filename.c_str());
-				ofs << Record::GetDBName() << endl;
-				boost::archive::text_oarchive oa(ofs);
-				oa << r;
-				cout << dec << r.size() << " records saved to '"
-					<< filename << "'" << endl;
+		if( !immediate_display ) {
+			// not dumped yet, sort then dump
+			if( SortKeys.size() && SortKeys.find(Record::GetDBName()) != SortKeys.end() ) {
+				sort(records.begin(), records.end(),
+					NamedFieldCmp<Record>(SortKeys[Record::GetDBName()]));
 			}
+			else {
+				sort(records.begin(), records.end());
+			}
+			DumpAll();
+		}
 
-		} catch( boost::archive::archive_exception &ae ) {
-			cerr << "Archive exception in ~Store(): "
-			     << ae.what() << endl;
+		cout << "Store counted " << dec << from_device_count << " records read from device, and " << dec << to_device_count << " records written to device." << endl;
+#ifdef __BTOOL_BOOST_MODE__
+		if( !load && filename.size() ) {
+			// filename is available, attempt to save
+			cout << "Saving: " << filename << endl;
+			string errmsg;
+			if( !SaveBoostFile(filename, records, errmsg) ) {
+				cerr << errmsg << endl;
+			}
+			cout << dec << records.size() << " records saved to '"
+				<< filename << "'" << endl;
 		}
 #endif
 	}
 
+	void DumpAll()
+	{
+		typename vector<Record>::const_iterator i = records.begin();
+		for( ; i != records.end(); ++i ) {
+			Dump(*i);
+		}
+	}
+
+	void Dump(const Record &rec)
+	{
+		if( vformat_mode ) {
+#ifdef __BARRY_SYNC_MODE__
+			MimeDump<Record> md;
+			md.Dump(cout, rec);
+#endif
+		}
+		else {
+			cout << rec << endl;
+		}
+	}
+
 	// storage operator
 	void operator()(const Record &rec)
 	{
-		count++;
-		std::cout << rec << std::endl;
+		from_device_count++;
+		if( immediate_display )
+			Dump(rec);
 		records.push_back(rec);
 	}
 
 	// retrieval operator
-	bool operator()(Record &rec, unsigned int databaseId) const
+	bool operator()(Record &rec, Builder &builder) const
 	{
 		if( rec_it == records.end() )
 			return false;
+		to_device_count++;
 		rec = *rec_it;
 		rec_it++;
 		return true;
 	}
 };
 
-class DataDumpParser : public Barry::Parser
+shared_ptr<Parser> GetParser(const string &name,
+			const string &filename,
+			bool null_parser,
+			bool immediate_display,
+			bool vformat_mode,
+			bool bbackup_mode)
 {
-	uint32_t m_id;
+	bool dnow = immediate_display;
+	bool vmode = vformat_mode;
 
-public:
-	virtual void Clear() {}
-
-	virtual void SetIds(uint8_t RecType, uint32_t UniqueId)
-	{
-		m_id = UniqueId;
-	}
-
-	virtual void ParseHeader(const Data &, size_t &) {}
-
-	virtual void ParseFields(const Barry::Data &data, size_t &offset,
-				const IConverter *ic)
-	{
-		std::cout << "Raw record dump for record: "
-			<< std::hex << m_id << std::endl;
-		std::cout << data << std::endl;
-	}
-
-	virtual void Store() {}
-};
-
-auto_ptr<Parser> GetParser(const string &name, const string &filename, bool null_parser)
-{
 	if( null_parser ) {
 		// use null parser
-		return auto_ptr<Parser>( new DataDumpParser );
+		return shared_ptr<Parser>( new Barry::HexDumpParser(cout) );
+	}
+	else if( bbackup_mode ) {
+#ifdef __BARRY_BACKUP_MODE__
+		// Only one backup file per run
+		static shared_ptr<Parser> backup;
+		if( !backup.get() ) {
+			backup.reset( new Backup(filename) );
+		}
+		return backup;
+#else
+		return shared_ptr<Parser>( new Barry::HexDumpParser(cout) );
+#endif
 	}
 	// check for recognized database names
 	else if( name == Contact::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<Contact, Store<Contact> > (
-				new Store<Contact>(filename, false)));
+				new Store<Contact>(filename, false, dnow, vmode)));
 	}
 	else if( name == Message::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<Message, Store<Message> > (
-				new Store<Message>(filename, false)));
+				new Store<Message>(filename, false, dnow, vmode)));
 	}
 	else if( name == Calendar::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<Calendar, Store<Calendar> > (
-				new Store<Calendar>(filename, false)));
+				new Store<Calendar>(filename, false, dnow, vmode)));
+	}
+	else if( name == CalendarAll::GetDBName() ) {
+		return shared_ptr<Parser>(
+			new RecordParser<CalendarAll, Store<CalendarAll> > (
+				new Store<CalendarAll>(filename, false, dnow, vmode)));
 	}
 	else if( name == CallLog::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<CallLog, Store<CallLog> > (
-				new Store<CallLog>(filename, false)));
+				new Store<CallLog>(filename, false, dnow, vmode)));
+	}
+	else if( name == Bookmark::GetDBName() ) {
+		return shared_ptr<Parser>(
+			new RecordParser<Bookmark, Store<Bookmark> > (
+				new Store<Bookmark>(filename, false, dnow, vmode)));
 	}
 	else if( name == ServiceBook::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<ServiceBook, Store<ServiceBook> > (
-				new Store<ServiceBook>(filename, false)));
+				new Store<ServiceBook>(filename, false, dnow, vmode)));
 	}
 
 	else if( name == Memo::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<Memo, Store<Memo> > (
-				new Store<Memo>(filename, false)));
+				new Store<Memo>(filename, false, dnow, vmode)));
 	}
 	else if( name == Task::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<Task, Store<Task> > (
-				new Store<Task>(filename, false)));
+				new Store<Task>(filename, false, dnow, vmode)));
 	}
 	else if( name == PINMessage::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<PINMessage, Store<PINMessage> > (
-				new Store<PINMessage>(filename, false)));
+				new Store<PINMessage>(filename, false, dnow, vmode)));
 	}
 	else if( name == SavedMessage::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<SavedMessage, Store<SavedMessage> > (
-				new Store<SavedMessage>(filename, false)));
+				new Store<SavedMessage>(filename, false, dnow, vmode)));
 	}
 	else if( name == Sms::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<Sms, Store<Sms> > (
-				new Store<Sms>(filename, false)));
+				new Store<Sms>(filename, false, dnow, vmode)));
 	}
 	else if( name == Folder::GetDBName() ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<Folder, Store<Folder> > (
-				new Store<Folder>(filename, false)));
+				new Store<Folder>(filename, false, dnow, vmode)));
 	}
-	else if( name == Timezone::GetDBName() ) {
-		return auto_ptr<Parser>(
-			new RecordParser<Timezone, Store<Timezone> > (
-				new Store<Timezone>(filename, false)));
+	else if( name == TimeZone::GetDBName() ) {
+		return shared_ptr<Parser>(
+			new RecordParser<TimeZone, Store<TimeZone> > (
+				new Store<TimeZone>(filename, false, dnow, vmode)));
+	}
+	else if( name == HandheldAgent::GetDBName() ) {
+		return shared_ptr<Parser>(
+			new RecordParser<HandheldAgent, Store<HandheldAgent> > (
+				new Store<HandheldAgent>(filename, false, dnow, vmode)));
 	}
 	else {
 		// unknown database, use null parser
-		return auto_ptr<Parser>( new DataDumpParser );
+		return shared_ptr<Parser>( new Barry::HexDumpParser(cout) );
 	}
 }
 
-auto_ptr<Builder> GetBuilder(const string &name, const string &filename)
+shared_ptr<Builder> GetBuilder(const string &name, const string &filename)
 {
 	// check for recognized database names
 	if( name == Contact::GetDBName() ) {
-		return auto_ptr<Builder>(
+		return shared_ptr<Builder>(
 			new RecordBuilder<Contact, Store<Contact> > (
-				new Store<Contact>(filename, true)));
+				new Store<Contact>(filename, true, true, false)));
 	}
 	else if( name == Calendar::GetDBName() ) {
-		return auto_ptr<Builder>(
+		return shared_ptr<Builder>(
 			new RecordBuilder<Calendar, Store<Calendar> > (
-				new Store<Calendar>(filename, true)));
+				new Store<Calendar>(filename, true, true, false)));
+	}
+	else if( name == CalendarAll::GetDBName() ) {
+		return shared_ptr<Builder>(
+			new RecordBuilder<CalendarAll, Store<CalendarAll> > (
+				new Store<CalendarAll>(filename, true, true, false)));
 	}
 	else if( name == Memo::GetDBName() ) {
-		return auto_ptr<Builder>(
+		return shared_ptr<Builder>(
 			new RecordBuilder<Memo, Store<Memo> > (
-				new Store<Memo>(filename, true)));
+				new Store<Memo>(filename, true, true, false)));
 	}
 	else if( name == Task::GetDBName() ) {
-		return auto_ptr<Builder>(
+		return shared_ptr<Builder>(
 			new RecordBuilder<Task, Store<Task> > (
-				new Store<Task>(filename, true)));
+				new Store<Task>(filename, true, true, false)));
 	}
 /*
 	else if( name == "Messages" ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<Message, Store<Message> > (
-				new Store<Message>(filename, true)));
+				new Store<Message>(filename, true, true, false)));
 	}
 	else if( name == "Service Book" ) {
-		return auto_ptr<Parser>(
+		return shared_ptr<Parser>(
 			new RecordParser<ServiceBook, Store<ServiceBook> > (
-				new Store<ServiceBook>(filename, true)));
+				new Store<ServiceBook>(filename, true, true, false)));
 	}
 */
 	else {
@@ -344,30 +429,6 @@ auto_ptr<Builder> GetBuilder(const string &name, const string &filename)
 	}
 }
 
-void ShowParsers()
-{
-	cout << "Supported Database parsers:\n"
-	<< "   Address Book\n"
-	<< "   Messages\n"
-	<< "   Calendar\n"
-	<< "   Phone Call Logs\n"
-	<< "   Service Book\n"
-	<< "   Memos\n"
-	<< "   Tasks\n"
-	<< "   PIN Messages\n"
-	<< "   Saved Email Messages\n"
-	<< "   SMS Messages\n"
-	<< "   Folders\n"
-	<< "   Time Zones (read only)\n"
-	<< "\n"
-	<< "Supported Database builders:\n"
-	<< "   Address Book\n"
-	<< "   Calendar\n"
-	<< "   Memo\n"
-	<< "   Task\n"
-	<< endl;
-}
-
 struct StateTableCommand
 {
 	char flag;
@@ -437,8 +498,21 @@ bool ParseEpOverride(const char *arg, Usb::EndpointPair *epp)
 	return true;
 }
 
+void ParseSortKey(const std::string &key)
+{
+	istringstream iss(key);
+	string db, spec;
+	getline(iss, db, ':');
+	getline(iss, spec, ':');
+
+	if( db.size() && spec.size() )
+		SortKeys[db] = spec;
+}
+
 int main(int argc, char *argv[])
 {
+	INIT_I18N(PACKAGE);
+
 	cout.sync_with_stdio(true);	// leave this on, since libusb uses
 					// stdio for debug messages
 
@@ -449,14 +523,19 @@ int main(int argc, char *argv[])
 			show_dbdb = false,
 			ldif_contacts = false,
 			data_dump = false,
+			vformat_mode = false,
 			reset_device = false,
 			list_contact_fields = false,
 			list_ldif_map = false,
 			epp_override = false,
 			threaded_sockets = true,
-			record_state = false,
+			record_state_table = false,
 			clear_database = false,
-			null_parser = false;
+			null_parser = false,
+			bbackup_mode = false,
+			sort_records = false,
+			show_parsers = false,
+			show_fields = false;
 		string ldifBaseDN, ldifDnAttr;
 		string filename;
 		string password;
@@ -469,7 +548,7 @@ int main(int argc, char *argv[])
 
 		// process command line options
 		for(;;) {
-			int cmd = getopt(argc, argv, "a:B:c:C:d:D:e:f:hi:lLm:MnN:p:P:r:R:Ss:tT:vXzZ");
+			int cmd = getopt(argc, argv, "a:b:B:c:C:d:D:e:f:F:hi:IlLm:MnN:p:P:r:R:Ss:tT:vVXzZ");
 			if( cmd == -1 )
 				break;
 
@@ -480,6 +559,23 @@ int main(int argc, char *argv[])
 				clearDbNames.push_back(string(optarg));
 				break;
 
+			case 'b':	// Barry backup filename (tar.gz)
+#ifdef __BARRY_BACKUP_MODE__
+				if( filename.size() == 0 ) {
+					filename = optarg;
+					bbackup_mode = true;
+				}
+				else {
+					cerr << "Do not use -f with -b\n";
+					return 1;
+				}
+#else
+				cerr << "-b option not supported - no Barry "
+					"Backup library support available\n";
+				return 1;
+#endif
+				break;
+
 			case 'B':	// busname
 				busname = optarg;
 				break;
@@ -511,8 +607,14 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'f':	// filename
-#ifdef __BARRY_BOOST_MODE__
-				filename = optarg;
+#ifdef __BTOOL_BOOST_MODE__
+				if( !bbackup_mode && filename.size() == 0 ) {
+					filename = optarg;
+				}
+				else {
+					cerr << "Do not use -f with -b\n";
+					return 1;
+				}
 #else
 				cerr << "-f option not supported - no Boost "
 					"serialization support available\n";
@@ -520,10 +622,18 @@ int main(int argc, char *argv[])
 #endif
 				break;
 
+			case 'F':	// sort key
+				ParseSortKey(optarg);
+				break;
+
 			case 'i':	// international charset (iconv)
 				iconvCharset = optarg;
 				break;
 
+			case 'I':	// sort before dump
+				sort_records = true;
+				break;
+
 			case 'l':	// list only
 				list_only = true;
 				break;
@@ -571,15 +681,18 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'S':	// show supported databases
-				ShowParsers();
-				return 0;
+				if( show_parsers )
+					show_fields = true;
+				else
+					show_parsers = true;
+				break;
 
 			case 't':	// display database database
 				show_dbdb = true;
 				break;
 
 			case 'T':	// show RecordStateTable
-				record_state = true;
+				record_state_table = true;
 				dbNames.push_back(string(optarg));
 				break;
 
@@ -587,6 +700,16 @@ int main(int argc, char *argv[])
 				data_dump = true;
 				break;
 
+			case 'V':	// vformat MIME mode
+#ifdef __BARRY_SYNC_MODE__
+				vformat_mode = true;
+#else
+				cerr << "-V option not supported - no Sync "
+					"library support available\n";
+				return 1;
+#endif
+				break;
+
 			case 'X':	// reset device
 				reset_device = true;
 				break;
@@ -606,9 +729,20 @@ int main(int argc, char *argv[])
 			}
 		}
 
+		if( show_parsers ) {
+			ShowParsers(show_fields, true);
+			ShowBuilders();
+			return 0;
+		}
+
 		// Initialize the barry library.  Must be called before
 		// anything else.
 		Barry::Init(data_dump);
+		if( data_dump ) {
+			int logical, major, minor;
+			const char *Version = Barry::Version(logical, major, minor);
+			cout << Version << endl;
+		}
 
 		// Create an IConverter object if needed
 		auto_ptr<IConverter> ic;
@@ -682,8 +816,8 @@ int main(int argc, char *argv[])
 
 		if( ldif_contacts )
 			cout << "# ";
-		cout << "Using device (PIN): " << setbase(16)
-			<< probe.Get(activeDevice).m_pin << endl;
+		cout << "Using device (PIN): "
+			<< probe.Get(activeDevice).m_pin.Str() << endl;
 
 		if( reset_device ) {
 			Usb::Device dev(probe.Get(activeDevice).m_dev);
@@ -696,13 +830,43 @@ int main(int argc, char *argv[])
 		if( epp_override ) {
 			device.m_ep.read = epOverride.read;
 			device.m_ep.write = epOverride.write;
-			device.m_ep.type = 2;	// FIXME - override this too?
+			// FIXME - override this too?
+			device.m_ep.type = Usb::EndpointDescriptor::BulkType;
 			cout << "Endpoint pair (read,write) overridden with: "
 			     << hex
 			     << (unsigned int) device.m_ep.read << ","
 			     << (unsigned int) device.m_ep.write << endl;
 		}
 
+		//
+		// execute each mode that was turned on
+		//
+
+
+		// Dump current LDIF mapping
+		if( list_ldif_map ) {
+			cout << ldif << endl;
+		}
+
+		// Dump list of Contact field names
+		if( list_contact_fields ) {
+			for( const ContactLdif::NameToFunc *n = ldif.GetFieldNames(); n->name; n++ ) {
+				cout.fill(' ');
+				cout << "  " << left << setw(20) << n->name << ": "
+					<< n->description << endl;
+			}
+		}
+
+		// Check if Desktop access is needed
+		if( !(	show_dbdb ||
+			ldif_contacts ||
+			record_state_table ||
+			clear_database ||
+			stCommands.size() ||
+			dbNames.size() ||
+			saveDbNames.size() ) )
+			return 0;	// done
+
 		//
 		// Create our controller object
 		//
@@ -714,51 +878,30 @@ int main(int argc, char *argv[])
 		// here we want the user to pick.
 		//
 		auto_ptr<SocketRoutingQueue> router;
-		auto_ptr<Barry::Controller> pcon;
 		if( threaded_sockets ) {
 			router.reset( new SocketRoutingQueue );
 			router->SpinoffSimpleReadThread();
-			pcon.reset( new Barry::Controller(device, *router) );
-		}
-		else {
-			pcon.reset( new Barry::Controller(device) );
 		}
 
-		Barry::Controller &con = *pcon;
-		Barry::Mode::Desktop desktop(con, *ic);
-
-		//
-		// execute each mode that was turned on
-		//
+		DesktopConnector connector(password.c_str(),
+			iconvCharset, device, router.get());
+		if( !connector.Connect() ) {
+			// bad password (default action is not to prompt)
+			cerr << connector.GetBadPassword().what() << endl;
+			return 1;
+		}
 
+		Barry::Mode::Desktop &desktop = connector.GetDesktop();
 
 		// Dump list of all databases to stdout
 		if( show_dbdb ) {
 			// open desktop mode socket
-			desktop.Open(password.c_str());
 			cout << desktop.GetDBDB() << endl;
 		}
 
-		// Dump list of Contact field names
-		if( list_contact_fields ) {
-			for( const ContactLdif::NameToFunc *n = ldif.GetFieldNames(); n->name; n++ ) {
-				cout.fill(' ');
-				cout << "  " << left << setw(20) << n->name << ": "
-					<< n->description << endl;
-			}
-		}
-
-		// Dump current LDIF mapping
-		if( list_ldif_map ) {
-			cout << ldif << endl;
-		}
-
 		// Dump list of contacts to an LDAP LDIF file
 		// This uses the Controller convenience templates
 		if( ldif_contacts ) {
-			// make sure we're in desktop mode
-			desktop.Open(password.c_str());
-
 			// create a storage functor object that accepts
 			// Barry::Contact objects as input
 			Contact2Ldif storage(ldif);
@@ -768,14 +911,12 @@ int main(int argc, char *argv[])
 		}
 
 		// Dump record state table to stdout
-		if( record_state ) {
+		if( record_state_table ) {
 			if( dbNames.size() == 0 ) {
 				cout << "No db names to process" << endl;
 				return 1;
 			}
 
-			desktop.Open(password.c_str());
-
 			vector<string>::iterator b = dbNames.begin();
 			for( ; b != dbNames.end(); b++ ) {
 				unsigned int id = desktop.GetDBID(*b);
@@ -794,9 +935,9 @@ int main(int argc, char *argv[])
 				return 1;
 			}
 
-			desktop.Open(password.c_str());
 			unsigned int id = desktop.GetDBID(dbNames[0]);
-			auto_ptr<Parser> parse = GetParser(dbNames[0],filename,null_parser);
+			shared_ptr<Parser> parse = GetParser(dbNames[0],filename,
+				null_parser, true, vformat_mode, bbackup_mode);
 
 			for( unsigned int i = 0; i < stCommands.size(); i++ ) {
 				desktop.GetRecord(id, stCommands[i].index, *parse.get());
@@ -814,56 +955,54 @@ int main(int argc, char *argv[])
 			return 0;
 		}
 
-		// Clear databases
-		if (clear_database) {
-			if( clearDbNames.size() == 0 ) {
-				cout << "No db names to clear" << endl;
-				return 1;
-			}
-
-			vector<string>::iterator b = clearDbNames.begin();
-
-			desktop.Open(password.c_str());
-
-			for( ; b != clearDbNames.end(); b++ ) {
-				unsigned int id = desktop.GetDBID(*b);
-				cout << "Clearing all recordss from " << (*b) << "..." << endl;
-				desktop.ClearDatabase(id);
-			}
-
-			return 0;
-		}
-
 		// Dump contents of selected databases to stdout, or
 		// to file if specified.
 		// This is retrieving data from the Blackberry.
 		if( dbNames.size() ) {
 			vector<string>::iterator b = dbNames.begin();
 
-			desktop.Open(password.c_str());
 			for( ; b != dbNames.end(); b++ ) {
-				auto_ptr<Parser> parse = GetParser(*b,filename,null_parser);
+				shared_ptr<Parser> parse = GetParser(*b,
+					filename, null_parser, !sort_records,
+					vformat_mode, bbackup_mode);
 				unsigned int id = desktop.GetDBID(*b);
 				desktop.LoadDatabase(id, *parse.get());
 			}
 		}
 
+		// Clear databases
+		if( clear_database ) {
+			if( clearDbNames.size() == 0 ) {
+				cout << "No db names to erase" << endl;
+				return 1;
+			}
+
+			vector<string>::iterator b = clearDbNames.begin();
+
+			for( ; b != clearDbNames.end(); b++ ) {
+				unsigned int id = desktop.GetDBID(*b);
+				cout << "Deleting all records from " << (*b) << "..." << endl;
+				desktop.ClearDatabase(id);
+			}
+
+			return 0;
+		}
+
 		// Save contents of file to specified databases
 		// This is writing data to the Blackberry.
 		if( saveDbNames.size() ) {
 			vector<string>::iterator b = saveDbNames.begin();
 
-			desktop.Open(password.c_str());
 			for( ; b != saveDbNames.end(); b++ ) {
-				auto_ptr<Builder> build =
-					GetBuilder(*b, filename);
+				shared_ptr<Builder> build = GetBuilder(*b,
+					filename);
 				unsigned int id = desktop.GetDBID(*b);
 				desktop.SaveDatabase(id, *build);
 			}
 		}
 
 	}
-	catch( Usb::Error &ue) {
+	catch( Usb::Error &ue ) {
 		std::cerr << "Usb::Error caught: " << ue.what() << endl;
 		return 1;
 	}
