@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2005-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
+#include "i18n.h"
 
 using namespace std;
 
@@ -60,6 +61,8 @@ void PrintHex(const char *str)
 
 int main()
 {
+	INIT_I18N(PACKAGE);
+
 	cout.sync_with_stdio(false);
 
 	while( cin ) {
