@@ -5,7 +5,7 @@
 ///
 
 /*
-    Copyright (C) 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,13 +27,15 @@
 #include <vector>
 #include <string>
 #include <memory>
-#include <getopt.h>
+#include <stdlib.h>
 #include <sys/select.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <signal.h>
+#include "i18n.h"
 
+#include "barrygetopt.h"
 
 using namespace std;
 using namespace Barry;
@@ -43,12 +45,12 @@ volatile bool signal_end = false;
 
 void Usage()
 {
-   int major, minor;
-   const char *Version = Barry::Version(major, minor);
+   int logical, major, minor;
+   const char *Version = Barry::Version(logical, major, minor);
 
    cerr
    << "pppob - PPP over Barry\n"
-   << "        Copyright 2007-2009, Net Direct Inc. (http://www.netdirect.ca/)\n"
+   << "        Copyright 2007-2012, Net Direct Inc. (http://www.netdirect.ca/)\n"
    << "        Using: " << Version << "\n"
    << "\n"
    << "   -l file   Direct pppob log output to file (useful with -v)\n"
@@ -127,6 +129,8 @@ void ProcessStdin(Modem &modem)
 
 int main(int argc, char *argv[])
 {
+	INIT_I18N(PACKAGE);
+
 	cout.sync_with_stdio(true);	// leave this on, since libusb uses
 					// stdio for debug messages
 
@@ -137,6 +141,18 @@ int main(int argc, char *argv[])
 		std::string logfile;
 		std::string password;
 
+		// check for options via the fifo first, so the command
+		// line args can override them
+		FifoClient fifo;
+		if( fifo.Fetch(4) ) {
+			const FifoArgs &args = fifo.GetArgs();
+			pin = args.m_pin.Value();
+			force_serial = args.m_use_serial_mode;
+			logfile = args.m_log_filename;
+			password = args.m_password;
+			data_dump = args.m_verbose;
+		}
+
 		// process command line options
 		for(;;) {
 			int cmd = getopt(argc, argv, "l:p:P:sv");
@@ -184,6 +200,13 @@ int main(int argc, char *argv[])
 			Barry::Init(data_dump, &std::cerr);
 		}
 
+		// Display version if in data_dump mode
+		if( data_dump ) {
+			int logical, major, minor;
+			const char *Version = Barry::Version(logical, major, minor);
+			barryverbose(Version);
+		}
+
 		// Probe the USB bus for Blackberry devices and display.
 		// If user has specified a PIN, search for it in the
 		// available device list here as well
