@@ -4,7 +4,7 @@
 ///
 
 /*
-    Copyright (C) 2006-2009, Net Direct Inc. (http://www.netdirect.ca/)
+    Copyright (C) 2006-2012, Net Direct Inc. (http://www.netdirect.ca/)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,9 +24,11 @@
 #include <iostream>
 #include <fstream>
 #include <vector>
+#include <stdlib.h>
 #include <string>
-#include <getopt.h>
+#include "i18n.h"
 
+#include "barrygetopt.h"
 
 using namespace std;
 using namespace Barry;
@@ -35,9 +37,10 @@ void Usage()
 {
    cerr
    << "upldif - Command line LDIF uploader\n"
-   << "         Copyright 2006-2009, Net Direct Inc. (http://www.netdirect.ca/)\n\n"
+   << "         Copyright 2006-2012, Net Direct Inc. (http://www.netdirect.ca/)\n\n"
    << "   -p pin    PIN of device to talk with\n"
    << "             If only one device plugged in, this flag is optional\n"
+   << "   -P pass   Simplistic method to specify device password\n"
    << "   -u        Do the upload.  If not specified, only dumps parsed\n"
    << "             LDIF data to stdout.\n"
    << "   -v        Dump protocol data during operation\n"
@@ -78,7 +81,7 @@ struct Store
 
 	// Retrieval operator -- called by Barry during the upload
 	// process to get the next object
-	bool operator()(Record &rec, unsigned int databaseId) const
+	bool operator()(Record &rec, Builder &builder) const
 	{
 		if( rec_it == records.end() )
 			return false;
@@ -106,6 +109,8 @@ std::ostream& operator<< (std::ostream &os, const Store<Record> &store)
 
 int main(int argc, char *argv[])
 {
+	INIT_I18N(PACKAGE);
+
 	cout.sync_with_stdio(true);	// leave this on, since libusb uses
 					// stdio for debug messages
 
@@ -114,10 +119,11 @@ int main(int argc, char *argv[])
 		uint32_t pin = 0;
 		bool	data_dump = false,
 			do_upload = false;
+		string password;
 
 		// process command line options
 		for(;;) {
-			int cmd = getopt(argc, argv, "hp:uv");
+			int cmd = getopt(argc, argv, "hp:P:uv");
 			if( cmd == -1 )
 				break;
 
@@ -127,6 +133,10 @@ int main(int argc, char *argv[])
 				pin = strtoul(optarg, NULL, 16);
 				break;
 
+			case 'P':	// Device password
+				password = optarg;
+				break;
+
 			case 'u':	// do upload
 				do_upload = true;
 				break;
@@ -169,7 +179,7 @@ int main(int argc, char *argv[])
 
 		// make sure we're in desktop mode
 		Barry::Mode::Desktop desktop(con);
-		desktop.Open();
+		desktop.Open(password.c_str());
 
 		// upload all records to device
 		desktop.SaveDatabaseByType<Barry::Contact>(contactStore);
